/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/CustDetailNotification.java-arc   1.12   11 Apr 2005 11:31:44   MinhN  $
 * $Revision:   1.12  $
 * $Date:   11 Apr 2005 11:31:44  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 *                     Modification History
 *                     ====================
 *  Date Changed      Changed By     Comments
 *  ------------      ----------     ------------------------------------------
 *  29 May 2007       W.Mills        d3775 Store original date strings for redisplay
 *  21-Aug-2008       Bob Jones      Defect 4985 - increase each telephone number from one to three.
 *  14-Aug-2012       Bob Jones      7984 Java upgrade from 6 to 7.
 *  09-Jun-2017       S.Premkumar    SMP - pkg7b, adding emailAddress to the contact name
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.ConcessionCardRebateCode;
import com.capgemini.nemmco.b2bb.enums.MovementType;
import com.capgemini.nemmco.b2bb.enums.SensitiveLoad;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import com.capgemini.nemmco.b2bb.validators.ContactName;
import com.capgemini.nemmco.b2bb.validators.PersonName;
import com.capgemini.nemmco.b2bb.validators.PostalAddress;
import com.capgemini.nemmco.b2bb.validators.Telephone;

import java.text.ParseException;

import java.util.List;

/**
 * The class <code>CustDetailNotification</code> represents 
 * a CustomerDetailNotification Transaction.
 */
public class CustDetailNotification extends Transaction {

  public static final String BUSINESS_NAME_FIELD                  = "businessName";
  public static final String CONCESSION_CARD_FROM_DATE_FIELD      = "concessionCardFromDate";
  public static final String CONCESSION_CARD_NUMBER_FIELD         = "concessionCardNumber";
  public static final String CONCESSION_CARD_REBATE_CODE_FIELD    = "concessionCardRebateCode";
  public static final String CONCESSION_CARD_TO_DATE_FIELD        = "concessionCardToDate";
  public static final String CUSTOMER_NAME_FIELD                  = "customerName";
  public static final String ADDRESS_FIELD                        = "address";
  public static final String BUSINESS_CONTACT_NAME_FIELD          = "businessContactName";
  public static final String LAST_MODIFIED_DATE_TIME_FIELD        = "lastModifiedDateTime";
  public static final String MOVEMENT_TYPE_FIELD                  = "movementType";
  public static final String SENSITIVE_LOAD_FIELD                 = "sensitiveLoad";
  public static final String PHONE1_FIELD                         = "phone1";
  public static final String PHONE2_FIELD                         = "phone2";
  public static final String PHONE3_FIELD                         = "phone3";
  public static final String EMAIL_ADDRESS                        =  "emailAddress";

  protected PostalAddress address;
  protected ContactName businessContactName;
  protected PersonName customerName;
  protected Telephone phone1;
  protected Telephone phone2;
  protected Telephone phone3;
  
  protected String businessName               = "";
  protected String concessionCardFromDate     = "";
  protected String concessionCardFromDateDisplay = "";    //d3775 WM store original string for redisplay
  protected String concessionCardNumber       = "";
  protected String concessionCardRebateCode   = "";
  protected String concessionCardToDate       = "";  
  protected String concessionCardToDateDisplay = "";      //d3775 WM store original string for redisplay
  protected String deliveryPointIdentifier    = "";
  protected String lastModifiedDate           = "";
  protected String lastModifiedDateTime       = "";
  protected String lastModifiedTime           = "";
  protected String movementType               = "";
  protected String sensitiveLoad              = "";
  protected String emailAddress ="";

  /**
   * Default constructor
   */
  public CustDetailNotification() {
    super(TransactionType.CUST_DETAIL_NOT);
  }

  /**
   * Getter for address
   * @return PostalAddress
   */
  public PostalAddress getAddress() {
    return address;
  }

  /**
   * Getter for business contact name
   * @return PersonName
   */
  public PersonName getBusinessContactName() {
    return businessContactName;
  }

  /**
   * Getter for business name
   * @return String
   */
  public String getBusinessName() {
    return businessName;
  }

  /**
   * Getter for concession card from date
   * @return String
   */
  public String getConcessionCardFromDate() {
    String dateString = concessionCardFromDate;
    //d3775 WM Get original string if validated string is null
    if (dateString == null || dateString.length() == 0)
      dateString = concessionCardFromDateDisplay;
    else if (getDateFormat() == B2BDateFormatter.B2BDATE)
      dateString = B2BDateFormatter.convertDateFormat(concessionCardFromDate, B2BDateFormatter.XMLDATE, B2BDateFormatter.B2BDATE);
    return dateString;
  }

  /**
   * Getter for concession card number
   * @return String
   */
  public String getConcessionCardNumber() {
    return concessionCardNumber;
  }

  /**
   * Getter for concession card rebate code
   * @return String
   */
  public String getConcessionCardRebateCode() {
    return concessionCardRebateCode;
  }

  /**
   * Getter for rebate codes
   * @return List
   */
  public List<? extends Object> getConcessionCardRebateCodes() {
    return ConcessionCardRebateCode.REBATE_CODES;
  }

  /**
   * Getter for concession card to date
   * @return String
   */
  public String getConcessionCardToDate() {
    String dateString = concessionCardToDate;
    //d3775 WM Get original string if validated string is null
    if (dateString == null || dateString.length() == 0)
      dateString = concessionCardToDateDisplay;
    else if (getDateFormat() == B2BDateFormatter.B2BDATE)
      dateString = B2BDateFormatter.convertDateFormat(concessionCardToDate, B2BDateFormatter.XMLDATE, B2BDateFormatter.B2BDATE);
    return dateString;    
  }

  /**
   * Getter for customer name
   * @return PersonName
   */
  public PersonName getCustomerName() {
    return customerName;
  }

  /**
   * Getter for last modified date
   * @return String
   */
  public String getLastModifiedDate() {
    return lastModifiedDate;
  }

  /**
   * Getter for last modified date time
   * @return String
   */
  public String getLastModifiedDateTime() {
    String dateString;
    B2BDateTimeFormatter dateTimeFormatter;
    
    try
    {
      dateTimeFormatter = new B2BDateTimeFormatter(getLastModifiedDate() + " " + getLastModifiedTime(), B2BDateTimeFormatter.B2BDATE);
      dateString = dateTimeFormatter.getXMLFormattedDateTime();
    }
    catch (ParseException e)
    {
      dateString = "";
    }
    return dateString;
  }

  /**
   * Getter for last modified time
   * @return String
   */
  public String getLastModifiedTime() {
    return lastModifiedTime;
  }

  /**
   * Getter for movement type
   * @return String
   */
  public String getMovementType() {
    return movementType;
  }

  /**
   * Getter for movement types
   * @return List
   */
  public List<? extends Object> getMovementTypes() {
    return MovementType.MOVEMENT_TYPES;
  }

  /**
   * Getter for phone numbers
   * @return Telephone
   */
  public Telephone getPhone1() {
    return phone1;
  }
  
  public Telephone getPhone2() {
    return phone2;
  }
  
  public Telephone getPhone3() {
    return phone3;
  }

  /**
   * Getter for sensitive load
   * @return String
   */
  public String getSensitiveLoad() {
    return sensitiveLoad;
  }

  /**
   * Getter for sensitive loads
   * @return String
   */
  public List<? extends Object> getSensitiveLoads() {
    return SensitiveLoad.SENSITIVE_LOADS;
  }

  /**
   * Setter for address
   * @param address 
   */
  public void setAddress(PostalAddress address) {
    this.address = address;
  }

  /**
   * Setter for business contact name 
   * @param businessContactName 
   */
  public void setBusinessContactName(PersonName businessContactName) {
    this.businessContactName = new ContactName(businessContactName);
  }

  /**
   * Setter for business name
   * @param businessName 
   */
  public void setBusinessName(String businessName) {
    this.businessName = businessName;
  }

  /**
   * Setter for concession card from date
   * @param concessionCardFromDate 
   */
  public void setConcessionCardFromDate(String concessionCardFromDate) {
    //d3775 WM Save original string for redisplay
    this.concessionCardFromDateDisplay = concessionCardFromDate;
    this.concessionCardFromDate = B2BDateFormatter.convertDateFormat(concessionCardFromDate, getDateFormat(), B2BDateFormatter.XMLDATE);
  }

  /**
   * Setter for concession card number
   * @param concessionCardNumber 
   */
  public void setConcessionCardNumber(String concessionCardNumber) {
    this.concessionCardNumber = concessionCardNumber;
  }

  /**
   * Setter for concession card rebate code
   * @param concessionCardRebatecode 
   */
  public void setConcessionCardRebateCode(String concessionCardRebatecode) {
    this.concessionCardRebateCode = concessionCardRebatecode;
  }

  /**
   * Setter for concession card to date
   * @param concessionCardToDate 
   */
  public void setConcessionCardToDate(String concessionCardToDate) {
    //d3775 WM Save original string for redisplay
    this.concessionCardToDateDisplay = concessionCardToDate;
    this.concessionCardToDate = B2BDateFormatter.convertDateFormat(concessionCardToDate, getDateFormat(), B2BDateFormatter.XMLDATE);
  }

  /**
   * Setter for customer name
   * @param customerName 
   */
  public void setCustomerName(PersonName customerName) {
    this.customerName = customerName;
  }

  /**
   * Setter for last modified date
   * @param lastModifiedDate 
   */
  public void setLastModifiedDate(String lastModifiedDate) {
    this.lastModifiedDate = lastModifiedDate;
  }

  /**
   * Setter for last modified date and time
   * @param lastModifiedDateTime 
   */
  public void setLastModifiedDateTime(String lastModifiedDateTime) {
    B2BDateTimeFormatter dateTimeFormatter;
    this.lastModifiedDateTime = lastModifiedDateTime;
    try
    {
      dateTimeFormatter = new B2BDateTimeFormatter(lastModifiedDateTime, B2BDateTimeFormatter.XMLDATE);
      setLastModifiedDate(dateTimeFormatter.getB2BDate());
      setLastModifiedTime(dateTimeFormatter.getB2BTime());
    }
    catch (ParseException e)
    {
      setLastModifiedDate("");
      setLastModifiedTime("");
    }
  }

  /**
   * Setter for last modified time
   * @param lastModifiedTime 
   */
  public void setLastModifiedTime(String lastModifiedTime) {
    this.lastModifiedTime = lastModifiedTime;
  }

  /**
   * Setter for movement types
   * @param movementType 
   */
  public void setMovementType(String movementType) {
    this.movementType = movementType;
  }

  /**
   * Setter for phone numbers
   * @param phone1, 2 or 3 
   */
  public void setPhone1(Telephone phone1) {
    this.phone1 = phone1;
  }

  public void setPhone2(Telephone phone2) {
    this.phone2 = phone2;
  }
  
  public void setPhone3(Telephone phone3) {
    this.phone3 = phone3;
  }
  
  /**
   * Setter for sensitive load
   * @param sensitiveLoad 
   */
  public void setSensitiveLoad(String sensitiveLoad) {
    this.sensitiveLoad = sensitiveLoad;
  }

   public void setEmailAddress(String emailAddress) {
      this.emailAddress = emailAddress;
   }

   public String getEmailAddress() {
      return emailAddress;
   }
}

