/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/Header.java-arc   1.13   Apr 01 2005 10:33:50   adij  $
 * $Revision:   1.13  $
 * $Date:   Apr 01 2005 10:33:50  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 * 03-Oct-2012    W.Mills           8081 - Java 7 upgrade
 * 22-Sep-2017    Harpreet Kaur     Package 24 : Modify method getTos() to display participants depending upon sendToAnyParticipant value
 * 04-Oct-2017    Harpreet Kaur     Package 24 : Fix for Defect 8020
 *
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.Market;
import com.capgemini.nemmco.b2bb.enums.Priority;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.lookup.Participant;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;

import java.util.List;

/**
 * Defines a header element.
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.13  $ $Date:   Apr 01 2005 10:33:50  $
 */
public class Header extends XMLObject {
  // define the unique name given to each field in this transaction type
  public static final String FROM = "from";
  public static final String MARKET = "market";
  public static final String MESSAGE_DATE = "messageDate";
  public static final String MESSAGE_ID = "messageID";
  public static final String PRIORITY = "priority";
  public static final String TO = "to";
  public static final String TRANSACTION_GROUP = "transactionGroup";

  private String from = "";
  private String market = Market.NEM.getCode();
  private String messageDate = "";
  private String messageDisplayDate = "";
  private boolean messageDateSet;
  private String messageID = "";
  private String priority = "";
  private String to = "";
  private String transactionGroup = "";

  /* Added by Ram Alag for B2B New Transaction */
  private String transactionType = "";

  private String sendToAnyParticipant = " ";

  /**
   * Default Constructor
   */
  public Header() {
    super(XMLObjectType.HEADER);
  }

  /**
   * @return Returns the from.
   */
  public String getFrom() {
    return from;
  }

  /**
   * @return Returns the market.
   */
  public String getMarket() {
    return market;
  }

  /**
   * @return Returns the messageDate.
   */
  public String getMessageDate() {
    return messageDate;
  }

  /**
   * @return Returns the messageDisplayDate.
   */
  public String getMessageDisplayDate() {
    return messageDisplayDate;
  }

  /**
   * @return Returns the messageID.
   */
  public String getMessageID() {
    return messageID;
  }

  /**
   * @return Returns the priority list
   */
  public List<? extends Object> getPriorities() {
    return Priority.PRIORITIES;
  }

  /**
   * @return Returns the priority.
   */
  public String getPriority() {
    return priority;
  }

  /**
   * @return Returns the to.
   */
  public String getTo() {
    return to;
  }

  /**
   * @return Returns the participant list
   */
  public List<Participant> getTos() {
    List<Participant> toList = null;

   try {
       if(sendToAnyParticipant.equals("true")){
          toList = Participant.getAllParticipants();
       }
       else{
          toList = Participant.getList(transactionType,"R");
       }

    } catch (Exception e) {
      toList = null;
    }

    return toList;
  }//end of getTos()

  /**
   * @return Returns the transactionGroup.
   */
  public String getTransactionGroup() {
    return transactionGroup;
  }

  /**
   * @return Returns the messageDateSet.
   */
  public boolean isMessageDateSet() {
    return messageDateSet;
  }

  /**
   * @param from The from to set.
   */
  public void setFrom(String from) {
    this.from = from;
  }

  /**
   * @param market The market to set.
   */
  public void setMarket(String market) {
    this.market = market;
  }

  /**
   * @param messageDate The messageDate to set.
   */
  public void setMessageDate(String messageDate) {
    this.messageDateSet = true;
    this.messageDate = messageDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(messageDate, B2BDateTimeFormatter.XMLDATE);
      this.messageDisplayDate = df.getB2BFormattedDateTime();
    } catch (Exception e) {
      this.messageDisplayDate = "";
    }
  }

  /**
   * @param messageDisplayDate The messageDisplayDate to set.
   */
  public void setMessageDisplayDate(String messageDisplayDate) {
    this.messageDateSet = false;
    this.messageDisplayDate = messageDisplayDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(messageDisplayDate, B2BDateTimeFormatter.B2BDATE);
      this.messageDate = df.getXMLFormattedDateTime();
    } catch (Exception e) {
      this.messageDate = "";
    }
  }

  /**
   * @param messageDateSet The messageDateSet to set.
   */
  public void setMessageDateSet(boolean messageDateSet) {
    this.messageDateSet = messageDateSet;
  }

  /**
   * @param messageID The messageID to set.
   */
  public void setMessageID(String messageID) {
    this.messageID = messageID;
  }

  /**
   * @param priority The priority to set.
   */
  public void setPriority(String priority) {
    this.priority = priority;
  }

  /**
   * @param to The to to set.
   */
  public void setTo(String to) {
    this.to = to;
  }

  /**
   * @param transactionGroup The transactionGroup to set.
   */
  public void setTransactionGroup(String transactionGroup) {
    this.transactionGroup = transactionGroup;
  }

   public String getTransactionType() {
      return transactionType;
   }

   public void setTransactionType(String transactionType) {
      this.transactionType = transactionType;
   }

   public void setSendToAnyParticipant(String sendToAnyParticipant) {
      this.sendToAnyParticipant = sendToAnyParticipant;
   }

   public String getSendToAnyParticipant() {
      return this.sendToAnyParticipant;
   }
}
