/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/MeterDataVerifyRequest.java-arc   1.13   18 May 2005 15:04:22   RayanS  $
 * $Revision:   1.13  $
 * $Date:   18 May 2005 15:04:22  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 * ====================
 *  Date Changed      Changed By     Comments
 *  ------------      ----------     ------------------------------------------
 *  29 May 2007       W.Mills        d3775 Store original date strings for redisplay
 *  14-Aug-2012       Bob Jones      7984 Java upgrade from 6 to 7.  
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.InvestigationCode;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.lookup.B2BParticipantRole;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

import java.util.List;

/**
 * The class <code>MeterDataVerifyRequest</code> represents a MeterDataVerifyRequest
 * Transaction.
 */
public class MeterDataVerifyRequest extends Transaction {

  public static final String CURRENT_CONSUMPTION_FIELD        = "currentConsumption";
  public static final String CURRENT_READ_DATE_FIELD          = "currentReadDate";
  public static final String CURRENT_READ_FIELD               = "currentRead";
  public static final String END_READ_DATE_FIELD              = "endReadDate";
  public static final String INVESTIGATION_CODE_FIELD         = "investigationCode";
  public static final String INVESTIGATION_DESCRIPTION_FIELD1 = "investigationDescription1";
  public static final String INVESTIGATION_DESCRIPTION_FIELD2 = "investigationDescription2";
  public static final String INVESTIGATION_DESCRIPTION_FIELD3 = "investigationDescription3";
  public static final String METER_SERIAL_FIELD               = "meterSerial";
  public static final String NMI_CONFIGURATION_FIELD          = "nmiConfiguration";
  public static final String NMI_SUFFIX_FIELD                 = "nmiSuffix";
  public static final String REGISTER_ID_FIELD                = "registerId";
  public static final String ROLE_FIELD                       = "role";
  public static final String START_READ_DATE_FIELD            = "startReadDate";
  public static final String SUFFIX_FIELD                     = "suffix";
  
  protected String currentConsumption         = "";
  protected String currentRead                = "";
  protected String currentReadDate            = "";
  protected String currentReadDateDisplay     = "";    //d3775 WM store original string for redisplay
  protected String endReadDate                = "";
  protected String endReadDateDisplay         = "";    //d3775 WM store original string for redisplay
  protected String investigationCode          = "";
  protected String investigationDescription1  = "";
  protected String investigationDescription2  = "";
  protected String investigationDescription3  = "";
  protected String meterSerial                = "";
  protected String nmiConfiguration           = "";
  protected String registerId                 = "";
  protected String role                       = "";
  protected String startReadDate              = "";
  protected String startReadDateDisplay       = "";    //d3775 WM store original string for redisplay
  protected String nmiSuffix1                 = "";
  protected String nmiSuffix2                 = "";
  protected String nmiSuffix3                 = "";
  protected String nmiSuffix4                 = "";
  protected String nmiSuffix5                 = "";
  protected String nmiSuffix6                 = "";
  protected String nmiSuffix7                 = "";
  protected String nmiSuffix8                 = "";
  protected String nmiSuffix9                 = "";
  protected String nmiSuffix10                = "";
  protected String suffix                     = "";

  /**
   * Default constructor
   */
  public MeterDataVerifyRequest() {
    super(TransactionType.METER_DATA_VERIFY_REQ);
  }

  /**
   * Getter for current consumption
   * @return String
   */
  public String getCurrentConsumption() {
    return currentConsumption;
  }

  /**
   * Getter for current read
   * @return String
   */
  public String getCurrentRead() {
    return currentRead;
  }

  /**
   * Getter for current read date
   * @return String
   */
  public String getCurrentReadDate() {
    String dateString = currentReadDate;
    //d3775 WM Get original string if validated string is null
    if (dateString == null || dateString.length() == 0)
      dateString = currentReadDateDisplay;
    else if (getDateFormat() == B2BDateFormatter.B2BDATE)
      dateString = B2BDateFormatter.convertDateFormat(currentReadDate, B2BDateFormatter.XMLDATE, B2BDateFormatter.B2BDATE);
    return dateString;
  }

  /**
   * Getter for start read date
   * @return String
   */
  public String getStartReadDate() {
    String dateString = startReadDate;
    //d3775 WM Get original string if validated string is null
    if (dateString == null || dateString.length() == 0)
      dateString = startReadDateDisplay;
    else if (getDateFormat() == B2BDateFormatter.B2BDATE)
      dateString = B2BDateFormatter.convertDateFormat(startReadDate, B2BDateFormatter.XMLDATE, B2BDateFormatter.B2BDATE);
    return dateString;
  }

  /**
   * Getter for end read date
   * @return String
   */
  public String getEndReadDate() {
    String dateString = endReadDate;
    //d3775 WM Get original string if validated string is null
    if (dateString == null || dateString.length() == 0)
      dateString = endReadDateDisplay;
    else if (getDateFormat() == B2BDateFormatter.B2BDATE)
      dateString = B2BDateFormatter.convertDateFormat(endReadDate, B2BDateFormatter.XMLDATE, B2BDateFormatter.B2BDATE);
    return dateString;   
  }

  /**
   * Getter for investigation code
   * @return String
   */
  public String getInvestigationCode() {
    return investigationCode;
  }

  /**
   * Getter for investigation codes
   * @return String
   */
  public List<? extends Object> getInvestigationCodes() {
    return InvestigationCode.INVESTIGATION_CODES;
  }

  /**
   * Getter for investigation description
   * @return String
   */
  public String getInvestigationDescription1() {
    return investigationDescription1;
  }

  /**
   * Getter for investigation description
   * @return String
   */
  public String getInvestigationDescription2() {
    return investigationDescription2;
  }

  /**
   * Getter for investigation description
   * @return String
   */
  public String getInvestigationDescription3() {
    return investigationDescription3;
  }
  
  /**
   * Getter for meter serial
   * @return String
   */
  public String getMeterSerial() {
    return meterSerial;
  }

  /**
   * Getter for nmi configuration
   * @return String
   */
  public String getNmiConfiguration() {
    return nmiConfiguration;
  }

  /**
   * Getter for nmi suffix 1
   * @return String
   */
  public String getNmiSuffix1() {
    return nmiSuffix1;
  }

  /**
   * Getter for nmi suffix 2
   * @return String
   */
  public String getNmiSuffix2() {
    return nmiSuffix2;
  }

  /**
   * Getter for nmi suffix 3
   * @return String
   */  
  public String getNmiSuffix3() {
    return nmiSuffix3;
  }

  /**
   * Getter for nmi suffix 4
   * @return String
   */
  public String getNmiSuffix4() {
    return nmiSuffix4;
  }

  /**
   * Getter for nmi suffix 5
   * @return String
   */
  public String getNmiSuffix5() {
    return nmiSuffix5;
  }

  /**
   * Getter for nmi suffix 6
   * @return String
   */
  public String getNmiSuffix6() {
    return nmiSuffix6;
  }

  /**
   * Getter for nmi suffix 7
   * @return String
   */
  public String getNmiSuffix7() {
    return nmiSuffix7;
  }

  /**
   * Getter for nmi suffix 8
   * @return String
   */  
  public String getNmiSuffix8() {
    return nmiSuffix8;
  }

  /**
   * Getter for nmi suffix 9
   * @return String
   */
  public String getNmiSuffix9() {
    return nmiSuffix9;
  }

  /**
   * Getter for nmi suffix 10
   * @return String
   */
  public String getNmiSuffix10() {
    return nmiSuffix10;
  }

  /**
   * Getter for participant roles
   * @return String
   */
  public List<Object> getParticipantRoles() throws UnableToLookupResourceException {
    // Defect 3353 (Bryan Kim) : Need to get a subset of participant roles
  	return B2BParticipantRole.getList();
  }

  /**
   * Getter for register id
   * @return String
   */
  public String getRegisterId() {
    return registerId;
  }

  /**
   * Getter for participant role
   * @return String
   */
  public String getRole() {
    return role;
  }

  /**
   * Setter for current consumption
   * @param currentConsumption 
   */
  public void setCurrentConsumption(String currentConsumption) {
    this.currentConsumption = currentConsumption;
  }

  /**
   * Setter for current read
   * @param currentRead 
   */
  public void setCurrentRead(String currentRead) {
    this.currentRead = currentRead;
  }

  /**
   * Setter for current read date
   * @param currentReadDate 
   */
  public void setCurrentReadDate(String currentReadDate) {    
    //d3775 WM Save original string for redisplay
    this.currentReadDateDisplay = currentReadDate;
    this.currentReadDate = B2BDateFormatter.convertDateFormat(currentReadDate, getDateFormat(), B2BDateFormatter.XMLDATE);        
  }
  
  /**
   * Setter for start read date
   * @param startReadDate 
   */
  public void setStartReadDate(String startReadDate) {
    //d3775 WM Save original string for redisplay
    this.startReadDateDisplay = startReadDate;
    this.startReadDate = B2BDateFormatter.convertDateFormat(startReadDate, getDateFormat(), B2BDateFormatter.XMLDATE);        
  }

  /**
   * Setter for end read date
   * @param endReadDate 
   */
  public void setEndReadDate(String endReadDate) {
    //d3775 WM Save original string for redisplay
    this.endReadDateDisplay = endReadDate;
    this.endReadDate = B2BDateFormatter.convertDateFormat(endReadDate, getDateFormat(), B2BDateFormatter.XMLDATE);          
  }

  /**
   * Setter for investigation code
   * @param investigationCode 
   */
  public void setInvestigationCode(String investigationCode) {
    this.investigationCode = investigationCode;
  }

  /**
   * Setter for investigation description
   * @param investigationDescription 
   */
  public void setInvestigationDescription1(String investigationDescription1) {
    this.investigationDescription1 = investigationDescription1;
  }
  /**
   * Setter for investigation description
   * @param investigationDescription 
   */
  public void setInvestigationDescription2(String investigationDescription2) {
    this.investigationDescription2 = investigationDescription2;
  }
  /**
   * Setter for investigation description
   * @param investigationDescription 
   */
  public void setInvestigationDescription3(String investigationDescription3) {
    this.investigationDescription3 = investigationDescription3;
  }
  /**
   * Setter for meter serial
   * @param meterSerial 
   */
  public void setMeterSerial(String meterSerial) {
    this.meterSerial = meterSerial;
  }

  /**
   * Setter for nmi configuration
   * @param nmiConfiguration 
   */
  public void setNmiConfiguration(String nmiConfiguration) {
    this.nmiConfiguration = nmiConfiguration;
  }

  /**
   * Setter for nmi suffix 1
   * @param nmiSuffix 
   */
  public void setNmiSuffix1(String nmiSuffix) {
    this.nmiSuffix1 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 2
   * @param nmiSuffix 
   */
  public void setNmiSuffix2(String nmiSuffix) {
    this.nmiSuffix2 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 3
   * @param nmiSuffix 
   */  
  public void setNmiSuffix3(String nmiSuffix) {
    this.nmiSuffix3 = nmiSuffix;
  }  

  /**
   * Setter for nmi suffix 4
   * @param nmiSuffix 
   */
  public void setNmiSuffix4(String nmiSuffix) {
    this.nmiSuffix4 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 5
   * @param nmiSuffix 
   */
  public void setNmiSuffix5(String nmiSuffix) {
    this.nmiSuffix5 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 6
   * @param nmiSuffix 
   */
  public void setNmiSuffix6(String nmiSuffix) {
    this.nmiSuffix6 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 7
   * @param nmiSuffix 
   */
  public void setNmiSuffix7(String nmiSuffix) {
    this.nmiSuffix7 = nmiSuffix;
  }

  /**
   * Setter for nmi suffix 8
   * @param nmiSuffix 
   */  
  public void setNmiSuffix8(String nmiSuffix) {
    this.nmiSuffix8 = nmiSuffix;
  }  

  /**
   * Setter for nmi suffix 9
   * @param nmiSuffix 
   */
  public void setNmiSuffix9(String nmiSuffix) {
    this.nmiSuffix9 = nmiSuffix;
  }
  
  /**
   * Setter for nmi suffix 10
   * @param nmiSuffix 
   */  
  public void setNmiSuffix10(String nmiSuffix) {
    this.nmiSuffix10 = nmiSuffix;
  }  

  /**
   * Setter for register id
   * @param registerId 
   */
  public void setRegisterId(String registerId) {
    this.registerId = registerId;
  }

  /**
   * Setter for participant role
   * @param role 
   */
  public void setRole(String role) {
    this.role = role;
  }

  /**
   * @return Returns the suffix.
   */
  public String getSuffix() {
    return suffix;
  }
  
  /**
   * @param suffix The suffix to set.
   */
  public void setSuffix(String suffix) {
    this.suffix = suffix;
  }
}

