/*
 * MeterFaultAndIssueNotification.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 10-Jul-2017    Harpreet Kaur     Created
 * 31-Aug-2017    Harpreet Kaur     7135 Package 7B: Defect#7844 fix
 */

 package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.MeterFaultAndIssueReasons;
import com.capgemini.nemmco.b2bb.enums.MeterFaultAndIssueSupplyOffOptions;
import com.capgemini.nemmco.b2bb.enums.MeterFaultAndIssueSupplyOnOptions;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.B2BTimeFormatter;

import java.util.List;

public class MeterFaultAndIssueNotification extends Transaction {

   public static final String DATE = "date";
   public static final String START_DATE = "startDate";
   public static final String START_TIME = "startTime";
   public static final String END_DATE = "endDate";
   public static final String DURATION = "duration";
   public static final String SUPPLY_ON = "supplyOn";
   public static final String SUPPLY_OFF = "supplyOff";
   public static final String METER_SERIAL_NUMBER1 = "meterSerialNumber1";
   public static final String METER_SERIAL_NUMBER2 = "meterSerialNumber2";
   public static final String METER_SERIAL_NUMBER3 = "meterSerialNumber3";
   public static final String METER_SERIAL_NUMBER4 = "meterSerialNumber4";
   public static final String REASON_FOR_NOTICE = "reasonForNotice";
   public static final String NOTES = "notes";

   private String date = "";
   private boolean dateSet;
   private String displayDate = "";

   private String startDate = "";
   private boolean startDateSet;
   private String startDisplayDate = "";

   private String startTime = "";
   private String startDisplayTime="";
   private boolean isStartTimeSet=false;

   private String endDate = "";
   private boolean endDateSet;
   private String endDisplayDate = "";

   private String duration = "";
   private String supplyOn = "";
   private String supplyOff = "";
   private String meterSerialNumber1 = "";
   private String meterSerialNumber2 = "";
   private String meterSerialNumber3 = "";
   private String meterSerialNumber4 = "";
   private String reasonForNotice = "";
   private String notes = "";


   public MeterFaultAndIssueNotification() {
      super(TransactionType.METER_FAULT_AND_ISSUE_NOT);
   }

   /**
    *
    * @return return the date
    */
   public String getDate() {
      return date;
   }

   /**
    *
    * @return returns the displayDate
    */
   public String getDisplayDate() {
      return displayDate;
   }

   /**
    *
    * @param date The date to set
    */
   public void setDate(String date) {
      this.dateSet = true;
      this.date = date;
      try {
         B2BDateFormatter df = new B2BDateFormatter(date, B2BDateFormatter.XMLDATE);
         this.displayDate = df.getB2BDate();
      } catch (Exception e) {
         this.displayDate = "";
      }
   }

   /**
    *
    * @param displayDate The displayDate to set
    */
   public void setDisplayDate(String displayDate) {

      this.dateSet = false;
      this.displayDate = displayDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(displayDate, B2BDateFormatter.B2BDATE);
         this.date = df.getXMLDate();
      } catch (Exception e) {
         this.date = "";
      }
   }

   /**
    *
    * @return returns true if date is set, false otherwise
    */
   public boolean isDateSet() {
      return dateSet;
   }

   /**
    *
    * @param dateSet The dateSet to set
    */
   public void setDateSet(boolean dateSet) {
      this.dateSet = dateSet;
   }


   /**
    *
    * @return return the startDate
    */
   public String getStartDate() {
      return startDate;
   }

   /**
    *
    * @return returns the startDisplayDate
    */
   public String getStartDisplayDate() {
      return startDisplayDate;
   }

   /**
    *
    * @param startDate The startDate to set
    */
   public void setStartDate(String startDate) {
      this.startDateSet = true;
      this.startDate = startDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(startDate, B2BDateFormatter.XMLDATE);
         this.startDisplayDate = df.getB2BDate();
      } catch (Exception e) {
         this.startDisplayDate = "";
      }
   }

   /**
    *
    * @param startDisplayDate The startDisplayDate to set
    */
   public void setStartDisplayDate(String startDisplayDate) {

      this.startDateSet = false;
      this.startDisplayDate = startDisplayDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(startDisplayDate, B2BDateFormatter.B2BDATE);
         this.startDate = df.getXMLDate();
      } catch (Exception e) {
         this.startDate = "";
      }
   }

   /**
    *
    * @return returns true if startDate is set, false otherwise
    */
   public boolean isStartDateSet() {
      return startDateSet;
   }

   /**
    *
    * @param startDateSet The startDateSet to set
    */
   public void setStartDateSet(boolean startDateSet) {
      this.startDateSet = startDateSet;
   }

   /**
    *
    * @return returns the isStartTimeSet
    */
   public boolean getIsStartTimeSet() {
     return isStartTimeSet;
   }

   /**
    * @return Returns the startTime.
    */
   public String getStartTime() {
     if (startTime == null || startTime.equals("")) {
       try {
         B2BTimeFormatter tf = new B2BTimeFormatter(getStartDisplayTime(), B2BTimeFormatter.B2BTIME);
         this.startTime = tf.getXMLTime();
       } catch (Exception e) {
         this.startTime = "";
       }
     }
     return this.startTime;
   }

   /**
    * @param startTime The startTime to set.
    */
   public void setStartTime(String startTime) {
     this.isStartTimeSet = true;
     this.startTime = startTime;
     try {
       B2BTimeFormatter df = new B2BTimeFormatter(startTime, B2BTimeFormatter.XMLTIME);
       this.startDisplayTime = df.getB2BTime();
     } catch (Exception e) {
       this.startDisplayTime = "";
     }
   }

   /**
    * @return Returns the startDisplayTime.
    */
   public String getStartDisplayTime() {
     return this.startDisplayTime;
   }

   /**
    * @param startDisplayTime The startTime to set.
    */
   public void setStartDisplayTime(String startDisplayTime) {
     this.isStartTimeSet = false;
     this.startDisplayTime = startDisplayTime;
   }

   /**
    *
    * @return returns the endDate
    */
   public String getEndDate() {
      return endDate;
   }

   /**
    *
    * @return returns the endDisplayDate
    */
   public String getEndDisplayDate() {
      return endDisplayDate;
   }

   /**
    *
    * @param endDate
    */
   public void setEndDate(String endDate) {
      this.endDateSet = true;
      this.endDate = endDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(endDate, B2BDateFormatter.XMLDATE);
         this.endDisplayDate = df.getB2BDate();
      } catch (Exception e) {

         this.endDisplayDate = "";
      }
   }

   /**
    *
    * @param endDisplayDate
    */
   public void setEndDisplayDate(String endDisplayDate) {
      this.endDateSet = false;
      this.endDisplayDate = endDisplayDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(endDisplayDate, B2BDateFormatter.B2BDATE);
         this.endDate = df.getXMLDate();
      } catch (Exception e) {
         this.endDate = "";
      }
   }

   /**
    *
    * @return returns true if endDate is set, false otherwise
    */
   public boolean isEndDateSet() {
      return endDateSet;
   }

   /**
    *
    * @param endDateSet The endDateSet to set
    */
   public void setEndDateSet(boolean endDateSet) {
      this.endDateSet = endDateSet;
   }


   public void setDuration(String duration) {
      this.duration = duration;
   }

   public String getDuration() {
      return duration;
   }

   public void setSupplyOn(String supplyOn) {
      this.supplyOn = supplyOn;
   }

   public String getSupplyOn() {
      return supplyOn;
   }

   /**
    * Getter for supplyOnOptions
    * @return List
    */
   public List<? extends Object> getSupplyOnOptions() {
      return MeterFaultAndIssueSupplyOnOptions.METER_FAULT_AND_ISSUE_SUPPLY_ON_OPTIONS;
   }

   public void setSupplyOff(String supplyOff) {
      this.supplyOff = supplyOff;
   }

   public String getSupplyOff() {
      return supplyOff;
   }

   /**
    * Getter for supplyOffOptions
    * @return List
    */
   public List<? extends Object> getSupplyOffOptions() {
      return MeterFaultAndIssueSupplyOffOptions.METER_FAULT_AND_ISSUE_SUPPLY_OFF_OPTIONS;
   }

   public void setMeterSerialNumber1(String meterSerialNumber1) {
      this.meterSerialNumber1 = meterSerialNumber1;
   }

   public String getMeterSerialNumber1() {
      return meterSerialNumber1;
   }

   public void setMeterSerialNumber2(String meterSerialNumber2) {
      this.meterSerialNumber2 = meterSerialNumber2;
   }

   public String getMeterSerialNumber2() {
      return meterSerialNumber2;
   }

   public void setMeterSerialNumber3(String meterSerialNumber3) {
      this.meterSerialNumber3 = meterSerialNumber3;
   }

   public String getMeterSerialNumber3() {
      return meterSerialNumber3;
   }

   public void setMeterSerialNumber4(String meterSerialNumber4) {
      this.meterSerialNumber4 = meterSerialNumber4;
   }

   public String getMeterSerialNumber4() {
      return meterSerialNumber4;
   }

   public void setReasonForNotice(String reasonForNotice) {
      this.reasonForNotice = reasonForNotice;
   }

   public String getReasonForNotice() {
      return reasonForNotice;
   }

   /**
    * Getter for reasonsForNotice
    * @return List
    */
   public List<? extends Object> getReasonsForNotice() {
      return MeterFaultAndIssueReasons.METER_FAULT_AND_ISSUE_REASONS;
   }

   public void setNotes(String notes) {
      this.notes = notes;
   }

   public String getNotes() {
      return notes;
   }

}

