/**
 * OneWayNotification.java
 *
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the 
 * National Electricity Market Management Company Limited.
 *
 * Copyright National Electricity Market Management Company Limited. 2009
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 21-Jan-2009    Eddie Graham      Created.
 * 18-Mar-2011    Eddie Graham      Defect 7087. Add new method getOneWayNotificationNames() 
 *                                  to return list of One Way Notification Names.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */
   
package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.OneWayNotificationName;
import com.capgemini.nemmco.b2bb.enums.TransactionType;

import java.util.List;

public class OneWayNotification extends Transaction {

  public static final String CSV_NOTIFICATION_DETAIL = "CSVNotificationDetail";
  public static final String NAME = "name";

  private String CSVNotificationDetail = "";
  private String name = "";

  /**
   * Default Constructor
   */
  public OneWayNotification() {
    super(TransactionType.ONE_WAY_NOT);
  }

  /**
   * Retrieve CSVNotificationDetail
   * 
   * @return CSVNotificationDetail.
   */
  public String getCSVNotificationDetail() {
    return CSVNotificationDetail;
  }

  /**
   * Retrieve name
   * 
   * @return name.
   */
  public String getName() {
    return name;
  }
  
  /**
   * Retrieve list of One Way Notification Names.
   * 
   * @return list of One Way Notification Names.
  */  
  public List<? extends Object> getOneWayNotificationNames() {
    return OneWayNotificationName.ONE_WAY_NOTIFICATION_NAME_CODES;
  }
  
  /**
   * Set CSVNotificationDetail
   * 
   * @param CSVNotificationDetail.
   */
  public void setCSVNotificationDetail(String CSVNotificationDetail) {
    this.CSVNotificationDetail = CSVNotificationDetail;
  }

  /**
   * Set name
   * 
   * @param name.
   */
  public void setName(String name) {
    this.name = name;
  }
}

