/*
 * PlannedInterruptionNotification.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 30-Jun-2017    Harpreet Kaur     Created
 * 31-Aug-2017    Harpreet Kaur     7135 Package 7B: Defect#7844 fix
 */

 package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.PlannedInterruptionReasons;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.B2BTimeFormatter;

import java.util.List;

public class PlannedInterruptionNotification extends Transaction {
   public static final String START_DATE = "startDate";
   public static final String END_DATE = "endDate";
   public static final String START_TIME = "startTime";
   public static final String DURATION = "duration";
   public static final String PLANNED_INTERRUPTION_REASON = "plannedInterruptionReason";
   public static final String NOTES = "notes";

   private String startTime = "";
   private String startDisplayTime="";
   private boolean isStartTimeSet=false;

   private String duration = "";
   private String plannedInterruptionReason = "";
   private String notes = "";
   private String startDate = "";
   private boolean startDateSet;
   private String startDisplayDate = "";
   private String endDate = "";
   private boolean endDateSet;
   private String endDisplayDate = "";

   /**
    * Default Constructor
    */
   public PlannedInterruptionNotification() {
      super(TransactionType.PLANNED_INTERRUPT_NOT);
   }

   /**
    *
    * @return return the startDate
    */
   public String getStartDate() {
      return startDate;
   }

   /**
    *
    * @return returns the startDisplayDate
    */
   public String getStartDisplayDate() {
      return startDisplayDate;
   }

   /**
    *
    * @param startDate The startDate to set
    */
   public void setStartDate(String startDate) {
      this.startDateSet = true;
      this.startDate = startDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(startDate, B2BDateFormatter.XMLDATE);
         this.startDisplayDate = df.getB2BDate();
      } catch (Exception e) {
         this.startDisplayDate = "";
      }
   }

   /**
    *
    * @param startDisplayDate The startDisplayDate to set
    */
   public void setStartDisplayDate(String startDisplayDate) {

      this.startDateSet = false;
      this.startDisplayDate = startDisplayDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(startDisplayDate, B2BDateFormatter.B2BDATE);
         this.startDate = df.getXMLDate();
      } catch (Exception e) {
         this.startDate = "";
      }
   }

   /**
    *
    * @return returns true if startDate is set, false otherwise
    */
   public boolean isStartDateSet() {
      return startDateSet;
   }

   /**
    *
    * @param startDateSet The startDateSet to set
    */
   public void setStartDateSet(boolean startDateSet) {
      this.startDateSet = startDateSet;
   }

   /**
    *
    * @return returns the endDate
    */
   public String getEndDate() {
      return endDate;
   }

   /**
    *
    * @return returns the endDisplayDate
    */
   public String getEndDisplayDate() {
      return endDisplayDate;
   }

   /**
    *
    * @param endDate
    */
   public void setEndDate(String endDate) {
      this.endDateSet = true;
      this.endDate = endDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(endDate, B2BDateFormatter.XMLDATE);
         this.endDisplayDate = df.getB2BDate();
      } catch (Exception e) {

         this.endDisplayDate = "";
      }
   }

   /**
    *
    * @param endDisplayDate
    */
   public void setEndDisplayDate(String endDisplayDate) {
      this.endDateSet = false;
      this.endDisplayDate = endDisplayDate;
      try {
         B2BDateFormatter df = new B2BDateFormatter(endDisplayDate, B2BDateFormatter.B2BDATE);
         this.endDate = df.getXMLDate();
      } catch (Exception e) {
         this.endDate = "";
      }
   }

   /**
    *
    * @return returns true if endDate is set, false otherwise
    */
   public boolean isEndDateSet() {
      return endDateSet;
   }

   /**
    *
    * @param endDateSet The endDateSet to set
    */
   public void setEndDateSet(boolean endDateSet) {
      this.endDateSet = endDateSet;
   }

   /**
    *
    * @return return the isStartTimeSet
    */
   public boolean getIsStartTimeSet() {
     return isStartTimeSet;
   }


   /**
    * @return Returns the startTime.
    */
   public String getStartTime() {
     if (startTime == null || startTime.equals("")) {
       try {
         B2BTimeFormatter tf = new B2BTimeFormatter(getStartDisplayTime(), B2BTimeFormatter.B2BTIME);
         this.startTime = tf.getXMLTime();
       } catch (Exception e) {
         this.startTime = "";
       }
     }
     return this.startTime;
   }

   /**
    * @param startTime The startTime to set.
    */
   public void setStartTime(String startTime) {
     this.isStartTimeSet = true;
     this.startTime = startTime;
     try {
       B2BTimeFormatter df = new B2BTimeFormatter(startTime, B2BTimeFormatter.XMLTIME);
       this.startDisplayTime = df.getB2BTime();
     } catch (Exception e) {
       this.startDisplayTime = "";
     }
   }

   /**
    * @return Returns the startDisplayTime.
    */
   public String getStartDisplayTime() {
     return this.startDisplayTime;
   }

   /**
    * @param startDisplayTime The startTime to set.
    */
   public void setStartDisplayTime(String startDisplayTime) {
     this.isStartTimeSet = false;
     this.startDisplayTime = startDisplayTime;
   }

   public void setDuration(String duration) {
      this.duration = duration;
   }

   public String getDuration() {
      return duration;
   }

   public void setNotes(String notes) {
      this.notes = notes;
   }

   public String getNotes() {
      return notes;
   }

   public void setPlannedInterruptionReason(String plannedInterruptionReason) {
      this.plannedInterruptionReason = plannedInterruptionReason;
   }

   public String getPlannedInterruptionReason() {
      return plannedInterruptionReason;
   }

   /**
    * Getter for reasonsForInterruption
    * @return List
    */
   public List<? extends Object> getPlannedInterruptionReasons() {
      return PlannedInterruptionReasons.PLANNED_INTERRUPTION_REASONS;
   }

}
