/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/SORResponse.java-arc   1.8   11 Apr 2005 11:14:22   hendyl  $
 * $Revision:   1.8  $
 * $Date:   11 Apr 2005 11:14:22  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Modification History
 *
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 21-Aug-2008    Bob Jones         Defect 4985 - increase each telephone number from one to three.
 * 18-Mar-2011    Eddie Graham      Created.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb;

import java.util.List;

import com.capgemini.nemmco.b2bb.enums.ExceptionCode;
import com.capgemini.nemmco.b2bb.enums.ResponseType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderStatus;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import com.capgemini.nemmco.b2bb.validators.Address;
import com.capgemini.nemmco.b2bb.validators.PersonName;
import com.capgemini.nemmco.b2bb.validators.Telephone;

/**
 * Create response transaction Service Order Response
 *
 * @author $Author:   hendyl  $
 * @version $Revision:   1.8  $ $Date:   11 Apr 2005 11:14:22  $
 */
public class SORResponse extends Transaction {
  public static String ACTUAL_DATE_TIME = "actualDateTime";
  public static String ADDRESS = "address";
  public static String NMI = "NMI";
  public static String NMI_CHECKSUM = "NMIChecksum";
  public static String PERSON_NAME = "personName";
  public static String PRODUCT_CODE1 = "productCode1";
  public static String PRODUCT_CODE2 = "productCode2";
  public static String PRODUCT_CODE3 = "productCode3";
  public static String RESPONSE_CODE = "responseCode";
  public static String RESPONSE_TYPE = "responseType";  
  public static String SERVICE_ORDER_NUM = "serviceOrderNum";
  public static String SERVICE_ORDER_STATUS = "serviceOrderStatus";
  public static String SERVICE_ORDER_TYPE = "serviceOrderType";
  public static String SERVICE_ORDER_SUB_TYPE = "serviceOrderSubType";
  public static String SERVICE_PROVIDER_REFERENCE = "serviceProviderReference";
  public static String SPECIAL_NOTE1 = "specialNote1";
  public static String SPECIAL_NOTE2 = "specialNote2";
  public static String SPECIAL_NOTE3 = "specialNote3";
  public static String TELEPHONE1 = "telephone1";  
  public static String TELEPHONE2 = "telephone2";  
  public static String TELEPHONE3 = "telephone3";  
  public static String INITIATING_HEADER = "initiatingHeader";
  public static String SOR = "serviceOrderRequest";
  
  private String actualDateTime = "";
  private boolean actualDateTimeSet;
  private String actualDisplayDate = "";
  private String actualDisplayTime = "";
  private Address address = null;
  private String nmi = "";
  private String NMIChecksum = "";
  private PersonName personName = null;
  private String productCode1 = "";
  private String productCode2 = "";
  private String productCode3 = "";
  private String responseCode = "";
  private String responseType = "";
  private String serviceOrderNum = "";
  private String serviceOrderStatus = "";
  private String serviceOrderType = "";
  private String serviceOrderSubType = "";
  private String serviceProviderReference = "";
  private ServiceOrderRequest sor = null;
  private String specialNote1 = "";
  private String specialNote2 = "";
  private String specialNote3 = "";
  private Telephone telephone1 = null;
  private Telephone telephone2 = null;
  private Telephone telephone3 = null;
  
  /**
   * Default constructor
   */
  public SORResponse() {
    super(TransactionType.SOR_RESPONSE);
  }

  /**
   * @return Returns the actualDateTime.
   */
  public String getActualDateTime() {
    if (actualDateTime == null || actualDateTime.equals("")) {
      try {
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(getActualDisplayDateTime(), B2BDateTimeFormatter.B2BDATE);
        this.actualDateTime = df.getXMLFormattedDateTime();
      } catch (Exception e) {
        this.actualDateTime = "";
      }
    }

    return actualDateTime;
  }

  /**
   * @return Returns the actualDisplayDate.
   */
  public String getActualDisplayDate() {
    return actualDisplayDate;
  }

  /**
   * @return Returns the actualDisplayTime.
   */
  public String getActualDisplayDateTime() {
    return actualDisplayDate + " " + actualDisplayTime;
  }

  /**
   * @return Returns the actualDisplayTime.
   */
  public String getActualDisplayTime() {
    return actualDisplayTime;
  }

  /**
   * 
   * @return Returns the address
   */
  public Address getAddress() {
    return address;
  }
  
  /**
   * 
   * @return Returns the NMI
   */
  public String getNMI() {
    return nmi;
  }
  
  /**
   * 
   * @return Returns the NMIChecksum
   */
  public String getNMIChecksum() {
    return NMIChecksum;
  }

  /**
   * 
   * @return Returns the personName
   */
  public PersonName getPersonName() {
    return personName;
  }
  
  /**
   * 
   * @return Returns the productCode1
   */
  public String getProductCode1() {
    return productCode1;
  }

  /**
   * 
   * @return Returns the productCode2
   */
  public String getProductCode2() {
    return productCode2;
  }

  /**
   * 
   * @return Returns the productCode3
   */
  public String getProductCode3() {
    return productCode3;
  }

  /**
   * 
   * @return Returns the responseCode
   */
  public String getResponseCode() {
    return responseCode;
  }

  /**
   * 
   * @return Returns the responseCodes list
   */
  public List<? extends Object> getResponseCodes() {
    return ExceptionCode.EXCEPTION_CODES;
  }

  /**
   * 
   * @return Returns the responseType
   */
  public String getResponseType() {
    return responseType;
  }

  /**
   * 
   * @return Returns the responseType list
   */
  public List<? extends Object> getResponseTypes() {
    return ResponseType.RESPONSE_TYPE;
  }

  /**
   * 
   * @return Returns the serviceOrderNumber
   */
  public String getServiceOrderNum() {
    return serviceOrderNum;
  }
  
  /**
   * 
   * @return Returns the serviceOrderRequest object
   */
  public ServiceOrderRequest getServiceOrderRequest() {
    return sor;
  }

  /**
   * 
   * @return Returns the serviceOrderStatus
   */
  public String getServiceOrderStatus() {
    return serviceOrderStatus;
  }

  /**
   * 
   * @return Returns the serviceOrderType
   */
  public String getServiceOrderType() {
    return serviceOrderType;
  }

  /**
   * 
   * @return Returns the serviceOrderSubType
   */
  public String getServiceOrderSubType() {
    return serviceOrderSubType;
  }

  /**
   * 
   * @return Returns the serviceOrderStatus list
   */
  public List<? extends Object> getServiceOrderStatuses() {
    return ServiceOrderStatus.SERVICE_ORDER_STATUS_TYPE;
  }

  /**
   * 
   * @return Returns the serviceProviderReference
   */
  public String getServiceProviderReference() {
    return serviceProviderReference;
  }

  /**
   * 
   * @return Returns the specialNote1
   */
  public String getSpecialNote1() {
    return specialNote1;
  }

  /**
   * 
   * @return Returns the specialNote2
   */
  public String getSpecialNote2() {
    return specialNote2;
  }

  /**
   * 
   * @return Returns the specialNote3
   */
  public String getSpecialNote3() {
    return specialNote3;
  }

  /**
   * 
   * @return Returns the telephone object
   */
  public Telephone getTelephone1() {
    return telephone1;
  }

  public Telephone getTelephone2() {
    return telephone2;
  }
  
  public Telephone getTelephone3() {
    return telephone3;
  }

  /**
   * @return Returns true if actualDateTimeSet is set, false otherwise
   */
  public boolean isActualDateTimeSet() {
    return actualDateTimeSet;
  }

  /**
   * @param actualDateTime The actualDateTime to set.
   */
  public void setActualDateTime(String actualDateTime) {
    this.actualDateTimeSet = true;
    this.actualDateTime = actualDateTime;
    try {
      // split
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(actualDateTime, B2BDateTimeFormatter.XMLDATE);
      this.actualDisplayDate = df.getB2BDate();
      this.actualDisplayTime = df.getB2BTime();
    } catch (Exception e) {
      this.actualDisplayDate = "";
      this.actualDisplayTime = "";
    }
  }

  /**
   * @param actualDateTimeSet The actualDateTimeSet to set.
   */
  public void setActualDateTimeSet(boolean actualDateTimeSet) {
    this.actualDateTimeSet = actualDateTimeSet;
  }

  /**
   * @param actualDisplayDate The actualDisplayDate to set.
   */
  public void setActualDisplayDate(String actualDisplayDate) {
    this.actualDateTimeSet = false;
    this.actualDisplayDate = actualDisplayDate;
  }

  /**
   * @param actualDisplayTime The actualDisplayTime to set.
   */
  public void setActualDisplayTime(String actualDisplayTime) {
    this.actualDateTimeSet = false;
    this.actualDisplayTime = actualDisplayTime;
  }

  /**
   * 
   * @param address The address object to set
   */
  public void setAddress(Address address) {
    this.address = address;
  }

  /**
   * 
   * @param NMI The NMI to set
   */
  public void setNMI(String NMI) {
    this.nmi = NMI;
  }

  /**
   * 
   * @param NMIChecksum The NMIChecksum to set
   */
  public void setNMIChecksum(String NMIChecksum) {
    this.NMIChecksum = NMIChecksum;
  }

  /**
   * 
   * @param personName The personName object to set
   */
  public void setPersonName(PersonName personName) {
    this.personName = personName;
  }

  /**
   * 
   * @param productCode1 The productCode1 to set
   */
  public void setProductCode1(String productCode1) {
    this.productCode1 = productCode1;
  }

  /**
   * 
   * @param productCode2 The productCode2 to set
   */
  public void setProductCode2(String productCode2) {
    this.productCode2 = productCode2;
  }

  /**
   * 
   * @param productCode3 The productCode3 to set
   */
  public void setProductCode3(String productCode3) {
    this.productCode3 = productCode3;
  }

  /**
   * 
   * @param responseCode The responseCode to set
   */
  public void setResponseCode(String responseCode) {
    this.responseCode = responseCode;
  }

  /**
   * 
   * @param responseType The responseType to set
   */
  public void setResponseType(String responseType) {
    this.responseType = responseType;
  }

  /**
   * 
   * @param serviceOrderNum The serviceOrderNum to set
   */
  public void setServiceOrderNum(String serviceOrderNum) {
    this.serviceOrderNum = serviceOrderNum;
  }
  
  /**
   * 
   * @param sor The ServiceOrderRequest object to set
   */
  public void setServiceOrderRequest(ServiceOrderRequest sor) {
    this.sor = sor;

  }

  /**
   * 
   * @param serviceOrderStatus The serviceOrderStatus to set
   */
  public void setServiceOrderStatus(String serviceOrderStatus) {
    this.serviceOrderStatus = serviceOrderStatus;
  }

  /**
   * 
   * @param serviceOrderStatus The serviceOrderType to set
   */
  public void setServiceOrderType(String serviceOrderType) {
    this.serviceOrderType = serviceOrderType;
  }

  /**
   * 
   * @param serviceOrderStatus The serviceOrderSubType to set
   */
  public void setServiceOrderSubType(String serviceOrderSubType) {
    this.serviceOrderSubType = serviceOrderSubType;
  }

  /**
   * 
   * @param serviceProviderRef The serviceProviderReference to set
   */
  public void setServiceProviderReference(String serviceProviderRef) {
    this.serviceProviderReference = serviceProviderRef;
  }

  /**
   * 
   * @param note1 The specialNote1 to set
   */
  public void setSpecialNote1(String note1) {
    this.specialNote1 = note1;
  }

  /**
   * 
   * @param note2 The specialNote2 to set
   */
  public void setSpecialNote2(String note2) {
    this.specialNote2 = note2;
  }

  /**
   * 
   * @param note3 The specialNote3 to set
   */
  public void setSpecialNote3(String note3) {
    this.specialNote3 = note3;
  }

  /**
   * 
   * @param phone The telephone objects to set
   */
  public void setTelephone1(Telephone phone) {
    this.telephone1 = phone;
  }
  
  public void setTelephone2(Telephone phone) {
    this.telephone2 = phone;
  }
  
  public void setTelephone3(Telephone phone) {
    this.telephone3 = phone;
  } 
  
}

