/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/SiteAddressNotification.java-arc   1.13   11 Apr 2005 11:14:14   hendyl  $
 * $Revision:   1.13  $
 * $Date:   11 Apr 2005 11:14:14  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */

package com.capgemini.nemmco.b2bb;

/**
 * Defines the site address notification transaction
 * a code
 *
 * @author $Author:   hendyl  $
 * @version $Revision:   1.13  $ $Date:   11 Apr 2005 11:14:14  $
 */
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import com.capgemini.nemmco.b2bb.validators.Address;

public class SiteAddressNotification extends Transaction {

  public static String ADDRESS = "address";
  public static String LAST_MODIFIED_DATETIME = "lastModifiedDateTime";
  public static String NMI = "NMI";
  public static String NMI_CHECKSUM = "NMIChecksum";
  
  private Address address = null;
  private String lastModifiedDateTime = "";
  private boolean lastModifiedDateTimeSet;
  private String lastModifiedDisplayDate = "";
  private String lastModifiedDisplayTime = "";
  private String nmi = "";
  private String NMIChecksum = "";

  /**
   * Default constructor
   */
  public SiteAddressNotification() {
    super(TransactionType.SITE_ADDRESS_NOT);
  }

  /**
   * 
   * @return returns the address object for this transaction
   */
  public Address getAddress() {
    return address;
  }

  /**
   * @return Returns the lastModifiedDateTime.
   */
  public String getLastModifiedDateTime() {
    if (lastModifiedDateTime == null || lastModifiedDateTime.equals("")) {
      try {
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(getLastModifiedDisplayDateTime(), B2BDateTimeFormatter.B2BDATE);
        this.lastModifiedDateTime = df.getXMLFormattedDateTime();
      } catch (Exception e) {
        this.lastModifiedDateTime = "";
      }
    }

    return lastModifiedDateTime;
  }

  /**
   * @return Returns the lastModifiedDisplayDate.
   */
  public String getLastModifiedDisplayDate() {
    return lastModifiedDisplayDate;
  }

  /**
   * @return Returns the lastModifiedDisplayTime.
   */
  public String getLastModifiedDisplayDateTime() {
    return lastModifiedDisplayDate + " " + lastModifiedDisplayTime;
  }

  /**
   * @return Returns the lastModifiedDisplayTime.
   */
  public String getLastModifiedDisplayTime() {
    return lastModifiedDisplayTime;
  }
  
  /**
   * 
   * @return returns the NMI
   */
  public String getNMI() {
    return nmi;
  }
  
  /**
   * 
   * @return returns the NMIChecksum
   */
  public String getNMIChecksum() {
    return NMIChecksum;
  }

  /**
   * @return Returns the lastModifiedDateTimeSet.
   */
  public boolean isLastModifiedDateTimeSet() {
    return lastModifiedDateTimeSet;
  }

  /**
   * 
   * @param address The Address object to set
   */
  public void setAddress(Address address) {
    this.address = address;
  }

  /**
   * @param lastModifiedDateTime The lastModifiedDateTime to set.
   */
  public void setLastModifiedDateTime(String lastModifiedDateTime) {
    this.lastModifiedDateTimeSet = true;
    this.lastModifiedDateTime = lastModifiedDateTime;
    try {
      // split
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(lastModifiedDateTime, B2BDateTimeFormatter.XMLDATE);
      this.lastModifiedDisplayDate = df.getB2BDate();
      this.lastModifiedDisplayTime = df.getB2BTime();
    } catch (Exception e) {
      this.lastModifiedDisplayDate = "";
      this.lastModifiedDisplayTime = "";
    }
  }

  /**
   * @param lastModifiedDateTimeSet The lastModifiedDateTimeSet to set.
   */
  public void setLastModifiedDateTimeSet(boolean lastModifiedDateTimeSet) {
    this.lastModifiedDateTimeSet = lastModifiedDateTimeSet;
  }

  /**
   * @param lastModifiedDisplayDate The lastModifiedDisplayDate to set.
   */
  public void setLastModifiedDisplayDate(String lastModifiedDisplayDate) {
    this.lastModifiedDateTimeSet = false;
    this.lastModifiedDisplayDate = lastModifiedDisplayDate;
  }

  /**
   * @param lastModifiedDisplayTime The lastModifiedDisplayTime to set.
   */
  public void setLastModifiedDisplayTime(String lastModifiedDisplayTime) {
    this.lastModifiedDateTimeSet = false;
    this.lastModifiedDisplayTime = lastModifiedDisplayTime;
  }

  /**
   * 
   * @param NMI The NMI to set
   */
  public void setNMI(String NMI) {
    this.nmi = NMI;
  }

  /**
   * 
   * @param NMIChecksum The NMIChecksum to set
   */
  public void setNMIChecksum(String NMIChecksum) {
    this.NMIChecksum = NMIChecksum;
  }
}

