/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/enums/B2BBEntity.java-arc   1.4   26 Apr 2005 14:22:36   RayanS  $
 * $Revision:   1.4  $
 * $Date:   26 Apr 2005 14:22:36  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 22-Jan-2009    Eddie Graham      Defect 4793. Add new constant B2BB_U202_OWN.
 * 23-Dec-2010    Bob Jones         6712 Add new constant B2BB_TRANSFORMS.
 * 01-Jun-2011    NG                7417 - MSATS Inbox files to be archived & archive copy/replication to inactive/backup participant servers -- same as for outbox archival/replication
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 27-Jun-2017    Harpreet Kaur    7135 Package 7B:Add new constant SITE_ACCESS_RE
 * 08-Jun-2017    SP              new screen for SMP
 * 30-Jun-2017    Harpreet Kaur    7135 Package 7B:Add new constant B2BB_U202_PINTERRUPTN
 * 04-Jul-2017    Harpreet Kaur    7135 Package 7B:Add new constant B2BB_U202_MFIN
 * 10-Jul-2017    SP              SMP  workpkg7b- adding the Notice Of metering works
 * 28-Jul-2017    Harpreet Kaur    7135 Package 7B: Changes doen for SOR Meter Service Works screens as part of POC SMP
 * 09-Aug-2017    Harpreet Kaur    7135 Package 7B: Changes done for SOR Re-Energisation, De-Energisation and SpecialRead  screens as part of POC SMP
 * 31-Oct-2017    Harpreet Kaur    7135 Package 7B: Defect# 8298 Re-Energisation : New Reading Required changes
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * A typesafe enumeration of the B2BB entities available in the B2B Browser Application.
 *
 * Immutable, Thread safe, and resolves properly on deserialization.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.4  $ $Date:   26 Apr 2005 14:22:36  $
 */

public final class B2BBEntity extends Enumeration {
  public static final B2BBEntity B2BB_DIR_INBOX = new B2BBEntity("B2BB_DIR_INBOX", "B2B Directory Inbox");
  public static final B2BBEntity B2BB_DIR_OUTBOX = new B2BBEntity("B2BB_DIR_OUTBOX", "B2B Directory Outbox");
  public static final B2BBEntity B2BB_HUB_MESSAGE = new B2BBEntity("B2BB_HUB_MESSAGE", "B2B Hub Message");
  public static final B2BBEntity B2BB_HUB_QUEUE = new B2BBEntity("B2BB_HUB_QUEUE", "B2B Hub Queue");
  public static final B2BBEntity B2BB_DIR_ARCHIVE = new B2BBEntity("B2BB_DIR_ARCHIVE", "B2B Directory Archive");
  public static final B2BBEntity B2BB_DIR_INBOX_ARCHIVE = new B2BBEntity("B2BB_DIR_INBOX_ARCHIVE", "B2B Directory Inbox_Archive");

  public static final B2BBEntity B2BB_U202_SORALLOCATENMI = new B2BBEntity("B2BB_U202_SORALLOCATENMI", "B2B Trans Service Order Request Allocation NMI");
  public static final B2BBEntity B2BB_U202_SORNEWCONN = new B2BBEntity("B2BB_U202_SORNEWCONN", "B2B Trans Service Order Request New Connection");
  public static final B2BBEntity B2BB_U202_SORNEWCONN_T = new B2BBEntity("B2BB_U202_SORNEWCONN_T", "B2B Trans Service Order Request New Connection Temporary");
  public static final B2BBEntity B2BB_U202_SORNEWCONN_TP = new B2BBEntity("B2BB_U202_SORNEWCONN_TP", "B2B Trans Service Order Request New Connection Temporary in Permanent");
  public static final B2BBEntity B2BB_U202_SORNEWCONN_SUPP_ALT = new B2BBEntity("B2BB_U202_SORNEWCONN_SUPP_ALT", "B2B Trans Service Order Request New Connection Supply Alteration");

  // Re-Energisation
  public static final B2BBEntity B2BB_U202_SORRENER = new B2BBEntity("B2BB_U202_SORRENER", "B2B Trans Service Order Request Re-energisation Remote");
  public static final B2BBEntity B2BB_U202_SORRENERPHYVISIT = new B2BBEntity("B2BB_U202_SORRENERPHYVISIT", "B2B Trans Service Order Request Re-energisation Physical Visit");
  public static final B2BBEntity B2BB_U202_SORRENERRETROMOVEIN = new B2BBEntity("B2BB_U202_SORRENERRETROMOVEIN", "B2B Trans Service Order Request Re-energisation Retrospective Move-in");
  public static final B2BBEntity B2BB_U202_SORRENERRECDISCR = new B2BBEntity("B2BB_U202_SORRENERRECDISCR", "B2B Trans Service Order Request Re-energisation Recipient Discretion");
  public static final B2BBEntity B2BB_U202_SORRENERAFTDISCONN = new B2BBEntity("B2BB_U202_SORRENERAFTDISCONN", "B2B Trans Service Order Request Re-energisation After Disconnection For Non-Payment");
  public static final B2BBEntity B2BB_U202_SORRENERMOVEIN = new B2BBEntity("B2BB_U202_SORRENERMOVEIN", "B2B Trans Service Order Request Re-energisation Move-in");
  public static final B2BBEntity B2BB_U202_SORRENERNEWREADREQ = new B2BBEntity("B2BB_U202_SORRENERNEWREADREQ", "B2B Trans Service Order Request Re-energisation New Reading Required");

 //De-Energisation
  public static final B2BBEntity B2BB_U202_SORDENER = new B2BBEntity("B2BB_U202_SORDENER", "B2B Trans Service Order Request De-energisation");
  public static final B2BBEntity B2BB_U202_SORDENERREMOTE = new B2BBEntity("B2BB_U202_SORDENERREMOTE", "B2B Trans Service Order Request De-energisation Remote");
  public static final B2BBEntity B2BB_U202_SORDENERLOCMETDIS = new B2BBEntity("B2BB_U202_SORDENERLOCMETDIS", "B2B Trans Service Order Request De-energisation Local meter Disconnection");
  public static final B2BBEntity B2BB_U202_SORDENERRECDISC = new B2BBEntity("B2BB_U202_SORDENERRECDISC", "B2B Trans Service Order Request De-energisation Recipient Discretion");
  public static final B2BBEntity B2BB_U202_SORDENERREMOVEFUSE = new B2BBEntity("B2BB_U202_SORDENERREMOVEFUSE", "B2B Trans Service Order Request De-energisation Remove Fuse");

  //Special Read
  public static final B2BBEntity B2BB_U202_SORSREAD = new B2BBEntity("B2BB_U202_SORSREAD", "B2B Trans Service Order Request Special Read");
  public static final B2BBEntity B2BB_U202_SORSREADFINAL = new B2BBEntity("B2BB_U202_SORSREADFINAL", "B2B Trans Service Order Request Special Read Final");
  public static final B2BBEntity B2BB_U202_SORSREADCHECK = new B2BBEntity("B2BB_U202_SORSREADCHECK", "B2B Trans Service Order Request Special Read Check");

 //Meter Service Works
  public static final B2BBEntity B2BB_U202_SORMSWINSTALL = new B2BBEntity("B2BB_U202_SORMSWINSTALL", "B2B Trans Service Order Request Meter Service Works Install Meter");
  public static final B2BBEntity B2BB_U202_SORMSWMOVE = new B2BBEntity("B2BB_U202_SORMSWMOVE", "B2B Trans Service Order Request Meter Service Works Move Meter");
  public static final B2BBEntity B2BB_U202_SORMSWREMOVE = new B2BBEntity("B2BB_U202_SORMSWREMOVE", "B2B Trans Service Order Request Meter Service Works Remove Meter");
  public static final B2BBEntity B2BB_U202_SORMSWEXCHANGE = new B2BBEntity("B2BB_U202_SORMSWEXCHANGE", "B2B Trans Service Order Request Meter Service Works Exchange Meter");
  public static final B2BBEntity B2BB_U202_SORMSWCLOAD = new B2BBEntity("B2BB_U202_SORMSWCLOAD", "B2B Trans Service Order Request Meter Service Works Controlled Load");
  public static final B2BBEntity B2BB_U202_SORMRECONFIG = new B2BBEntity("B2BB_U202_SORMRECONFIG", "B2B Trans Service Order Request Meter Reconfiguration");
  public static final B2BBEntity B2BB_U202_SORMRESEAL = new B2BBEntity("B2BB_U202_SORMRESEAL", "B2B Trans Service Order Request Meter Reconfiguration - Reseal");
  public static final B2BBEntity B2BB_U202_SORMINVEST = new B2BBEntity("B2BB_U202_SORMINVEST", "B2B Trans Service Order Request Meter Investigation");
  public static final B2BBEntity B2BB_U202_SORSABOLISH = new B2BBEntity("B2BB_U202_SORSABOLISH", "B2B Trans Service Order Request Supply Abolishment");
  public static final B2BBEntity B2BB_U202_SORTEMPISOLATION = new B2BBEntity("B2BB_U202_SORTEMPISOLATION", "B2B Trans Service Order Request Temporary Isoaltion");
  public static final B2BBEntity B2BB_U202_SORTEMPISOLATION_GROUP = new B2BBEntity("B2BB_U202_SORTEMPISOLATION_GROUP", "B2B Trans Service Order Request Temporary Isoaltion Group");
  public static final B2BBEntity B2BB_U202_SORMISS = new B2BBEntity("B2BB_U202_SORMISS", "B2B Trans Service Order Request Miscellaneous");
  public static final B2BBEntity B2BB_U202_SORCANCL = new B2BBEntity("B2BB_U202_SORCANCL", "B2B Trans Cancel Service Order Request");
  public static final B2BBEntity B2BB_U202_CDN = new B2BBEntity("B2BB_U202_CDN", "B2B Trans Customer Details Notification");
  public static final B2BBEntity B2BB_U202_SACCESSR = new B2BBEntity("B2BB_U202_SACCESSR", "B2B Trans Site Access Request");
  public static final B2BBEntity B2BB_U202_SACCESSN = new B2BBEntity("B2BB_U202_SACCESSN", "B2B Trans Site Access Notification");
  public static final B2BBEntity B2BB_U202_SADDRESSN = new B2BBEntity("B2BB_U202_SADDRESSN", "B2B Trans Site Address Notification");
  public static final B2BBEntity B2BB_U202_PMDR = new B2BBEntity("B2BB_U202_PMDR", "B2B Trans Provide Meter Data Request");
  public static final B2BBEntity B2BB_U202_MDVR = new B2BBEntity("B2BB_U202_VMDR", "B2B Trans Verify Meter Data Request");
  public static final B2BBEntity B2BB_U202_SORESPONSE = new B2BBEntity("B2BB_U202_SORESPONSE", "B2B Trans Service Order Response");
  public static final B2BBEntity B2BB_U202_SORSTARRIFCHANGE = new B2BBEntity("B2BB_U202_SORSTARRIFCHANGE", "B2B Trans Service Order Request Tariff Change");

  public static final B2BBEntity B2BB_U107_MSG_ACK = new B2BBEntity("B2BB_U107_MSG_ACK", "B2B Message Acknowledgement");

  public static final B2BBEntity B2BB_U204_TRANS_ACK = new B2BBEntity("B2BB_U204_TRANS_ACK", "B2B Trans Acknowledgment");

  public static final B2BBEntity B2BB_U207_TRANSLOG = new B2BBEntity("B2BB_U207_TRANSLOG", "B2B Transaction Log");
  public static final B2BBEntity B2BB_U217_TRANSLOG = new B2BBEntity("B2BB_U217_TRANSLOG", "B2B Hub Transaction Log");
  public static final B2BBEntity B2BB_U301_FCONINFO = new B2BBEntity("B2BB_U301_FCONINFO", "B2B Flow Control Information");
  public static final B2BBEntity B2BB_U303_DASHBOARD = new B2BBEntity("B2BB_U303_DASHBOARD", "B2B Hub Dashboard");
  public static final B2BBEntity B2BB_U302_STATUSREPORT = new B2BBEntity("B2BB_U302_STATUSREPORT", "B2B Participant Status Report");
  public static final B2BBEntity B2BB_U402_ACTIVITY = new B2BBEntity("B2BB_U402_ACTIVITY", "B2B Activity Log");
  public static final B2BBEntity B2BB_U403_TOGMODFUNC = new B2BBEntity("B2BB_U403_TOGMODFUNC", "B2B Toggle Modify Functionality");
  public static final B2BBEntity B2BB_U404_AUTODELSTAT = new B2BBEntity("B2BB_U404_AUTODELSTAT", "B2B Auto Delete Status");
  public static final B2BBEntity B2BB_DELETEALL = new B2BBEntity("B2BB_DELETEALL", "B2B Delete All");

  // 22-Jan-2009 Eddie Graham Defect 4793. Add new constant B2BB_U202_OWN.
  public static final B2BBEntity B2BB_U202_OWN = new B2BBEntity("B2BB_U202_OWN", "B2B Trans One Way Notification");
  public static final B2BBEntity B2BB_U202_PINTERRUPTN = new B2BBEntity("B2BB_U202_PINTERRUPTN", "B2B Planned Interruption Notification");

  public static final B2BBEntity B2BB_TRANSFORMS = new B2BBEntity("B2BB_TRANSFORMS", "B2B Transforms");
  public static final B2BBEntity B2BB_U202_REMOTESERVICEREQ = new B2BBEntity("B2BB_U202_REMOTESERVICEREQ", "B2B Remote Service Request");
  public static final B2BBEntity B2BB_U202_REMOTESERVICERES = new B2BBEntity("B2BB_U202_REMOTESERVICERES", "B2B Remote Service Response");

  public static final B2BBEntity B2BB_U202_MFIN = new B2BBEntity("B2BB_U202_MFIN", "B2B Meter Fault And Issue Notification");
  public static final B2BBEntity B2BB_U202_NOTICEOFMETERINGWORKS = new B2BBEntity("B2BB_U202_NOTICEOFMETERINGWORKS", "B2B Notice Of Metering Works");
  public static final B2BBEntity B2BB_U202_PEERTOPEER = new B2BBEntity("B2BB_U202_PEERTOPEER", "B2B Peer To Peer Data Exchange");
  public static final B2BBEntity B2BB_U202_NOTIFIPARTY = new B2BBEntity("B2BB_U202_NOTIFIPARTY", "B2B Notified Party");

  private static final B2BBEntity[] B2BBEntity_ARRAY = {
      B2BB_DIR_INBOX,
      B2BB_DIR_OUTBOX,
      B2BB_HUB_MESSAGE,
      B2BB_HUB_QUEUE,
      B2BB_DIR_ARCHIVE,
      B2BB_U202_SORALLOCATENMI,
      B2BB_U202_SORNEWCONN,


   // Re-Energisation
      B2BB_U202_SORRENER,
      B2BB_U202_SORRENERPHYVISIT,
      B2BB_U202_SORRENERRETROMOVEIN,
      B2BB_U202_SORRENERRECDISCR,
      B2BB_U202_SORRENERAFTDISCONN,
      B2BB_U202_SORRENERMOVEIN,
      B2BB_U202_SORRENERNEWREADREQ,


      //De-Energisation
      B2BB_U202_SORDENER,
      B2BB_U202_SORDENERREMOTE,
      B2BB_U202_SORDENERLOCMETDIS,
      B2BB_U202_SORDENERRECDISC,
      B2BB_U202_SORDENERREMOVEFUSE,

	  //Special Read
      B2BB_U202_SORSREAD,
      B2BB_U202_SORSREADFINAL,
      B2BB_U202_SORSREADCHECK,

      B2BB_U202_SORMSWINSTALL,
      B2BB_U202_SORMSWMOVE,
      B2BB_U202_SORMSWREMOVE,
      B2BB_U202_SORMSWEXCHANGE,
      B2BB_U202_SORMSWCLOAD,
      B2BB_U202_SORMRECONFIG,
      B2BB_U202_SORMINVEST,
      B2BB_U202_SORSABOLISH,
      B2BB_U202_SORMISS,
      B2BB_U202_SORCANCL,
      B2BB_U202_CDN,
      B2BB_U202_SACCESSR,
      B2BB_U202_SACCESSN,
      B2BB_U202_SADDRESSN,
      B2BB_U202_PMDR,
      B2BB_U202_MDVR,
      B2BB_U202_REMOTESERVICEREQ,
      B2BB_U202_REMOTESERVICERES,
      B2BB_U202_SORESPONSE,
      B2BB_U107_MSG_ACK,
      B2BB_U204_TRANS_ACK,
      B2BB_U207_TRANSLOG,
      B2BB_U217_TRANSLOG,
      B2BB_U301_FCONINFO,
      B2BB_U303_DASHBOARD,
      B2BB_U302_STATUSREPORT,
      B2BB_U402_ACTIVITY,
      B2BB_U403_TOGMODFUNC,
      B2BB_U404_AUTODELSTAT,
      B2BB_DELETEALL,
      B2BB_U202_OWN,
      B2BB_U202_PINTERRUPTN,
      B2BB_U202_MFIN,
      B2BB_U202_NOTICEOFMETERINGWORKS,
      B2BB_TRANSFORMS,
	  B2BB_U202_NOTIFIPARTY ,
	  B2BB_U202_SORSTARRIFCHANGE,
   B2BB_U202_SORTEMPISOLATION,
    B2BB_U202_SORNEWCONN_T,
          B2BB_U202_SORNEWCONN_TP,
          B2BB_U202_SORNEWCONN_SUPP_ALT,
		        B2BB_U202_PEERTOPEER,
    B2BB_U202_SORTEMPISOLATION_GROUP
      };

  public static final List<? extends Object> B2BBEntities = Collections.unmodifiableList(Arrays.asList(B2BBEntity_ARRAY));

  private B2BBEntity(String code, String description) {
    this.code = code;
    this.description = description;
  }
}