/*
 * DynamicDropdown.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 27-Jun-2017    NG                Created for 7135 - PoC SMP Changes
 * 10-Jul-2017    W.Mills           7378 Transaction Type drop down is blank
 * 18-Jul-2017    Harpreet Kaur     7135 Package 7B: Changes done for SPECIAL_READ
 * 02-Aug-2017    Harpreet Kaur     7135 Package 7B: Changes done for REENERGISATION & DEENERGISATION
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * A typesafe enumeration of the Dynamic Dropdowns available in the B2B Browser Application.
 *
 * Immutable, Thread safe, and resolves properly on deserialization.

 */

public final class DynamicDropdown extends Enumeration {
  public static final DynamicDropdown SUPPLY_SERVICE_WORKS = new DynamicDropdown("Supply Service Works", "Supply Service Works");
  public static final DynamicDropdown REENERGISATION = new DynamicDropdown("Re-energisation", "Re-energisation");
  public static final DynamicDropdown DEENERGISATION = new DynamicDropdown("De-energisation", "De-energisation");
  public static final DynamicDropdown SPECIAL_READ = new DynamicDropdown("Special Read", "Special Read");
  public static final DynamicDropdown METERING_SERVICE_WORKS = new DynamicDropdown("Metering Service Works", "Metering Service Works");

  private static final DynamicDropdown[] TYPES_WITH_SUBTYPES_ARRAY = {
      SUPPLY_SERVICE_WORKS,
      //Uncomment the following when developieng those screens
      REENERGISATION,
      DEENERGISATION,
      SPECIAL_READ,
      METERING_SERVICE_WORKS
      };

  public static final List<DynamicDropdown> typesWithSubTypes = Collections.unmodifiableList(Arrays.asList(TYPES_WITH_SUBTYPES_ARRAY));



  public DynamicDropdown(String code, String description) {
    this.code = code;
    this.description = description;
  }

  public static boolean isDynamicDropDown(String transactionType) {
    for (DynamicDropdown dynamicDropdown : typesWithSubTypes)
      if (dynamicDropdown.getCode().equals(transactionType))
        return true;
    return false;
  }
}