/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/enums/ExceptionCode.java-arc   1.4   15 Apr 2005 14:11:20   hendyl  $
 * $Revision:   1.4  $
 * $Date:   15 Apr 2005 14:11:20  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 * 26-Jul-2017    Hamish McNeish PoC changes
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.capgemini.nemmco.b2bb.ServiceOrderRequest;

/**
 * Defines the Enumeration
 *
 * @author $Author:   hendyl  $
 * @version $Revision:   1.4  $ $Date:   15 Apr 2005 14:11:20  $
 */

public class ExceptionCode extends Enumeration 
                           implements Comparable<Object>, java.io.Serializable {
    // Public constants
  private static int count = 0;
  
    /**
          * The following define the enumerations.
          */
    public static final ExceptionCode METERING_PROBLEM = new ExceptionCode("Metering Problem", "Metering Problem");
    public static final ExceptionCode READING_PROBLEM = new ExceptionCode("Reading Problem", "Reading Problem");
    public static final ExceptionCode UNABLE_TO_ACCESS = new ExceptionCode("Unable To Access", "Unable To Access");
    public static final ExceptionCode NO_SUPPLY = new ExceptionCode("No Supply", "No Supply");
    public static final ExceptionCode UNSAFE = new ExceptionCode("Unsafe", "Unsafe");
    public static final ExceptionCode RECIPIENT_CANCELLATION = new ExceptionCode("Recipient Cancellation", "Recipient Cancellation");
    public static final ExceptionCode INITIATOR_CANCELLATION = new ExceptionCode("Initiator Cancellation", "Initiator Cancellation");
    public static final ExceptionCode UNKNOWN_LOAD = new ExceptionCode("Unknown Load", "Unknown Load");
    public static final ExceptionCode DOC_NOT_PROVIDED = new ExceptionCode("Documentation Not Provided", "Documentation Not Provided");
    public static final ExceptionCode REQUEST_SUBMITTED_BY_ANOTHER_INITIATOR = new ExceptionCode("Request Submitted By Another Initiator", "Request Submitted By Another Initiator");
    public static final ExceptionCode DE_ENERGISATION_NOT_COMPLETED = new ExceptionCode("De-energisation Not Completed Due To A Re-energisation", "De-energisation Not Completed Due To A Re-energisation");
    public static final ExceptionCode CUST_PREVENTED = new ExceptionCode("Customer Prevented", "Customer Prevented");
    public static final ExceptionCode CUST_ON_SITE = new ExceptionCode("Customer On-Site", "Customer On-Site");
    public static final ExceptionCode NEW_CUST_ON_SITE = new ExceptionCode("New Customer On-Site", "New Customer On-Site");
    public static final ExceptionCode SENSITIVE_LOAD = new ExceptionCode("Sensitive Load", "Sensitive Load");
    public static final ExceptionCode OTHER = new ExceptionCode("Other", "Other");
    public static final ExceptionCode METER_READING_ONLY_UNDERTAKEN_DUE_TO_PRIOR_RE_ENERGISATION = new ExceptionCode("Meter Reading Only Undertaken Due To Prior Re-energisation", "Meter Reading Only Undertaken Due To Prior Re-energisation");
    
    //HM added new PoC codes
    public static final ExceptionCode LIFE_SUPPORT = new ExceptionCode("Life Support", "Life Support");
    public static final ExceptionCode NO_COMMS = new ExceptionCode("No Comms", "No Comms");
    public static final ExceptionCode UNKNOWN_CONNECTION_STATUS = new ExceptionCode("Unknown Connection Status", "Unknown Connection Status");
    public static final ExceptionCode METER_NOT_RETRIEVED = new ExceptionCode("Meter Not Retrieved", "Meter Not Retrieved");
    public static final ExceptionCode SITE_ALREADY_ENERGISED = new ExceptionCode("Site Already Energised", "Site Already Energised");
    public static final ExceptionCode SHARED_SUPPLY_POINT = new ExceptionCode("Shared Supply Point", "Shared Supply Point");
    public static final ExceptionCode TARIFF_CHANGE_NOT_APPROVED = new ExceptionCode("Tariff Change Not Approved", "Tariff Change Not Approved");
    public static final ExceptionCode METERING_NOT_COMPATIBLE = new ExceptionCode("Metering not compatible with proposed Tariff Change", "Metering not compatible with proposed Tariff Change");
    public static final ExceptionCode INADEQUATE_INFRASTRUCTURE = new ExceptionCode("Inadequate infrastructure", "Inadequate infrastructure");
        
    private static final ExceptionCode[] EXCEPTION_CODE_ARRAY =
      { METERING_PROBLEM,
        READING_PROBLEM,
        UNABLE_TO_ACCESS,
        NO_SUPPLY,
        UNSAFE,
        RECIPIENT_CANCELLATION,
        INITIATOR_CANCELLATION,
        UNKNOWN_LOAD,
        DOC_NOT_PROVIDED,
        REQUEST_SUBMITTED_BY_ANOTHER_INITIATOR,
        DE_ENERGISATION_NOT_COMPLETED,
        CUST_PREVENTED,
        CUST_ON_SITE,
        NEW_CUST_ON_SITE,
        SENSITIVE_LOAD,
        METER_READING_ONLY_UNDERTAKEN_DUE_TO_PRIOR_RE_ENERGISATION,
        //HM PoC additions
        LIFE_SUPPORT,
        NO_COMMS,
        UNKNOWN_CONNECTION_STATUS,
        METER_NOT_RETRIEVED,
        SITE_ALREADY_ENERGISED,
        SHARED_SUPPLY_POINT,
        TARIFF_CHANGE_NOT_APPROVED,
        METERING_NOT_COMPATIBLE,
        INADEQUATE_INFRASTRUCTURE,
        
        OTHER
      };

    public static final List<? extends Object> getAllCompletedCodes() {
        ExceptionCode[] result = new ExceptionCode[]{METER_READING_ONLY_UNDERTAKEN_DUE_TO_PRIOR_RE_ENERGISATION};
        return Collections.unmodifiableList(Arrays.asList(result));
    }
    
    public static final List<? extends Object> getAllNotCompletedCodes() {
        ExceptionCode[] result = new ExceptionCode[]{
            RECIPIENT_CANCELLATION,
            DOC_NOT_PROVIDED,
            NO_SUPPLY,
            OTHER,
            INITIATOR_CANCELLATION,
            REQUEST_SUBMITTED_BY_ANOTHER_INITIATOR,
            UNABLE_TO_ACCESS,
            NO_COMMS,
            UNKNOWN_CONNECTION_STATUS,
            UNKNOWN_LOAD,
            UNSAFE,
            SITE_ALREADY_ENERGISED,
            SHARED_SUPPLY_POINT,
            TARIFF_CHANGE_NOT_APPROVED,
            METERING_NOT_COMPATIBLE,
            INADEQUATE_INFRASTRUCTURE,
            CUST_ON_SITE,
            CUST_PREVENTED,
            DE_ENERGISATION_NOT_COMPLETED,
            NEW_CUST_ON_SITE,
            SENSITIVE_LOAD,
            LIFE_SUPPORT
        };
        return Collections.unmodifiableList(Arrays.asList(result));
    }
    public static final List<? extends Object> getNotCompletedCodes(ServiceOrderRequest request) {
      String tempServiceOrderType = request.getServiceOrderType();
      String tempServiceOrderSubType = request.getServiceOrderSubType();
      String deenergisationReason = request.getDeEnergisationReason();
        
      List<ExceptionCode> result = new ArrayList<ExceptionCode>();
      result.add(RECIPIENT_CANCELLATION);
      result.add(DOC_NOT_PROVIDED);
      result.add(NO_SUPPLY);
      result.add(OTHER);
      result.add(INITIATOR_CANCELLATION);
      result.add(REQUEST_SUBMITTED_BY_ANOTHER_INITIATOR);
      result.add(UNABLE_TO_ACCESS);
      result.add(NO_COMMS);
      result.add(UNKNOWN_CONNECTION_STATUS);
      result.add(UNKNOWN_LOAD);
      result.add(UNSAFE);
      result.add(SITE_ALREADY_ENERGISED);
      result.add(SHARED_SUPPLY_POINT);
      result.add(TARIFF_CHANGE_NOT_APPROVED);
      result.add(METERING_NOT_COMPATIBLE);
      result.add(INADEQUATE_INFRASTRUCTURE);
	  
      if (ServiceOrderType.DE_ENERGISATION.getCode().equals(tempServiceOrderType)) {
        if (!((ServiceOrderSubType.REMOVE_FUSE.getCode().equals(tempServiceOrderSubType) || ServiceOrderSubType.PILLAR_OR_POLE.getCode().equals(tempServiceOrderSubType))
                && DeEnergisationReason.NON_PAYMENT.getCode().equals(deenergisationReason))) {
          result.add(CUST_ON_SITE);
        }
        result.add(CUST_PREVENTED);
        result.add(DE_ENERGISATION_NOT_COMPLETED);
        result.add(NEW_CUST_ON_SITE);
        result.add(SENSITIVE_LOAD);
        result.add(LIFE_SUPPORT);
      }
      return Collections.unmodifiableList(result);
    }
    public static final List<? extends Object> getAllPartiallyCompletedCodes() {
        ExceptionCode[] result = new ExceptionCode[]{
            METERING_PROBLEM,
            OTHER,
            READING_PROBLEM,
            METER_NOT_RETRIEVED
        };
        return Collections.unmodifiableList(Arrays.asList(result));
    }
    public static final List<? extends Object> getPartiallyCompletedCodes(ServiceOrderRequest request) {
        List<ExceptionCode> result = new ArrayList<ExceptionCode>();
        result.add(METERING_PROBLEM);
        result.add(OTHER);
        result.add(READING_PROBLEM);

        String tempServiceOrderSubType = request.getServiceOrderSubType();
        String tempServiceOrderType = request.getServiceOrderType();
        if (ServiceOrderType.SUPP_SRV_WRK.getCode().equals(tempServiceOrderType) && ServiceOrderSubType.SUPPLY_ABOLISHMENT.getCode().equals(tempServiceOrderSubType)) {
            result.add(METER_NOT_RETRIEVED);
        }
        return Collections.unmodifiableList(result);
    }
    

    public static final List<? extends Object> EXCEPTION_CODES =
      Collections.unmodifiableList(Arrays.asList(EXCEPTION_CODE_ARRAY));

    private ExceptionCode(String code, 
                   String description) {
      this.code = code;
      this.description = description;
      this.number = ++count;
    }

    public String toString() {
      return code + "-" + description;
    }

}

