/*
 * ModifyFunctionality.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By       Description
 * ------------   ----------       -----------
 * 11-Apr-2005    RayanS           Created.
 * 14-Aug-2012    Bob Jones        7984 Java upgrade from 6 to 7.
 * 06-Apr-2017    W.Mills          Amended for B2B Hub Queue.
 * 15-Sep-2017    Harpreet Kaur    7135 Package 7B: Changes done for new combo box for RECEIVE_TRANS_ACK_NOTIFY_PARTY as part of POC SMP
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Defines the priority Enumeration
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.1  $ $Date:   11 Apr 2005 11:10:46  $
 */
public class ModifyFunctionality extends Enumeration implements Comparable<Object>, java.io.Serializable {

  // count will be seen as zero despite its value unless it's before the
  // public constant declarations
  private static int count = 0;
  
  public static final ModifyFunctionality DELETE_FILE = new ModifyFunctionality("DEL_FILE", "Delete File");
  public static final ModifyFunctionality UPLOAD_FILE_NO_VAL = new ModifyFunctionality("UPLOAD_NO_VAL", "Upload File (No Validation)");
  public static final ModifyFunctionality CREATE_RESPONSE_MSG_ACK = new ModifyFunctionality("CR_MSG_ACK", "Create Message Acknowledgement");
  public static final ModifyFunctionality AUTO_DELETE_FROM_INBOX = new ModifyFunctionality("AUTO_DEL_INBOX", "Auto Delete From Inbox");
  public static final ModifyFunctionality CREATE_NEW_TRANS = new ModifyFunctionality("C_NEW_TRANS", "Create New Transaction");
  public static final ModifyFunctionality CREATE_RESPONSE_TRANS = new ModifyFunctionality("CR_TRANS", "Create Response");
  public static final ModifyFunctionality CREATE_RESPONSE_TRANS_ACK = new ModifyFunctionality("CR_TRANS_ACK", "Create Transaction Acknowledgement");
  public static final ModifyFunctionality UPLOAD_VALIDATE_TRANS = new ModifyFunctionality("UPLOAD_VAL_TRANS", "Upload File and Validate");
  public static final ModifyFunctionality DELETE_HUB_QUEUE_MSG_ACK = new ModifyFunctionality("DEL_HUB_Q_MSG_ACK", "Delete Hub Queue Message Acknowledgement");
  public static final ModifyFunctionality REV_ONLY_REJ_TACKS = new ModifyFunctionality("REV_ONLY_REJ_TACKS", "Receive only reject Transaction Acknowledgements from Notified Parties");

  private static final ModifyFunctionality[] MODIFY_FUNC_ARRAY = {
      DELETE_FILE, 
      UPLOAD_FILE_NO_VAL, 
      CREATE_RESPONSE_MSG_ACK,
      AUTO_DELETE_FROM_INBOX,
      CREATE_NEW_TRANS,
      CREATE_RESPONSE_TRANS,
      CREATE_RESPONSE_TRANS_ACK,
      UPLOAD_VALIDATE_TRANS,
      DELETE_HUB_QUEUE_MSG_ACK,
      REV_ONLY_REJ_TACKS
      };

  public static final List<? extends Object> MODIFY_FUNCS = Collections.unmodifiableList(Arrays.asList(MODIFY_FUNC_ARRAY));

  /**
   * Return the ModifyFunctionality object given by code (key)
   * 
   * @param code
   *          The key to find in this enumeration
   * @return The object associated with the key. Null otherwise
   */
  public static ModifyFunctionality getModifyFunctionality(String code) {
    for (int i = 0; i < MODIFY_FUNCS.size(); i++) {
      if ((((Enumeration) MODIFY_FUNCS.get(i)).getCode()).equals(code))
          return (ModifyFunctionality) MODIFY_FUNCS.get(i);
    }
    return null;
  }

  private ModifyFunctionality(String code, String description) {
    this.code = code;
    this.description = description;
    this.number = ++count;
  }
}

