/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/enums/TransactionGroup.java-arc   1.5   11 Apr 2005 14:05:26   RayanS  $
 * $Revision:   1.5  $
 * $Date:   11 Apr 2005 14:05:26  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Modification History
 * 
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 23-Jan-2009    Eddie Graham      Defect 4793. Add new constant OWNP.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 15-Jun-2017    SP                  SMP  workpkg7b- adding the RemoteAccess Request screen
 * 05-Jul-2017    Harpreet Kaur     Package POC SMP Add new constant OWNX.
 * 17-Jul-2017    HAM				Added POC - NPNX, PTPE groups
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Defines the TransactionGroup Enumeration
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.5  $ $Date:   11 Apr 2005 14:05:26  $
 */
public class TransactionGroup extends Enumeration implements Comparable<Object>, java.io.Serializable {
  // Public constants
  private static int count = 0;

  /**
   * The following define the Transaction Type enumerations.
   */
  public static final TransactionGroup SITE = new TransactionGroup("SITE", "SITE");
  public static final TransactionGroup SORD = new TransactionGroup("SORD", "SORD");
  public static final TransactionGroup MTRD = new TransactionGroup("MTRD", "MTRD");
  public static final TransactionGroup CUST = new TransactionGroup("CUST", "CUST");
  // 23-Jan-2009 Eddie Graham Defect 4793. Add new constant OWNP.
  public static final TransactionGroup OWNP = new TransactionGroup("OWNP", "OWNP");
  public static final TransactionGroup MRSR = new TransactionGroup("MRSR", "MRSR");

  public static final TransactionGroup OWNX = new TransactionGroup("OWNX", "OWNX");
  public static final TransactionGroup NPNX = new TransactionGroup("NPNX", "NPNX");
  public static final TransactionGroup PTPE = new TransactionGroup("PTPE", "PTPE");
  
  private static final TransactionGroup[] TRANSACTION_GROUP_ARRAY = { SITE, SORD, MTRD, CUST, OWNP, MRSR, OWNX, NPNX, PTPE };

  public static final List<? extends Object> TRANSACTION_GROUPS = Collections.unmodifiableList(Arrays.asList(TRANSACTION_GROUP_ARRAY));
  
  /**
   * Return the Transaction object given by code (key)
   * 
   * @param code
   *          The key to find in this enumeration
   * @return The object associated with the key. Null otherwise
   */
  public static TransactionGroup getTransactionGroup(String code) {
    for (int i = 0; i < TRANSACTION_GROUPS.size(); i++) {
      if ((((Enumeration) TRANSACTION_GROUPS.get(i)).getCode()).equals(code))
          return (TransactionGroup) TRANSACTION_GROUPS.get(i);
    }
    return null;
  }

  private TransactionGroup(String code, String description) {
    this.code = code;
    this.description = description;
    this.number = ++count;
  }
}

