/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/enums/XMLObjectType.java-arc   1.7   Sep 15 2005 15:32:40   adij  $
 * $Revision:   1.7  $
 * $Date:   Sep 15 2005 15:32:40  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 23-Jan-2009    Eddie Graham      Defect 4793. Modify array XML_OBJECT_ARRAY by adding TransactionType.ONE_WAY_NOT.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 27-June-2017   NG                7135 - PoC SMP Changes
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * A typesafe enumeration of the XMLObjectTypes available in the B2B Browser Application.
 *
 * Immutable, Thread safe, and resolves properly on deserialization.
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.7  $ $Date:   Sep 15 2005 15:32:40  $
 */
public class XMLObjectType extends Enumeration implements Comparable<Object>, java.io.Serializable {

  private static int count = 0;

  public static final XMLObjectType MESSAGE                  = new XMLObjectType("Message"                    , "Message"                                     , "Message"                     , "com.capgemini.nemmco.b2bb.Message"                             , ""                                                                                        , "");
  public static final XMLObjectType HEADER                   = new XMLObjectType("Header"                     , "Header"                                      , "Header"                      , "com.capgemini.nemmco.b2bb.Header"                              , "com.capgemini.nemmco.b2bb.validators.HeaderExtValidator"                                 , "com.capgemini.nemmco.b2bb.validators.HeaderExtValidator"                 );
  public static final XMLObjectType TRANS_ACK                = new XMLObjectType("TransactionAcknowledgement" , "Transaction Acknowledgement"                 , "TransactionAcknowledgement"  , "com.capgemini.nemmco.b2bb.TransactionAcknowledgement"          , "com.capgemini.nemmco.b2bb.validators.TransactionAcknowledgementBasicValidator"           , "com.capgemini.nemmco.b2bb.validators.TransactionAcknowledgementBasicValidator");
  public static final XMLObjectType MSG_ACK                  = new XMLObjectType("MessageAcknowledgement"     , "Message Acknowledgement"                     , "MessageAcknowledgement"      , "com.capgemini.nemmco.b2bb.MessageAcknowledgement"              , "com.capgemini.nemmco.b2bb.validators.MessageAcknowledgementBasicValidator"               , "com.capgemini.nemmco.b2bb.validators.MessageAcknowledgementBasicValidator");
  public static final XMLObjectType ADDRESS                  = new XMLObjectType("Address"                    , "Address"                                     , "Address"                     , "com.capgemini.nemmco.b2bb.validators.Address"                  , "com.capgemini.nemmco.b2bb.validators.Address"                                            , "");
  public static final XMLObjectType PERSONNAME               = new XMLObjectType("PersonName"                 , "PersonName"                                  , "PersonName"                  , "com.capgemini.nemmco.b2bb.validators.PersonName"               , "com.capgemini.nemmco.b2bb.validators.PersonName"                                         , "");
  public static final XMLObjectType TELEPHONE                = new XMLObjectType("Telephone"                  , "Telephone"                                   , "Telephone"                   , "com.capgemini.nemmco.b2bb.validators.Telephone"                , "com.capgemini.nemmco.b2bb.validators.Telephone"                                          , "");
  public static final XMLObjectType LICENSEE                 = new XMLObjectType("Licensee"                   , "Licensee"                                    , "Licensee"                    , "com.capgemini.nemmco.b2bb.validators.Licensee"                 , "com.capgemini.nemmco.b2bb.validators.Licensee"                                           , "");
  public static final XMLObjectType EVENT                    = new XMLObjectType("Event"                      , "Event"                                       , "Event"                       , "com.capgemini.nemmco.b2bb.utils.EventObject"                   , "com.capgemini.nemmco.b2bb.validators.EventObjectBasicValidator"                          , "com.capgemini.nemmco.b2bb.validators.EventObjectBasicValidator");
  public static final XMLObjectType EVENT_MSG                = new XMLObjectType("EventMessage"               , "EventMessage"                                , "EventMessage"                , "com.capgemini.nemmco.b2bb.EventMessage"                        , ""                                                                                        , "");
  public static final XMLObjectType POSTALADDRESS            = new XMLObjectType("PostalAddress"              , "Postal Address"                              , "PostalAddress"               , "com.capgemini.nemmco.b2bb.validators.PostalAddress"            , "com.capgemini.nemmco.b2bb.validators.PostalAddress"                                      , "");
  public static final XMLObjectType CONTACTNAME              = new XMLObjectType("ContactName"                , "Contact Name"                                , "ContactName"                 , "com.capgemini.nemmco.b2bb.validators.ContactName"              , "com.capgemini.nemmco.b2bb.validators.ContactName"                                        , "");
  public static final XMLObjectType REQUESTDATA              = new XMLObjectType("RequestData"                , "Request Data"                                , "RequestData"                 , "com.capgemini.nemmco.b2bb.validators.RequestData"              , "com.capgemini.nemmco.b2bb.validators.RequestData"                                        , "");  //justina added

  // 23-Jan-2009 Eddie Graham Defect 4793. Modify array XML_OBJECT_ARRAY by adding TransactionType.ONE_WAY_NOT.
  private static final XMLObjectType[] XML_OBJECT_ARRAY = {
			HEADER,
			TRANS_ACK,
			MSG_ACK,
			ADDRESS,
			PERSONNAME,
			TELEPHONE,
			REQUESTDATA,
			LICENSEE,
			EVENT,
			EVENT_MSG,
			POSTALADDRESS,
			CONTACTNAME,
			TransactionType.SOR_CANCEL,
			
			TransactionType.SOR_ALLOCATE_NMI,
			TransactionType.SOR_NEW_CONNECTION,
			TransactionType.SOR_NEW_CONNECTION_T,
			TransactionType.SOR_NEW_CONNECTION_TP,
			TransactionType.SOR_SUPPLY_ALTERATION,
			TransactionType.SOR_SUPPLY_ABOLISHMENT,
			TransactionType.SOR_TEMP_ISOLATION,
			TransactionType.SOR_TEMP_ISOLATION_GRP,
			TransactionType.SOR_TARIFF_CHANGE,

			TransactionType.SOR_MSW_INSTALL_CLOAD,	
			TransactionType.SOR_MSW_INSTALL_METER,
			TransactionType.SOR_MSW_REMOVE_METER,
			TransactionType.SOR_MSW_MOVE_METER,
			TransactionType.SOR_MSW_EXCHANGE_METER,
			TransactionType.SOR_MSW_METER_RECONFIG,
			TransactionType.SOR_MSW_CHG_TSWITCH,
			TransactionType.SOR_MSW_METER_INV_INS,
			TransactionType.SOR_MSW_METER_INV_TEST,
			TransactionType.SOR_MSW_RESEAL,
			 
			TransactionType.SOR_REENERGISATION,
			TransactionType.SOR_REENERGISATION_PHY_VISIT,
			TransactionType.SOR_REENERGISATION_RETRO_MOVEIN,
			TransactionType.SOR_REENERGISATION_REC_DISCR,
			TransactionType.SOR_REENERGISATION_AFT_DISCONN,
			TransactionType.SOR_REENERGISATION_NEW_READ,
			TransactionType.SOR_REENERGISATION_MOVEIN,

			TransactionType.SOR_DEENERGISATION,
			TransactionType.SOR_DEENERGISATION_REMOTE,
			TransactionType.SOR_DEENERGISATION_LOC_MET_DISCONN,
			TransactionType.SOR_DEENERGISATION_REC_DISCR,
			TransactionType.SOR_DEENERGISATION_REMOVE_FUSE,

			TransactionType.SOR_SPECIAL_READ_CHECK,
			TransactionType.SOR_SPECIAL_READ_FINAL,
			TransactionType.SOR_SPECIAL_READ,

			TransactionType.SOR_MISCELLANEOUS,
			TransactionType.SOR_RESPONSE,
			TransactionType.SO_APPOINTMENT_NOT,

			TransactionType.CUST_DETAIL_NOT,
			TransactionType.SITE_ACCESS_REQ,
			TransactionType.SITE_ACCESS_NOT,
			TransactionType.SITE_ADDRESS_NOT,
			TransactionType.PROVIDE_METER_DATA_REQ,
			TransactionType.METER_DATA_VERIFY_REQ,
			TransactionType.CUST_DETAIL_REQUEST,
			TransactionType.METER_DATA_NOT,
			TransactionType.REMOTE_SERVICE_REQ,
			TransactionType.REMOTE_SERVICE_RES,
			TransactionType.SO_APPOINTMENT_NOT,
			TransactionType.ONE_WAY_NOT,
			TransactionType.PLANNED_INTERRUPT_NOT,
			TransactionType.PEER_TO_PEER
      };

  public static final List<? extends Object> XML_OBJECTS =
    Collections.unmodifiableList(Arrays.asList(XML_OBJECT_ARRAY));

  protected final String objectClass;
  protected final String basicValidatorClass;
  protected final String extValidatorClass;
  protected final String category;
  protected final String XMLIdentifier;

  protected XMLObjectType(String code,
      String description,
      String category,
      String objectClass,
      String basicValidatorClass,
      String extValidatorClass,
      String XMLIdentifier) {
    this.code = code;
    this.description = description;
    this.subDescription = "";
    this.category = category;
    this.objectClass = objectClass;
    this.basicValidatorClass = basicValidatorClass;
    this.extValidatorClass = extValidatorClass;
    this.XMLIdentifier = XMLIdentifier;
    this.number = ++count;
  }
   protected XMLObjectType(String code,
       String description,
       String subDescription,
       String category,
       String objectClass,
       String basicValidatorClass,
       String extValidatorClass,
       String XMLIdentifier) {
     this.code = code;
     this.description = description;
     this.subDescription = subDescription;
     this.category = category;
     this.objectClass = objectClass;
     this.basicValidatorClass = basicValidatorClass;
     this.extValidatorClass = extValidatorClass;
     this.XMLIdentifier = XMLIdentifier;
     this.number = ++count;
   }

  protected XMLObjectType(String code,
      String description,
      String category,
      String objectClass,
      String basicValidatorClass,
      String extValidatorClass) {
    this.code = code;
    this.description = description;
    this.subDescription = "";
    this.category = category;
    this.objectClass = objectClass;
    this.basicValidatorClass = basicValidatorClass;
    this.extValidatorClass = extValidatorClass;
    this.XMLIdentifier = null;
    this.number = ++count;
  }


  public String getObjectClass() {
    return objectClass;
  }

  public String getBasicValidatorClass() {
    return basicValidatorClass;
  }

  public String getExtendedValidatorClass() {
    return extValidatorClass;
  }

  public String getCategory() {
    return category;
  }

  public String getXMLIdentifier() {
    return XMLIdentifier;
  }

  /**
   * Return the XMLObjectType object given by code (key)
   *
   * @param code
   *          The key to find in this enumeration
   * @return The object associated with the key. Null otherwise
   */
  public static XMLObjectType getXMLObjectType(String code) {
    for (int i = 0; i < XML_OBJECTS.size(); i++) {
      if ((((Enumeration) XML_OBJECTS.get(i)).getCode()).equals(code))
          return (XMLObjectType) XML_OBJECTS.get(i);
    }
    return null;
  }

}

