/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/lookup/XPath.java-arc   1.20   Apr 11 2005 14:40:28   adij  $
 * $Revision:   1.20  $
 * $Date:   Apr 11 2005 14:40:28  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.lookup;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;

/**
 * Defines the Properties file Lookup fields
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.20  $ $Date:   Apr 11 2005 14:40:28  $
 */
public class XPath extends Lookup implements Comparable<Object>, java.io.Serializable {

  private static int count = 0;
  private static final String BASE_NAME = "com.capgemini.nemmco.b2bb.lookup.XPath";
  private static final String FileName = "com.capgemini.nemmco.b2bb.lookup.xpath";
  private static List<Object> list = null;

  /**
   * @return
   * @throws UnableToLookupResourceException
   * @deprecated Use getXPaths
   */
  public static List<Object> getList() throws UnableToLookupResourceException {
    if (list != null)
      return list;
    Vector<Object> values = new Vector<Object>();
    ResourceBundle resources = null;
    try {
      resources = ResourceBundle.getBundle(FileName);
    } catch (MissingResourceException e) {
      throw new UnableToLookupResourceException(e.getMessage());
    }
    Enumeration<String> en = resources.getKeys();
    while (en.hasMoreElements()) {
      String tmpKey = en.nextElement();
      values.add(new XPath(tmpKey, (String) resources.getObject(tmpKey)));
    }

    // create a list to return
    list = Collections.unmodifiableList(Arrays.asList(values.toArray()));
    return list;
  }

  /**
   * Given a code it will return the string representing the path. It will look for the property
   * <xmlObjectType>.<code> in the resource bundle
   * 
   * @param xmlObjectType the transaction type to look for
   * @param code the code to look for
   * @return the String representing the path. null if not found.
   * @throws UnableToLookupResourceException if the resource is not accessible
   * @deprecated Use getXPath
   */
  public static String getPathValue(String xmlObjectType, String code) throws UnableToLookupResourceException {
    List<Object> list = XPath.getList();
    for (int i = 0; i < list.size(); i++) {
      String pathValue = ((Lookup) list.get(i)).getCode();
      if (pathValue.equals(xmlObjectType + "." + code))
        return ((XPath) list.get(i)).getDescription();
    }
    // if we cant find the value throw an exception as the properties file is not complete
    throw new UnableToLookupResourceException("xpath property file is not completed: " + xmlObjectType + "." + code + " is missing");
  }

  /** 
   * @param xmlObjectType the transaction type to look for
   * @return the String representing the path. null if not found.
   * @throws UnableToLookupResourceException if the resource is not accessible
   */
  public static List<Object> getXPaths(XMLObjectType xmlObjectType) throws UnableToLookupResourceException {
    List<Object> list = XPath.getXPaths();
    List<Object> xPaths = new ArrayList<Object>();
    XPath xPath;

    for (int i = 0; i < list.size(); i++) {
      xPath = (XPath) list.get(i);
      String xPathKey = xPath.getCode();
      if (xPathKey != null && xPathKey.matches(xmlObjectType.getCategory() + "\\[\\w+\\].*")) {
        xPaths.add(xPath);
      }
    }

    return xPaths;
  }
  
  /**
   * @return
   * @throws UnableToLookupResourceException
   */
  public static List<Object> getXPaths() throws UnableToLookupResourceException {
    if (list != null) return list;
    
    ArrayList<Object> values = new ArrayList<Object>();
    ResourceBundle resources = null;
    
    try {
      resources = ResourceBundle.getBundle(BASE_NAME);
    } catch (MissingResourceException e) {
      throw new UnableToLookupResourceException(e.getMessage());
    }
    
    java.util.Enumeration<String> en = resources.getKeys();
    while (en.hasMoreElements()) {
      String tmpKey = en.nextElement();
      values.add(new XPath(tmpKey, (String) resources.getObject(tmpKey)));
    }

    // create a list to return
    list = Collections.unmodifiableList(values);
    return list;
  }

  public static String getChildCategory(String field) {
    int openIndex = field.indexOf("(");
    int closeIndex = field.lastIndexOf(")");
    
    if (openIndex > -1 && closeIndex > -1) {
      return field.substring(openIndex + 1, closeIndex);
    } else {
      return null;
    }
  }
  
  public static String getValue(String field) {
    int openIndex = field.indexOf("[.='");
    int closeIndex = field.lastIndexOf("']");
    
    if (openIndex > -1 && closeIndex > -1) {
      return field.substring(openIndex + 4, closeIndex);
    } else {
      return null;
    }
  }
  
  public static String removeLast(String xPath) {
    int index = xPath.lastIndexOf("/");
    
    if (index > -1) {
      return xPath.substring(0, index);
    } else {
      return xPath;
    }
  }
  
  public static String removeFirst(String xPath) {
    int index = xPath.indexOf("/");
    
    if (index > -1) {
      return xPath.substring(index + 1);
    } else {
      return xPath;
    }
  }

  /**
   * @param category
   * @param field
   * @return
   * @throws UnableToLookupResourceException
   */
  public static String getXPath(String category, String field) throws UnableToLookupResourceException {
    List<Object> list = getXPaths();
    XPath xPath;
    String[] fields = null;
    
    if (field != null && field.indexOf(".") > -1) {
      fields = field.split("\\.");
    
      if (fields != null) {
        if (fields.length == 2) {
          Iterator<Object> iter = list.iterator();
          while (iter.hasNext()) {
            xPath = (XPath) iter.next();
            
            if (xPath.getCode().matches(category + "\\[\\w+\\]\\.\\(\\w+\\)" + fields[0])) {
              return removeLast(xPath.getDescription()) + "/" + getXPath(getChildCategory(xPath.getCode()), fields[1]);
            }
          }
        }
      }
    } else {
      Iterator<Object> iter = list.iterator();
      while (iter.hasNext()) {
        xPath = (XPath) iter.next();
        if (xPath.getCode().matches(category + "\\[\\w+\\]\\.\\(\\w+\\)" + field)) {
          return xPath.getDescription();
        } else if (xPath.getCode().matches(category + "\\[\\w+\\]\\." + field)) {
          return xPath.getDescription();
        }
      }
    }
    
    return null;
  }
    
  private XPath(String code, String description) {
    this.code = code;
    this.description = description;
    this.number = ++count;
  }

  /**
   * Xpath is in the format transaction.fieldname. This will strip the transaction and return the fieldname
   * @return
   */
  public String getField() {
    String code = this.getCode();
    String field = code.substring(code.indexOf(".") + 1);
    return field;
  }
  
  /**
   * @return
   */
  public String getRoot() {
    String desc = this.getDescription();
    String root = desc.substring(0, desc.indexOf("/"));
    return root;
  }
  
  /**
   * @return
   */
  public String getValue() {
    String desc = this.getDescription();
    String value = null;
    if (desc != null) {
      value = getValue(desc);
    }
    return value;
  }
  
  /**
   * Xpath is in the format transaction.fieldname. This will strip the transaction and return the fieldname
   * @return
   */
  public static String getFieldName(String field) {
    return field.substring(field.indexOf(")") + 1);
  }
}

