/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/utils/B2BBDOMHelper.java-arc   1.19   Apr 11 2005 16:26:30   adij  $
 * $Revision:   1.19  $
 * $Date:   Apr 11 2005 16:26:30  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */
 /**
  *                     Modification History
  *                     ====================
  *  Date Changed      Changed By     Comments
  *  ------------      ----------     ------------------------------------------
  *  June 12 2006      NG             Fixed defect 3947. Modified public static String nodeToString(Node node, String omitXMLdeclaration)
  *  
**/
package com.capgemini.nemmco.b2bb.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.capgemini.nemmco.b2bb.enums.FileType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToParseXMLException;
import com.capgemini.nemmco.b2bb.utils.XPath;

/**
 * Define the B2BBDOMHelper
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.19  $ $Date:   Apr 11 2005 16:26:30  $
 */
public class B2BBDOMHelper {
  public B2BBDOMHelper() {
  }
  
  private static Element root;
  
  static {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);   
    DocumentBuilder builder;
    try {
      builder = factory.newDocumentBuilder();
      DOMImplementation impl = builder.getDOMImplementation();
      Document namespaceHolder = impl.createDocument(
          "http://www.w3.org/2001/XMLSchema-instance", 
          "ase:namespaceMapping", null);
      root = namespaceHolder.getDocumentElement();
      root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:ase",
      		"http://www.w3.org/2001/XMLSchema-instance");
    } catch (ParserConfigurationException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    
  }
  
  public static Document parseZipEntry(String path, String zipEntryName) throws Exception {
    if (path != null) {
      File file;
      file = new File(path);
      ZipFile zip = null;
      ZipEntry zipEntry = null;
      FileInputStream fis = null;
      InputSource is = null;
      
      if (path.toLowerCase().endsWith(FileType.ZIP.getCode())) {
        try {
          zip = new ZipFile(file);
         
          if (zipEntryName != null) {
            zipEntry = zip.getEntry(zipEntryName);
            if (zipEntryName.endsWith(FileType.ZIP.getCode())) {
              ZipInputStream zis = new ZipInputStream(zip.getInputStream(zipEntry));
              
              if (zis.getNextEntry() != null) {
                is = new InputSource(zis);
              }
            } else {
              is = new InputSource(zip.getInputStream(zipEntry));
            }
          }
            
          if (is != null) {
            return parseXML(is);
          } else {
            // error
            throw new Exception("Error creating Input Source");
          }
        } finally {
          try {
            if (zip != null) {
              zip.close();
            }
            if (fis != null) {
              fis.close();
            }
          } catch (Exception e) {
            throw new Exception("Error closing input streams: " + e.getMessage());
          }
        }
      } 
    }
    
    return null;
  }
  
  public static Document parseFile(String path) throws Exception {
    if (path != null) {
      File file;
      file = new File(path);
      FileInputStream fis = null;
      InputSource is = null;
      
      try {
        if (path.toLowerCase().endsWith(FileType.ZIP.getCode())) {
          fis = new FileInputStream(file);
          ZipInputStream zis = new ZipInputStream(fis);
          
          if (zis.getNextEntry() != null) {
            is = new InputSource(zis);
          }
            
        } else {
          fis = new FileInputStream(file);
          is = new InputSource(fis);
        }
	      
	    if (is != null) {
	      return parseXML(is);
        } else {
          // error
          throw new Exception("Error creating Input Source");
        }
	      
      } finally {
        try {
          if (fis != null) {
            fis.close();
          }
        } catch (Exception e) {
          throw new Exception("Error closing input streams: " + e.getMessage());
        }
      }
    }
    
    return null;
  }

  public static Document parseXML(String xml) throws SAXException, IOException {
    //String a = "<Transaction transactionID=\"TXUN-TXUR-001\" transactionDate=\"2002-01-01T12:00:00+10:00\"><CustomerDetailsNotification version=\"r11\"><Customers><Customer><NMI checksum=\"9\">12345</NMI><ServicePoint><Address updateStatus=\"New\"><AustralianAddress><StructuredAddress><FlatOrUnit><FlatOrUnitType>APT</FlatOrUnitType><FlatOrUnitNumber>A</FlatOrUnitNumber></FlatOrUnit><FloorOrLevel><FloorOrLevelType>B</FloorOrLevelType><FloorOrLevelNumber>A</FloorOrLevelNumber></FloorOrLevel><BuildingOrPropertyName>String</BuildingOrPropertyName><LocationDescriptor>A</LocationDescriptor><House><HouseNumber>99999</HouseNumber><HouseNumberSuffix>A</HouseNumberSuffix></House><Lot><LotNumber>A</LotNumber></Lot><Street><StreetName>A</StreetName><StreetType>ACCS</StreetType><StreetSuffix>CN</StreetSuffix></Street><PostalDelivery><PostalDeliveryType>CARE PO</PostalDeliveryType><PostalDeliveryNumber><PostalDeliveryNumberPrefix>A</PostalDeliveryNumberPrefix><PostalDeliveryNumberValue>99999</PostalDeliveryNumberValue><PostalDeliveryNumberSuffix>A</PostalDeliveryNumberSuffix></PostalDeliveryNumber></PostalDelivery></StructuredAddress><SuburbOrPlaceOrLocality>String</SuburbOrPlaceOrLocality><StateOrTerritory>AAT</StateOrTerritory><PostCode>0000</PostCode><DeliveryPointIdentifier>99999999</DeliveryPointIdentifier></AustralianAddress></Address></ServicePoint></Customer></Customers></CustomerDetailsNotification></Transaction>";
    DOMParser parser = new DOMParser();
    //parser.parse(xml);    
    parser.setFeature("http://xml.org/sax/features/namespaces", false);
    parser.setFeature("http://xml.org/sax/features/validation", false);
    parser.setFeature("http://apache.org/xml/features/validation/schema", false);
    parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", false);
    parser.parse(new InputSource(new StringReader(xml)));
    Document document = parser.getDocument();
    return document;
  }
  
  public static Document parseXML(InputSource is) throws SAXException, IOException {
    DOMParser parser = new DOMParser();
   
    parser.setFeature("http://xml.org/sax/features/namespaces", false);
    parser.setFeature("http://xml.org/sax/features/validation", false);
    parser.setFeature("http://apache.org/xml/features/validation/schema", false);
    parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", false);
    
    parser.parse(is);
    Document document = parser.getDocument();
    return document;
  }
  
  public static Document parseXML(InputSource is, ErrorHandler errorHandler, String schemaLocation, boolean validate) throws SAXException, IOException {
    DOMParser parser = new DOMParser();
   
    parser.setFeature("http://xml.org/sax/features/namespaces", validate);
    parser.setFeature("http://xml.org/sax/features/validation", validate);
    parser.setFeature("http://apache.org/xml/features/validation/schema", validate);
    parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", validate);
    
    parser.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", schemaLocation);
    
    parser.setErrorHandler(errorHandler);
    
    parser.parse(is);
    Document document = parser.getDocument();
    return document;
  }

  /**
   * Will append the first child in AppendFrom to the first child in appendTo
   * 
   * @param appendTo Changes this document
   * @param AppendFrom gets first node from this document
   */
  public static void appendDocuments(Document appendTo, Document AppendFrom) {
    Element appendFromElement = AppendFrom.getDocumentElement();

    // Import the node into the right document
    Element readyToUseElement = (Element) appendTo.importNode(appendFromElement, true);
    appendTo.getFirstChild().appendChild(readyToUseElement);
  }
  
  /**
   * Will append the nodes from childNode to the parentNode where
   * both are from two different documents
   * 
   * @param parentNode
   * @param childNode
   */
  public static void appendNode(Node parentNode, Node childNode) {
    Document doc = parentNode.getOwnerDocument();
    Node toAdd = doc.importNode(childNode, true);
    
    parentNode.appendChild(toAdd);
  }

  /**
   * converts the input document to a XML string
   * @param document Document to convert to String
   * @return Document in String format
   * @throws TransformerConfigurationException
   * @throws TransformerException
   */
  public static String domToString(Document document) throws TransformerConfigurationException, TransformerException {
    return nodeToString(document);
  }
  
  /**
   * Converts the input node to an XML string
   * @param node Node to convert to String
   * @return Node in String format
   * @throws TransformerConfigurationException
   * @throws TransformerException
   */
  public static String nodeToString(Node node) throws TransformerConfigurationException, TransformerException {
    return nodeToString(node, "no");
  }
  
  /**
   * Converts the input node to an XML string
   * @param node Node to convert to String
   * @return Node in String format
   * @throws TransformerConfigurationException
   * @throws TransformerException
   */
  public static String nodeToString(Node node, String omitXMLdeclaration) throws TransformerConfigurationException, TransformerException {
    Source src = new DOMSource(node);
    StringWriter str = new StringWriter();
    Result result = new StreamResult(str);
    Transformer transformer = TransformerFactory.newInstance().newTransformer();
    transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, omitXMLdeclaration);
    transformer.setOutputProperty(OutputKeys.METHOD, "xml");
    transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
    //Defect 3947 - NG - Commented out the following as it gives run time error in 10gas - no impact otherwise
    //transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
    transformer.setOutputProperty(OutputKeys.INDENT, "yes");
    transformer.transform(src, result);
     return StringEscapeUtils.unescapeXml(str.toString());
  }

  /**
   * Creates a new Document with the specified root node
   * 
   * @param rootNodeName root node name
   * @return Document with one element (the root element) with name as specified in the input
   */
  public static Document createEmptyDocument(String rootNodeName) throws ParserConfigurationException {
    Document parentNode = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
    Element element = parentNode.createElement(rootNodeName);
    parentNode.appendChild(element);
    return parentNode;

  }

  /**
   * Creates a new Node with the specified root node
   * 
   * @param rootNodeName root node name
   * @return Document with one element (the root element) with name as specified in the input
   */
  public static Node createEmptyNode(String rootNodeName) throws ParserConfigurationException {
    Document document = createEmptyDocument(rootNodeName);
    return document.getFirstChild();
  }

  public static String getSelectedNodeValue(Node rootNode, String xPath) throws TransformerException, UnableToParseXMLException {
    Node selectedNode = XPathAPI.selectSingleNode(rootNode, xPath);

    if (selectedNode == null) {
      return "";
    }
    String nodeValue = selectedNode.getNodeValue();

    if (selectedNode.getNodeType() == Node.ELEMENT_NODE) {
      NodeList children = selectedNode.getChildNodes();
      if (children.getLength() < 1) {
        nodeValue = "";
      } else {
        nodeValue = children.item(0).getNodeValue();
      }
    }
    if (nodeValue == null || nodeValue.equals("null")) {
      nodeValue = "";
    }

    return nodeValue;
  }

  public static Node getSelectedNode(Node rootNode, String xPath) throws TransformerException, UnableToParseXMLException {
    Node selectedNode = XPathAPI.selectSingleNode(rootNode, xPath);
    return selectedNode;
  }
  
  public static NodeList getSelectedNodes(Node rootNode, String xPath) throws TransformerException, UnableToParseXMLException {
    NodeList selectedNodes = XPathAPI.selectNodeList(rootNode, xPath);
    return selectedNodes;
  }

  public static void setElementValue(Document document, String xpath, String value) throws TransformerException {
    NodeList nodeList = XPathAPI.selectNodeList(document, xpath.toString());

    // Check if element already exists
    if (nodeList.getLength() > 0) {
      replaceText(document, (Element) nodeList.item(0), value);
    } else {
      XPath xp = new XPath(xpath);

      Element element = (Element) createNode(document, xp);
      replaceText(document, element, value);
    }
  }

  public static void setNodeValue(Document document, String xpath, String value) throws TransformerException {
    XPath xp = new XPath(xpath);
    short type = xp.getType();

    if (type == Node.ATTRIBUTE_NODE) {
      setAttributeValue(document, xp.toString(), value);
    } else if (type == Node.ELEMENT_NODE) {
      setElementValue(document, xp.toString(), value);
    }
  }

  public static void setAttributeValue(Document document, String xpath, String value) throws TransformerException {
    NodeList nodeList = null;
    
    if (xpath.lastIndexOf(":") <= -1) {
      nodeList = XPathAPI.selectNodeList(document, xpath);
    }
    
    // Check if attribute already exists
    if (nodeList != null && nodeList.getLength() > 0) {
      Attr attribute = (Attr) nodeList.item(0);
      attribute.setValue(value);
    } else {
      XPath xp = new XPath(xpath);
      Attr attribute = (Attr) createNode(document, xp);
      attribute.setValue(value);
    }
  }

  public static Node createNode(Document document, String xpath) throws TransformerException {
    XPath xp = new XPath(xpath);
    return createNode(document, xp);
  }

  private static Node createNode(Document document, XPath xpath) throws TransformerException {
    Node node = null;
    NodeList nodeList = null;
    
    if (xpath.getName().equals("ase:aseXML")) {
      nodeList = document.getChildNodes();
      
    } else if (xpath.toString() == ""
      	|| xpath.getType() != Node.ATTRIBUTE_NODE 
      	|| xpath.toString().lastIndexOf(":") <= -1) {
      nodeList = XPathAPI.selectNodeList(document, xpath.toString(), root);
    }
    
    // Check if node already exists
    if (nodeList != null && nodeList.getLength() > 0) {
      node = nodeList.item(0);
    } else {
      Node parentNode = createNode(document, xpath.getParent());

      if (xpath.getType() == Node.ATTRIBUTE_NODE) {
        String namespace = null;
        if (xpath.getNameWithoutPredicates().indexOf("xmlns") > -1) {
          namespace = "http://www.w3.org/2000/xmlns/";
        } else {
          namespace = "null";
        }
        ((Element) parentNode).setAttributeNS(namespace, xpath.getNameWithoutPredicates(), "null");
        node = ((Element) parentNode).getAttributeNode(xpath.getNameWithoutPredicates());
      } else {
        node = document.createElement(xpath.getNameWithoutPredicates());
        parentNode.appendChild(node);
      }
    }

    return node;
  }
  
  public static Node createNode(Node parentNode, String nodeName) throws TransformerException {
    Document doc = parentNode.getOwnerDocument();
    Node node = doc.createElement(nodeName);
    
    return parentNode.appendChild(node);
  }

  private static void replaceText(Document document, Element element, String text) {
    NodeList nl = element.getChildNodes();

    for (int i = nl.getLength() - 1; i >= 0; i--) {
      element.removeChild(nl.item(i));
    }

    element.appendChild(document.createTextNode(text));
  }
}

