/*
 * B2BDateTimeFormatter.java
 *
 * The Copyright of this work is vested in AEMO and the software is issued
 * in confidence for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of AEMO
 * and then only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * National Electricity Market Management Company Limited.
 *
 * Copyright AEMO 2009
 *
 * Modification History
 *
 * Date Changed   Changed By  Description
 * ------------   ----------  -----------
 * 11-Apr-2005    RayanS      Created
 * 05 Apr 2007    W.Mills     d3775 Set lenient parsing to false
 * 29 May 2007    W.Mills     d3775 Ensure parse consumes whole string, trap exceptions from DateDV
 * 26-Oct-2009    W.Mills     d5120: JDK1.6 Remove use of internal Xerces classes
 */

package com.capgemini.nemmco.b2bb.utils;

import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.SimpleTimeZone;

/**
 * Defines a database lookup enumeration for Nmis
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.7  $ $Date:   11 Apr 2005 11:22:04  $
 */
public class B2BDateTimeFormatter {
  public String B2BDateTimePattern = "d-MMM-yyyy HH:mm:ss";
  public String B2BDatePattern = "d-MMM-yyyy";
  public String B2BTimePattern = "HH:mm:ss";
  public String xmlDateTimePattern = "yyyy-MM-dd'T'HH:mm:ss";
  public String xmlDatePattern = "yyyy-MM-dd";
  public String xmlTimePattern = "HH:mm:ss";
  public static final int B2BDATE = 1;
  public static final int XMLDATE = 2;

  private SimpleDateFormat formatter;
  private SimpleTimeZone tz = new SimpleTimeZone(36000000, "MSATS");
  private java.util.Date dt;
  private int dateType = 0;

  /**
   * Default constructor
   */
  public B2BDateTimeFormatter() {
    dt = new java.util.Date();
  }
  
  /**
   * Constructor with long param
   */
  public B2BDateTimeFormatter(long time) {
    dt = new java.util.Date(time);
  }

  /**
   * Constructor which accepts a String date
   * which is either an XML or an B2B date.
   *
   * @param date the Date
   * @param DateType THe type of the date
   *
   */
  public B2BDateTimeFormatter(String date, int DateType) throws ParseException 
  {
    if (date==null || date.equals("")) throw new ParseException("Empty date time provided",0);
    if (DateType == B2BDATE) {
      formatter = new SimpleDateFormat(B2BDateTimePattern + "Z");
      // d3775 WM Set lenient parsing to false
      formatter.setLenient(false);
      // d3775 WM Ensure parse consumes whole string
      ParsePosition pp = new ParsePosition(0);
      dt = formatter.parse(date+"+1000",pp);
      if (dt == null || pp.getIndex() < (date="+1000").length())
        throw new ParseException("Invalid date time provided",0);
      xmlDateTimePattern += ".SZ";
      xmlTimePattern += ".SZ";
      dateType = B2BDATE;
    }
     // assume its an XML Date format, yyyy-MM-dd'T'HH:mm:ss with optional timezone
     // d5120: JDK1.6 Remove use of internal Xerces classes
     // Iterate through all acceptable patterns and throw ParseException if none fit.
    else {
       String[] patterns = {
        "yyyy-MM-dd'T'HH:mm:ss.SSSZ", "yyyy-MM-dd'T'HH:mm:ss.SSSz", "yyyy-MM-dd'T'HH:mm:ss.SSS",
        "yyyy-MM-dd'T'HH:mm:ssZ",     "yyyy-MM-dd'T'HH:mm:ssz",     "yyyy-MM-dd'T'HH:mm:ss"
       };
       formatter = new SimpleDateFormat(patterns[0]);
       formatter.setLenient(false);
       for (int i=0; i<patterns.length; i++) {
          try {
             formatter.applyPattern(patterns[i]);
             dt = formatter.parse(date);
             break;
          } catch (ParseException pe) {
             if ((i+1) >= patterns.length) {
                throw pe;
             }
          }
       }
      dateType = XMLDATE;
    }
  }

  /**
   * Return B2B Formatted date
   *
   * @return The B2B Formatted date
   *
   */
  public String getB2BFormattedDateTime() {
    formatter = new SimpleDateFormat(B2BDateTimePattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return XML Formatted date
   *
   * @return The XML Formatted date
   *
   */
  public String getXMLFormattedDateTime() {
    formatter = new SimpleDateFormat(xmlDateTimePattern);
    formatter.setTimeZone(tz);
    String formatedTime = formatter.format(dt);
    // for XML the GMT offset format is +XX:XX. In Java it is +XXXX
    formatedTime = formatedTime.substring(0, formatedTime.length() - 2) + ":00";
    return formatedTime;
  }

  /**
   * Return the Formatted date given by Pattern
   *
   * @return The Pattern Formatted date
   *
   */
  public String getPatternFormattedDate(String pattern) {
    formatter = new SimpleDateFormat(pattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return a java.util.date object
   *
   * @return The java.util.Date Date
   *
   */
  public java.util.Date getDate() {
    return dt;
  }

  /**
   * Return the date format of the original date provided to this class
   *
   * @return int representing the date type
   *
   */
  public int getDateType() {
    return dateType;
  }

  /**
   * Return the Date in B2B Format
   *
   */
  public String getB2BDate() {
    formatter = new SimpleDateFormat(B2BDatePattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return the Time in B2B Format
   *
   */
  public String getB2BTime() {
    formatter = new SimpleDateFormat(B2BTimePattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return the Date in XML Format
   *
   */
  public String getXMLDate() {
    formatter = new SimpleDateFormat(xmlDatePattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return the Time in XML Format
   *
   */
  public String getXMLTime() {
    formatter = new SimpleDateFormat(xmlTimePattern);
    formatter.setTimeZone(tz);
    String formatedTime = formatter.format(dt);
    // for XML the GMT offset format is +XX:XX. In Java it is +XXXX
    formatedTime = formatedTime.substring(0, formatedTime.length() - 2) + ":00";
    return formatedTime;
  }
}

