/*
 * B2BTimeFormatter.java
 *
 * The Copyright of this work is vested in AEMO and the software is issued
 * in confidence for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of AEMO
 * and then only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * National Electricity Market Management Company Limited.
 *
 * Copyright AEMO 2009
 *
 * Modification History
 *
 * Date Changed   Changed By  Description
 * ------------   ----------  -----------
 * 11-Apr-2005    RayanS      Created
 * 05 Apr 2007    W.Mills     d3775 Set lenient parsing to false
 * 29 May 2007    W.Mills     d3775 Ensure parse consumes whole string, trap exceptions from DateDV
 * 26-Oct-2009    W.Mills     d5120: JDK1.6 Remove use of internal Xerces classes
 * 31-Jul-2017    H McNeish   PoC extended XML Time to support +10:00 (HH:mm:ss.SSSX)
 * 31-Aug-2017    Harpreet    Removed : PoC extended XML Time to support +10:00 (HH:mm:ss.SSSX) as requested by Hamish and since it is not creating any impact
 */

package com.capgemini.nemmco.b2bb.utils;

import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;

import java.util.GregorianCalendar;
import java.util.SimpleTimeZone;

/**
 * Defines a database lookup enumeration for Nmis
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.7  $ $Date:   11 Apr 2005 11:22:04  $
 */

public class B2BTimeFormatter {
  public String B2BTimePattern = "HH:mm:ss";
  public String xmlTimePattern = "HH:mm:ss";
  public static final int B2BTIME = 1;
  public static final int XMLTIME = 2;

  private SimpleDateFormat formatter;    
  private SimpleTimeZone tz = new SimpleTimeZone(36000000,"MSATS");
  private java.util.Date dt;
  private int dateType = 0;

  /**
   * Default constructor
   */
  public B2BTimeFormatter()
  {
    dt = new java.util.Date();
    formatter.setTimeZone(tz);
  }

  /**
   * Constructor which accepts a String date
   * which is either an XML or an B2B date.
   *
   * @param date the Date
   * @param DateType THe type of the date
   *
   */
  public B2BTimeFormatter(String date, int DateType) throws ParseException
  {  
    if (date==null || date.equals("")) throw new ParseException("Empty time provided",0);
    if (DateType == B2BTIME) {
      formatter = new SimpleDateFormat(B2BTimePattern+"Z");
      // d3775 WM Set lenient parsing to false
      formatter.setLenient(false);
      // d3775 WM Ensure parse consumes whole string
      ParsePosition pp = new ParsePosition(0);
      dt = formatter.parse(date+"+1000",pp);
      if (dt == null)
        throw new ParseException("Invalid time provided",0);
      else if (pp.getIndex() < (date+"+1000").length())
        // check if remainder of string is a valid dateTime
        new B2BDateTimeFormatter((date+"+1000"), DateType);
      // assume no milli seconds
      xmlTimePattern+=".SZ";
      dateType = B2BTIME;
    }
    // assume its an XML Time format, HH:mm:ss with optional date and timezone components
    // d5120: JDK1.6 Remove use of internal Xerces classes
    // Iterate through all acceptable patterns and throw ParseException if none fit.
    else {
       String[] patterns = {
        "yyyy-MM-dd'T'HH:mm:ss.SSSZ", "yyyy-MM-dd'T'HH:mm:ss.SSSz", "yyyy-MM-dd'T'HH:mm:ss.SSS",
        "yyyy-MM-dd'T'HH:mm:ssZ",     "yyyy-MM-dd'T'HH:mm:ssz",     "yyyy-MM-dd'T'HH:mm:ss",
        "yyyy-MM-ddZ",                "yyyy-MM-ddz",                "yyyy-MM-dd",
        "HH:mm:ss.SSSZ",              "HH:mm:ss.SSSz",              "HH:mm:ss.SSS",
		"HH:mm:ssZ",                  "HH:mm:ssz",                  "HH:mm:ss"
       };
       formatter = new SimpleDateFormat(patterns[0]);
       formatter.setLenient(false);
       for (int i=0; i<patterns.length; i++) {
          try {
             formatter.applyPattern(patterns[i]);
             dt = formatter.parse(date);
             break;
          } catch (ParseException pe) {
             if ((i+1) >= patterns.length) {
                throw pe;
             }
          }
       }
       GregorianCalendar gc = new GregorianCalendar();
       gc.setTime(dt);
       gc.set(2000,1,15);
      dateType = XMLTIME;
    }
  }

  /**
   * Return the Formatted date given by Pattern
   *
   * @return The Pattern Formatted date
   *
   */
  public String getPatternFormattedTime(String pattern){
    formatter = new SimpleDateFormat(pattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }

  /**
   * Return a java.util.date object
   *
   * @return The java.util.Date Date
   *
   */
  public java.util.Date getDate(){
    return dt;
  }

  /**
   * Return the date format of the original date provided to this class
   *
   * @return int representing the date type
   *
   */
  public int getTimeType(){
    return dateType;
  }

  /**
   * Return the Time in B2B Format
   *
   */
  public String getB2BTime(){
    formatter = new SimpleDateFormat(B2BTimePattern);
    formatter.setTimeZone(tz);
    return formatter.format(dt);
  }
  /**
   * Return the Time in XML Format
   *
   */
  public String getXMLTime(){
    formatter = new SimpleDateFormat(xmlTimePattern);
    formatter.setTimeZone(tz);
    String formatedTime = formatter.format(dt);   
    // for XML the GMT offset format is +XX:XX. In Java it is +XXXX
    formatedTime = formatedTime.substring(0,formatedTime.length()-2)+":00";
    
    return formatedTime;
  }
}

