/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/MeterDataVerifyRequestBasicValidator.java-arc   1.23   Sep 06 2005 17:31:34   adij  $
 * $Revision:   1.23  $
 * $Date:   Sep 06 2005 17:31:34  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 *
 * Defect 3603 : 09-11-05 : NG : Removed 3355 validation from validateCurrentRead(). This was to allow only some investigation codes for current read
 * Defect 3584 : 17-11-05 : EG : Defect 3584 Validation of NMI Suffix should not allow entry of spaces.
 * Defect 3630 : 01-12-05 : VR : Removed mandatory field validation for Current Read for all investigation codes
 * Defect 3713 : 24-03-06 : BJ : Removed requirement that NMISuffix and RegisterSuffix cannot coexist.
 * Defect 3703 : 29-03-06 : EG : Add mandatory field validation for Current Read if Current Read Date or Current Consumption exist.
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.MeterDataVerifyRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.InvestigationCode;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;

/**
 * The class <code>MeterDataVerifyRequestBasicValidator</code> is the Basic Validator 
 * for a Verify Meter Data Request transaction.
 */
public class MeterDataVerifyRequestBasicValidator extends Validator {

  protected MeterDataVerifyRequest transaction;

  /**
   * Constructor
   * @param trans 
   */
  public MeterDataVerifyRequestBasicValidator(Transaction trans) {
    super(trans);
    this.transaction = (MeterDataVerifyRequest) trans;
  }

  /**
   * Validate this transaction
   * @return boolean
   */
  public boolean validate() {
    boolean result;

    result = validateTransactionID()
           & validateTransactionDate()
           & validateNMI() 
           & validateNMIChecksum()
           // Bob Jones 24/03/2006 3713: & validateSuffixCoexistance()
           & validateNmiSuffixMandatory() 
           & validateNmiSuffix1() 
           & validateNmiSuffix2()
           & validateNmiSuffix3()
           & validateNmiSuffix4()
           & validateNmiSuffix5()
           & validateNmiSuffix6() 
           & validateNmiSuffix7()
           & validateNmiSuffix8()
           & validateNmiSuffix9()
           & validateNmiSuffix10()
           & validateMeterSerial() 
           & validateRegisterId()
           & validateSuffix()
           & validateCurrentRead() 
           & validateCurrentReadDate() 
           & validateCurrentReadConsumption() 
           & validateInvestigationCode()
           & validateInvestigationDescription1() 
           & validateInvestigationDescription2() 
           & validateInvestigationDescription3() 
           & validateStartReadDate() 
           & validateEndReadDate();

    return result;
  }

  /**
   * Validate to ensure that only a nmi or register suffix exists and they dont co exist
   * @return boolean
   */  
  /* Bob Jones 24/03/2006 3713: Remove requirement that NMISuffix and RegisterSuffix cannot coexist
  protected boolean validateSuffixCoexistance()
  {
    boolean result;
    String fieldname = MeterDataVerifyRequest.NMI_SUFFIX_FIELD+"1"; 
    
    result = true;
    if (nmiSuffixExists() && registerSuffixExists())
    {
      result = false;
      try
      {
        Object[] args = {"NMI Suffix", "Suffix"};
        validator.addEvent(events, category, fieldname, args, EventCode.CODE_0201, "concurrentError");
      }
      catch (UnableToLookupResourceException e)
      {        
        validator.unexpectedException(events, fieldname,e.getMessage());
      }
    }
    return result;
  }
  */
  
  /**
   * Validate current read
   * @return boolean
   */
  protected boolean validateCurrentRead() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.CURRENT_READ_FIELD;
    String investigationCode = transaction.getInvestigationCode();
    String value = transaction.getCurrentRead();

    result = true;

    // Defect 3630 - Removed mandatory population of this field for specific investigation codes
//    if (validator.exists(transaction.getCurrentReadDate()) || validator.exists(transaction.getCurrentConsumption()) ||
//        investigationCode.equals(InvestigationCode.CONFIRM_READ_FROM_VACANT_SITE.getCode()) || investigationCode.equals(InvestigationCode.VERIFY_HIGH_READING.getCode())
//        || investigationCode.equals(InvestigationCode.VERIFY_LOW_READING.getCode()) || investigationCode.equals(InvestigationCode.CONFIRM_ZERO_CONSUMPTION.getCode())) {
//      result = validator.validateMandatory(events, category, fieldname, value) && validator.validateLengthRange(events, category, fieldname, value, 1, 15);
//    }
//    else 
    // 29Mar2006 Eddie Graham Defect 3703 - Add mandatory field validation for Current Read if Current Read Date or Current Consumption exist
    if (validator.exists(transaction.getCurrentReadDate()) || validator.exists(transaction.getCurrentConsumption())) {
      result = validator.validateMandatory(events, category, fieldname, value) && validator.validateLengthRange(events, category, fieldname, value, 1, 15);
    }
    else if (validator.exists(value)) {
      result = validator.validateLengthRange(events, category, fieldname, value, 1, 15);     
    }
    return result;
  }

  /**
   * Validate current read consumption
   * @return boolean
   */
  protected boolean validateCurrentReadConsumption() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.CURRENT_CONSUMPTION_FIELD;
    String value = transaction.getCurrentConsumption();

    result = true;
    if (validator.exists(transaction.getCurrentRead()) || validator.exists(transaction.getCurrentReadDate())) {
      result = validator.validateMandatory(events, category, fieldname, value) && validator.validateNumericDecimal(events, category, fieldname, value, 18, 3);
    }
    return result;
  }

  /**
   * Validate current read date
   * @return boolean
   */
  protected boolean validateCurrentReadDate() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.CURRENT_READ_DATE_FIELD;
    String value = transaction.getCurrentReadDate();

    result = true;
    if (validator.exists(transaction.getCurrentRead()) || validator.exists(transaction.getCurrentRead()) || validator.exists(transaction.getCurrentConsumption())) {
      result = validator.validateMandatory(events, category, fieldname, value) && validator.validateDate(events, category, fieldname, value, transaction.getDateFormat());
    }
    return result;
  }

  /**
   * Validate end read date
   * @return boolean
   */
  protected boolean validateEndReadDate() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.END_READ_DATE_FIELD;
    String endDate = transaction.getEndReadDate();
    String startDate = transaction.getStartReadDate();
    // Defect 3352 this is an optional field
    if (!validator.exists(endDate)) return true;
    result = validator.validateDate(events, category, fieldname, endDate, transaction.getDateFormat()) &&
             validator.validateDateRange(events, category, fieldname, startDate, transaction.getDateFormat(), endDate, transaction.getDateFormat());
    return result;
  }

  /**
   * Validate investigation code
   * @return boolean
   */  
  protected boolean validateInvestigationCode() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.INVESTIGATION_CODE_FIELD;
    String value = transaction.getInvestigationCode();

    result = validator.validateMandatory(events, category, fieldname, value)
        && validator.validateEnumeratedValue(events, category, fieldname, "Investigation Code", InvestigationCode.INVESTIGATION_CODES, value);
    return result;
  }

  /**
   * Validate investigation description
   * @return boolean
   */
  protected boolean validateInvestigationDescription1() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.INVESTIGATION_DESCRIPTION_FIELD1;
    String value = transaction.getInvestigationDescription1();
    result = validator.validateMandatory(events, category, fieldname, value) && validator.validateLengthRange(events, category, fieldname, value, 1, 80);
    return result;
  }
  /**
   * Validate investigation description
   * @return boolean
   */
  protected boolean validateInvestigationDescription2() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.INVESTIGATION_DESCRIPTION_FIELD2;
    String value = transaction.getInvestigationDescription2();
    result = validator.validateLengthRange(events, category, fieldname, value, 0, 80);
    return result;
  }
  /**
   * Validate investigation description
   * @return boolean
   */
  protected boolean validateInvestigationDescription3() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.INVESTIGATION_DESCRIPTION_FIELD3;
    String value = transaction.getInvestigationDescription3();
    result = validator.validateLengthRange(events, category, fieldname, value, 0, 80);
    return result;
  }
  /**
   * Validate meter serial
   * @return boolean
   */
  protected boolean validateMeterSerial() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.METER_SERIAL_FIELD;
    String value = transaction.getMeterSerial();
    
    result = true;
    if (validator.exists(value)) {
      result = validator.validateLengthRange(events, category, fieldname, value, 1, 12);
    }
    return result;
  }

  /**
   * Validate nmi configuration
   * @return boolean
   */
  protected boolean validateNmiConfiguration() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.NMI_CONFIGURATION_FIELD;
    String value = transaction.getNmiConfiguration();
    result = true;
    if (validator.exists(value))
    {
      result = validator.validateLengthRange(events, category, fieldname, value, 1, 240);
    }
    return result;
  }

  /**
   * Validate the mandatory condition for nmi suffix. ie. if current read exists then at least
   * one nmi suffix must also exists.
   * @return boolean
   */
  protected boolean validateNmiSuffixMandatory()
  {
    boolean result;
    String fieldname = MeterDataVerifyRequest.SUFFIX_FIELD;

    result = true;
    if (validator.exists(transaction.getCurrentRead()) && !(nmiSuffixExists()||registerSuffixExists())) 
    {
      Object[] args = { fieldname };      
      try
      {
        validator.addEvent(events, category, fieldname, args, EventCode.CODE_0201, "mandatoryMissing");
      }
      catch (Exception e)
      {
        validator.unexpectedException(events, fieldname, e.getMessage());
      }
      result = false;
    }
    return result;
  }

  /**
   * Validate nmi suffix
   * @return boolean
   */
  protected boolean validateNmiSuffix(String value, int index) {
    boolean result;
    String fieldname = MeterDataVerifyRequest.NMI_SUFFIX_FIELD + index;

    result = true;
    //Defect 3584 : 17-11-05 : EG : Defect 3584 Validation of NMI Suffix should not allow entry of spaces.    
    //if (validator.exists(value))
    if (value!=null && !value.equals(""))
    {
      //result = validator.validateLength(events, category, fieldname, value, 2);
      result = validator.validateAlphaNumeric(events, category, fieldname, value) &&
               validator.validateLength(events, category, fieldname, value, 2);
    }
    return result;
  }

  /**
   * Validate nmi suffix1
   * @return boolean
   */
  protected boolean validateNmiSuffix1()
  {
    return validateNmiSuffix(transaction.getNmiSuffix1(), 1);
  }

  /**
   * Validate nmi suffix2
   * @return boolean
   */
  protected boolean validateNmiSuffix2()
  {
    return validateNmiSuffix(transaction.getNmiSuffix2(), 2);
  }

  /**
   * Validate nmi suffix3
   * @return boolean
   */  
  protected boolean validateNmiSuffix3()
  {
    return validateNmiSuffix(transaction.getNmiSuffix3(), 3);
  }

  /**
   * Validate nmi suffix4
   * @return boolean
   */  
  protected boolean validateNmiSuffix4()
  {
    return validateNmiSuffix(transaction.getNmiSuffix4(), 4);
  }

  /**
   * Validate nmi suffix5
   * @return boolean
   */  
  protected boolean validateNmiSuffix5()
  {
    return validateNmiSuffix(transaction.getNmiSuffix5(), 5);
  }  

  /**
   * Validate nmi suffix6
   * @return boolean
   */  
  protected boolean validateNmiSuffix6()
  {
    return validateNmiSuffix(transaction.getNmiSuffix6(), 6);
  }

  /**
   * Validate nmi suffix7
   * @return boolean
   */
  protected boolean validateNmiSuffix7()
  {
    return validateNmiSuffix(transaction.getNmiSuffix7(), 7);
  }

  /**
   * Validate nmi suffix8
   * @return boolean
   */  
  protected boolean validateNmiSuffix8()
  {
    return validateNmiSuffix(transaction.getNmiSuffix8(), 8);
  }

  /**
   * Validate nmi suffix9
   * @return boolean
   */  
  protected boolean validateNmiSuffix9()
  {
    return validateNmiSuffix(transaction.getNmiSuffix9(), 9);
  }

  /**
   * Validate nmi suffix10
   * @return boolean
   */  
  protected boolean validateNmiSuffix10()
  {
    return validateNmiSuffix(transaction.getNmiSuffix10(), 10);
  } 

  /**
   * Validate suffix
   * @return boolean
   */
  protected boolean validateSuffix() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.SUFFIX_FIELD;
    String value = transaction.getSuffix();

    result = true;
    if (validator.exists(value))
    {
      result = validator.validateLength(events, category, fieldname, value, 2);
    }
    return result;
  }

  /**
   * Validate register id
   * @return boolean
   */
  protected boolean validateRegisterId() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.REGISTER_ID_FIELD;
    String value = transaction.getRegisterId();

    result = true;
    if (validator.exists(value)) 
    {
      result = validator.validateLengthRange(events, category, fieldname, value, 1, 10);
    }
    return result;
  }

  /**
   * Validate start read date
   * @return boolean
   */
  protected boolean validateStartReadDate() {
    boolean result;
    String fieldname = MeterDataVerifyRequest.START_READ_DATE_FIELD;
    String value = transaction.getStartReadDate();

    result = validator.validateMandatory(events, category, fieldname, value) && validator.validateDate(events, category, fieldname, value, transaction.getDateFormat());
    return result;
  }

  /**
   * Check to see if any nmi suffix is provided
   * @return boolean
   */
  protected boolean nmiSuffixExists()
  {
    return validator.exists(transaction.getNmiSuffix1()) ||
           validator.exists(transaction.getNmiSuffix2()) ||
           validator.exists(transaction.getNmiSuffix3()) ||
           validator.exists(transaction.getNmiSuffix4()) ||
           validator.exists(transaction.getNmiSuffix5()) ||
           validator.exists(transaction.getNmiSuffix6()) ||
           validator.exists(transaction.getNmiSuffix7()) ||
           validator.exists(transaction.getNmiSuffix8()) ||
           validator.exists(transaction.getNmiSuffix9()) ||
           validator.exists(transaction.getNmiSuffix10());
  }
  
  protected boolean registerSuffixExists()
  {
    return validator.exists(transaction.getSuffix());
  }
}

