/*
 * NoticeOfMeteringWorksBasicValidator.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By       Description
 * ------------   ----------       -----------
 * 11-Jul-2017    SP               Created as part of the SMP new screens
 * 13-Nov-2017    Hamish           Fix for defect 8454
 */
package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.NoticeOfMeteringWorks;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.ControlEquipmentType;
import com.capgemini.nemmco.b2bb.enums.CustomerClassCode;
import com.capgemini.nemmco.b2bb.enums.CustomerType;
import com.capgemini.nemmco.b2bb.enums.EnergisationStatus;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.GenerationType;
import com.capgemini.nemmco.b2bb.enums.NetWorkDeviceLocation;
import com.capgemini.nemmco.b2bb.enums.RemovedEquipmentType;
import com.capgemini.nemmco.b2bb.enums.SORRole;
import com.capgemini.nemmco.b2bb.enums.SupplyPhase;
import com.capgemini.nemmco.b2bb.enums.TransformerType;
import com.capgemini.nemmco.b2bb.enums.Voltage;
import com.capgemini.nemmco.b2bb.enums.WorkType;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

import java.util.ArrayList;
import java.util.List;

/**
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 12-Jul-2017    SP                SMP work pakg7b- new screens
 * 26-Sep-2017   Harpreet Kaur     7135 Package 7B: Defect#7980 fix
 * 09-Oct-2017   Harpreet Kaur     7135 Package 7B: Defect#7980 fix
 * 17-Oct-2017    Hamish McNeish    8159 B2B Errata make NMI checksum optional, 8219 Fix ValidationModule returning false with no error
 */

public class NoticeOfMeteringWorksBasicValidator extends Validator {

   protected NoticeOfMeteringWorks transaction;

   public NoticeOfMeteringWorksBasicValidator(Transaction trans) {
      super(trans);
     this.transaction = (NoticeOfMeteringWorks) trans;
   }

   @Override
   public boolean validate() {
      return validateTransactionID() &
      validateTransactionDate() &
      validateNomwID()&
      validateNMI()&
      validateNMIChecksum() &
      validateWorkType()&
      validateClassCode()&
      validateEnergisationStatus()&
      validatePrimVoltage()&
      validateLatitude() &
      validateLongitude() &
      validateParticipantID()&
	  validateParticipantRole()&
      validateWorkDateTime()&
      validateInstalledMeters() &
      validateMeterSerialNumber() &
	  validateSupplyPhase() &
      validateGeneralSupply() &
      validateGenType()&
      validateControlledLoad()&
      validateInstalledNetworkDevices() &
      validateNetworkDeviceNumber() &
      validateNetworkDeviceLocation()&
      validateControlConnectedNumber()&
      validateControlEquipmentType()&
      validateControlConnectedNumber() &
      validateControlEquipmentChannel() &
      validateTransformerNumber() &
      validateTransformerType() &
      validateTransformerRatio() &
      validateTransformerConnectedNumber() &
      validateRemovedEquipmentNumber()&
      validateRemovedEquipmentType()&
       validateRemovedRegister()&
      validateRemovedMeterReading() &
      validateNotes()  &
       validateRemovedMeters() &
      validateRemovedOthers();
   }

   /**
    * Validate nomW ID
    * @return boolean
    */
   protected boolean validateNomwID(){
      String fieldName = NoticeOfMeteringWorks.NOMWID;
      return validator.validateMandatory(events, category, fieldName, transaction.getNoMWID())
      		&& validator.validateLengthRange(events, category, fieldName, transaction.getNoMWID(),1, 12);

   }

   /**
    * Validate Field Work date and time
    * @return boolean
    */
   protected boolean validateWorkDateTime()
   {
     boolean result;
     String dateFieldname = NoticeOfMeteringWorks.WORK_DATE_TIME_FIELD;
     if (transaction.getDateFormat() == B2BDateFormatter.B2BDATE)
     {
       result  = validator.validateMandatory(events, category, dateFieldname, transaction.getFieldworkDate()) &&
                 validator.validateMandatory(events, category, dateFieldname, transaction.getFieldworkTime()) &&
                 validator.validateB2BDateTime(events, category, dateFieldname, transaction.getFieldworkDate() + " " + transaction.getFieldworkTime());
     }
     else
     {

       result  = validator.validateXMLDateTime(events, category, dateFieldname, transaction.getWorkDateTime());
     }
     return result;
   }

   /**
    * Validate Work Type
    * @return boolean
    */
   protected boolean validateWorkType(){
      String fieldName = NoticeOfMeteringWorks.WORK_TYPE;
      if(validator.exists(transaction.getWorkType())){
          return validator.validateEnumeratedValue(events,category,fieldName,"Work Type",WorkType.WORK_TYPES,transaction.getWorkType());
      }
      return validator.validateMandatory(events, category, fieldName, transaction.getWorkType());
   }


   /**
    * Validate Customer Classification Code
    * @return boolean
    */
   protected boolean validateClassCode(){
      String fieldName = NoticeOfMeteringWorks.CUST_CLASS_CODE;
      if(validator.exists(transaction.getCustClassCode())){
         return validator.validateEnumeratedValue(events,category,fieldName,"Customer Classifaction Code", CustomerClassCode.CUSTOMER_CLASS_CODES, transaction.getCustClassCode());
      }
      return validator.validateMandatory(events, category, fieldName, transaction.getCustClassCode());
   }

   /**
    * Validate primary voltage
    * @return boolean
    */
   protected boolean validatePrimVoltage(){
      String fieldName = NoticeOfMeteringWorks.PRIMARY_VOLTAGE;
      if(validator.exists(transaction.getPrimVol())  ){
          return validator.validateEnumeratedValue(events,category,fieldName,"Voltage",Voltage.VOLTAGES,transaction.getPrimVol());

      }
      return validator.validateMandatory(events, category, fieldName, transaction.getPrimVol());

   }

   /**
    * Validate Energisation Status
    * @return boolean
    */
   protected boolean validateEnergisationStatus(){
      String fieldName = NoticeOfMeteringWorks.ENERGISATION_STATUS;
      if(validator.exists(transaction.getEnergStatus())  ){
          return validator.validateEnumeratedValue(events,category,fieldName,"Energisation Status",EnergisationStatus.ENERGISATION_STATUSES ,transaction.getEnergStatus()) && validator.validateLengthRange(events, category, fieldName, transaction.getEnergStatus(),1, 50) ;

      }
      return validator.validateMandatory(events, category, fieldName, transaction.getEnergStatus());

   }

   /**
    * Validate Latitude
    * @return boolean
    */
   protected boolean validateLatitude(){
      String fieldName = NoticeOfMeteringWorks.LATITUDE;
      return !validator.exists(transaction.getLatitude())
      		|| validator.validateNumericDecimal2(events, category, fieldName, transaction.getLatitude(), "^(\\[?|\\+|-| ]?)([\\d]{1,2})(\\.)([\\d]{1,7})");
   }

   /**
    * Validate Longitude
    * @return boolean
    */
   protected boolean validateLongitude(){
      String fieldName = NoticeOfMeteringWorks.LONGITUDE;
      return !validator.exists(transaction.getLongitude())
      		|| validator.validateNumericDecimal2(events, category, fieldName, transaction.getLongitude(), "^(\\[?|\\+|-| ]?)([\\d]{1,3})(\\.)([\\d]{1,7})");
   }


   /**
    * Validate Participant ID
    * @return boolean
    */
   protected boolean validateParticipantID(){
      String fieldName = NoticeOfMeteringWorks.PARTICPANT_ID;
      return validator.validateMandatory(events, category, fieldName, transaction.getPartId())
      		&& validator.validateLengthRange(events, category, fieldName, transaction.getPartId(),1,10);

   }


   /**
    * Validate InstalledMeter
    * @return boolean
    */
   protected boolean validateInstalledMeters(){
      String fieldName = NoticeOfMeteringWorks.TOTAL_INSTALLED_METERS;
      return validator.validateMandatory(events, category, fieldName, transaction.getTotInstalledMeters())
      		&& validator.validateNumericDecimal(events, category, fieldName, transaction.getTotInstalledMeters(),2,0);

   }


   /**
    * Validate Meter Serial Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateMeterSerialNumer(String fieldname, String value){
     return validator.validateLengthRange(events, category, fieldname, value, 0, 12);
   }
   /**
    * Validate Meter Serial Number
    * @return boolean
    */
   protected boolean validateMeterSerialNumber(){
  	 //Optional, could check against number installed...
  	 return validateMeterSerialNumer(NoticeOfMeteringWorks.METER_SERIAL_NUMBER1, transaction.getMeterSerialNumber1())
  			 & validateMeterSerialNumer(NoticeOfMeteringWorks.METER_SERIAL_NUMBER2, transaction.getMeterSerialNumber2())
  			 & validateMeterSerialNumer(NoticeOfMeteringWorks.METER_SERIAL_NUMBER3, transaction.getMeterSerialNumber3());
   }

   /**
    * Validate Supply Phase
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateSupplyPhase(String fieldname, String value, String meterFieldname, String meterValue){
      //Mandatory when a matching Meter Serial Number exists
  	  if (validator.exists(meterValue) && !validator.validateMandatory(events, category, fieldname, value)) {
  	  	return false;
  	  }
  	  //Supply Phase can only exist if a matching Meter Serial Number exists
      if(validator.exists(value)){
    	  if (!validator.validateOneWayDependancy(events, category, fieldname, meterFieldname, value, meterValue)) {
          return false;
        }
         List<SupplyPhase> tempList = new ArrayList<SupplyPhase>();
         tempList.add(SupplyPhase.ONE_PHASE);
         tempList.add(SupplyPhase.TWO_PHASE);
         tempList.add(SupplyPhase.THREE_PHASE);
         tempList.add(SupplyPhase.OTHER_MULTIPHASE);

         return validator.validateEnumeratedValue(events,category,fieldname,"Supply Phases", tempList,value);

      }
      return true;

   }
   /**
    * Validate Supply Phase
    * @return boolean
    */
   protected boolean validateSupplyPhase(){
  	 return validateSupplyPhase(NoticeOfMeteringWorks.SUPPLY_PHASE1, transaction.getSupPhase1(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER1, transaction.getMeterSerialNumber1())
  			 & validateSupplyPhase(NoticeOfMeteringWorks.SUPPLY_PHASE2, transaction.getSupPhase2(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER2, transaction.getMeterSerialNumber2())
  			 & validateSupplyPhase(NoticeOfMeteringWorks.SUPPLY_PHASE3, transaction.getSupPhase3(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER3, transaction.getMeterSerialNumber3());
   }


   /**
    * Validate general supply
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateGeneralSupply(String fieldname, String value, String meterFieldname, String meterValue){
     //Mandatory when a matching Meter Serial Number exists
 	   if (validator.exists(meterValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //General Supply can only exist if a matching Meter Serial Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, meterFieldname, value, meterValue)
             && validator.validateEnumeratedValue(events, category, fieldname, "YesNo",YesNo.YESNO_CODES,value));
   }
   /**
    * Validate general supply
    * @return boolean
    */
   protected boolean validateGeneralSupply(){
  	 return validateGeneralSupply(NoticeOfMeteringWorks.GENERAL_SUPPLY1, transaction.getGenSup1(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER1, transaction.getMeterSerialNumber1())
  			 & validateGeneralSupply(NoticeOfMeteringWorks.GENERAL_SUPPLY2, transaction.getGenSup2(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER2, transaction.getMeterSerialNumber2())
  			 & validateGeneralSupply(NoticeOfMeteringWorks.GENERAL_SUPPLY3, transaction.getGenSup3(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER3, transaction.getMeterSerialNumber3());
   }

   /**
    * Validate controlled Load
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateControlledLoad(String fieldname, String value, String meterFieldname, String meterValue){
     //Mandatory when a matching Meter Serial Number exists
 	   if (validator.exists(meterValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Controlled Load can only exist if a matching Meter Serial Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, meterFieldname, value, meterValue)
             && validator.validateEnumeratedValue(events, category, fieldname, "YesNo",YesNo.YESNO_CODES,value));
   }
   /**
    * Validate controlled Load
    * @return boolean
    */
   protected boolean validateControlledLoad(){
     return validateControlledLoad(NoticeOfMeteringWorks.CONTROL_LOAD1, transaction.getConLod1(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER1, transaction.getMeterSerialNumber1())
    		 & validateControlledLoad(NoticeOfMeteringWorks.CONTROL_LOAD2, transaction.getConLod2(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER2, transaction.getMeterSerialNumber2())
    		 & validateControlledLoad(NoticeOfMeteringWorks.CONTROL_LOAD3, transaction.getConLod3(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER3, transaction.getMeterSerialNumber3());
   }

   /**
    * Validate generation type
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateGenType(String fieldname, String value, String meterFieldname, String meterValue){
     //Mandatory when a matching Meter Serial Number exists
 	   if (validator.exists(meterValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Gen Type can only exist if a matching Meter Serial Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, meterFieldname, value, meterValue)
             && validator.validateEnumeratedValue(events, category, fieldname, "GenerationType",GenerationType.GENERATION_TYPES,value));
   }
   /**
    * Validate generation type
    * @return boolean
    */
   protected boolean validateGenType(){
     return validateGenType(NoticeOfMeteringWorks.GENERATION_TYPE1, transaction.getGenType1(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER1, transaction.getMeterSerialNumber1())
    		 & validateGenType(NoticeOfMeteringWorks.GENERATION_TYPE2, transaction.getGenType2(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER2, transaction.getMeterSerialNumber2())
    		 & validateGenType(NoticeOfMeteringWorks.GENERATION_TYPE3, transaction.getGenType3(), NoticeOfMeteringWorks.METER_SERIAL_NUMBER3, transaction.getMeterSerialNumber3());
   }

   /**
    * Validate Installed network devices
    * @return boolean
    */
   protected boolean validateInstalledNetworkDevices(){
      String fieldName = NoticeOfMeteringWorks.TOTAL_NETWORK_DEVICES;
      return validator.validateMandatory(events, category, fieldName, transaction.getTotNetworkDevices())
      		&& validator.validateNumericDecimal(events, category, fieldName, transaction.getTotNetworkDevices(),2,0);
   }

   /**
    * Validate Network Device Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateNetworkDeviceNumber(String fieldname, String value){
      return validator.validateLengthRange(events, category, fieldname, value,0,12);
   }
   /**
    * Validate Network Device Number
    * @return boolean
    */
   protected boolean validateNetworkDeviceNumber(){
  	 return validateNetworkDeviceNumber(NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER1, transaction.getNetworkDevNumber1())
  			 & validateNetworkDeviceNumber(NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER2, transaction.getNetworkDevNumber2())
  			 & validateNetworkDeviceNumber(NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER3, transaction.getNetworkDevNumber3());
   }

   /**
    * Validate Network Device Location
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateNetworkDeviceLocation(String fieldname, String value, String networkDeviceFieldname, String networkDeviceValue){
     //Mandatory when a matching Network Device Number exists
 	   if (validator.exists(networkDeviceValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Location can only exist if a matching Network Device Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, networkDeviceFieldname, value, networkDeviceValue)
             && validator.validateEnumeratedValue(events, category, fieldname, "NetWorkDeviceLocation",NetWorkDeviceLocation.NETWORK_DEVICES,value));
   }
   /**
    * Validate Network Device Location
    * @return boolean
    */
   protected boolean validateNetworkDeviceLocation(){
  	 return validateNetworkDeviceLocation(NoticeOfMeteringWorks.NETWORK_DEVICE_LOC1, transaction.getNetworkDevLoc1(), NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER1, transaction.getNetworkDevNumber1())
  			 & validateNetworkDeviceLocation(NoticeOfMeteringWorks.NETWORK_DEVICE_LOC2, transaction.getNetworkDevLoc2(), NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER2, transaction.getNetworkDevNumber2())
  			 & validateNetworkDeviceLocation(NoticeOfMeteringWorks.NETWORK_DEVICE_LOC3, transaction.getNetworkDevLoc3(), NoticeOfMeteringWorks.NETWORK_DEVICE_NUMBER3, transaction.getNetworkDevNumber3());
   }


   /**
    * Validate Control Equipment Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateControlEquipmentNumber(String fieldname, String value){
     return validator.validateLengthRange(events, category, fieldname, value,0, 12);
   }
   /**
    * Validate Control Equipment Number
    * @return boolean
    */
   protected boolean validateControlEquipmentNumber(){
  	 return validateControlEquipmentNumber(NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER1, transaction.getConEquipNumber1())
  			 & validateControlEquipmentNumber(NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER2, transaction.getConEquipNumber2());
   }

   /**
    * Validate Control Equipment Type
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateControlEquipmentType(String fieldname, String value, String controlEquipmentNumberFieldname, String controlEquipmentNumberValue){
     //Control Equipment Type can only exist if a matching Control Equipment Number exists
 	   if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, controlEquipmentNumberFieldname, value, controlEquipmentNumberValue)) {
       return false;
     }

     if(validator.exists(value)){
         return validator.validateEnumeratedValue2(events, category, fieldname, "ControlEquipmentType",ControlEquipmentType.CONTROL_EQUIPMENT_TYPES,value);
     }
     return true;
   }

   /**
    * Validate Control Equipment Type
    * @return boolean
    */
   protected boolean validateControlEquipmentType(){
  	 return validateControlEquipmentType(NoticeOfMeteringWorks.CONTROL_EQUIPMENT_TYPE1, transaction.getConEquipType1(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER1, transaction.getConEquipNumber1())
  			 & validateControlEquipmentType(NoticeOfMeteringWorks.CONTROL_EQUIPMENT_TYPE2, transaction.getConEquipType2(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER2, transaction.getConEquipNumber2());
   }



   /**
   * Validate Control Channel
   *@param fieldname,  value
   * @return boolean
   */
   protected boolean validateControlEquipmentChannel(String fieldname, String value, String controlEquipmentNumberFieldname, String controlEquipmentNumberValue){
     //Control Equipment Channel can only exist if a matching Control Equipment Number exists
 	   if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, controlEquipmentNumberFieldname, value, controlEquipmentNumberValue)) {
       return false;
     }
     return validator.validateLengthRange(events, category, fieldname, value,0, 12);

   }
   /**
   * Validate Control Channel
   * @return boolean
   */
   protected boolean validateControlEquipmentChannel(){
  	 return validateControlEquipmentChannel(NoticeOfMeteringWorks.CONTROL_CHANNEL1, transaction.getConChannel1(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER1, transaction.getConEquipNumber1())
  			 & validateControlEquipmentChannel(NoticeOfMeteringWorks.CONTROL_CHANNEL2, transaction.getConChannel2(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER2, transaction.getConEquipNumber2());
   }

   /**
    * Validate Control Connected Number
    *@param fieldname,  value
    * @return boolean
    */
   protected boolean validateControlConnectedNumber(String fieldname, String value, String controlEquipmentNumberFieldname, String controlEquipmentNumberValue){
     //Control Connected Number can only exist if a matching Control Equipment Number exists
 	   if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, controlEquipmentNumberFieldname, value, controlEquipmentNumberValue)) {
       return false;
     }
     return validator.validateLengthRange(events, category, fieldname, value,0, 12);
   }
   /**
    * Validate Control Connected Number
    * @return boolean
    */
   protected boolean validateControlConnectedNumber(){
  	 return validateControlConnectedNumber(NoticeOfMeteringWorks.CONTROL_CONNEC_METER1, transaction.getConMeterNum1(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER1, transaction.getConEquipNumber1())
  			 & validateControlConnectedNumber(NoticeOfMeteringWorks.CONTROL_CONNEC_METER2, transaction.getConMeterNum2(), NoticeOfMeteringWorks.CONTROL_EQUIPMENT_NUMBER2, transaction.getConEquipNumber2());
   }

   /**
    * Validate Transformer Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateTransformerNumber(String fieldname, String value){
     return validator.validateLengthRange(events, category, fieldname,value,0,12);
   }

   /**
    * Validate Transformer Number
    * @return boolean
    */
   protected boolean validateTransformerNumber(){
  	 return validateTransformerNumber(NoticeOfMeteringWorks.TRANSFORMER_NUMBER1, transaction.getTransNumber1())
  			 & validateTransformerNumber(NoticeOfMeteringWorks.TRANSFORMER_NUMBER2, transaction.getTransNumber2());
   }

   /**
    * Validate Transformer Ratio
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateTransformerRatio(String fieldname, String value, String transformerNumberFieldname, String transformerNumberValue){
     //Mandatory when a matching Transformer Number exists
 	   if (validator.exists(transformerNumberValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Can only exist if a matching Transformer Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, transformerNumberFieldname, value, transformerNumberValue)
 	  		     && validator.validateLengthRange(events, category, fieldname,value,0,20));
   }

   /**
    * Validate Transformer Ratio
    * @return boolean
    */
   protected boolean validateTransformerRatio(){
  	 return validateTransformerRatio(NoticeOfMeteringWorks.TRANSFORMER_RATIO1, transaction.getTransRatio1(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER1, transaction.getTransNumber1())
  			 & validateTransformerRatio(NoticeOfMeteringWorks.TRANSFORMER_RATIO2, transaction.getTransRatio2(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER2, transaction.getTransNumber2());
   }


   /**
    * Validate Transformer Connected Meter Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateTransformerConnectedNumber(String fieldname, String value, String transformerNumberFieldname, String transformerNumberValue){
     //Can only exist if a matching Transformer Number exists
 	   if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, transformerNumberFieldname, value, transformerNumberValue)) {
       return false;
     }
 	   return validator.validateLengthRange(events, category, fieldname,value,0,20);
   }
   /**
    * Validate Transformer Connected Meter Number
    * @return boolean
    */
   protected boolean validateTransformerConnectedNumber(){
  	 return validateTransformerConnectedNumber(NoticeOfMeteringWorks.TRANSFORMER_CONN_NUMBER1, transaction.getTransConNumber1(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER1, transaction.getTransNumber1())
  			 & validateTransformerConnectedNumber(NoticeOfMeteringWorks.TRANSFORMER_CONN_NUMBER2, transaction.getTransConNumber2(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER2, transaction.getTransNumber2());
   }

   /**
    * Validate Transformer Type
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateTransformerType(String fieldname, String value, String transformerNumberFieldname, String transformerNumberValue){
     //Mandatory when a matching Transformer Number exists
 	   if (validator.exists(transformerNumberValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Can only exist if a matching Transformer Number exists
 	   return !validator.exists(value)
 	  		 || (validator.validateOneWayDependancy(events, category, fieldname, transformerNumberFieldname, value, transformerNumberValue)
             && validator.validateEnumeratedValue(events, category, fieldname, "TransformerType",TransformerType.TRANSFORMER_TYPES,value));
   }
   /**
    * Validate Transformer Type
    * @return boolean
    */
   protected boolean validateTransformerType(){
  	 return validateTransformerType(NoticeOfMeteringWorks.TRANSFORMER_TYPE1, transaction.getTransType1(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER1, transaction.getTransNumber1())
  			 & validateTransformerType(NoticeOfMeteringWorks.TRANSFORMER_TYPE2, transaction.getTransType2(), NoticeOfMeteringWorks.TRANSFORMER_NUMBER2, transaction.getTransNumber2());
   }

      /**
    * Validate Removed Equipment Number
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateRemovedEquipmentNumber(String fieldname, String value){
     return validator.validateLengthRange(events, category, fieldname,value,0,12);
   }
   /**
    * Validate Removed Equipment Number
    * @return boolean
    */
   protected boolean validateRemovedEquipmentNumber(){
  	 return validateRemovedEquipmentNumber(NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER1,transaction.getRemovedEquipNumber1())
  			 & validateRemovedEquipmentNumber(NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER2,transaction.getRemovedEquipNumber2());
   }

   /**
    * Validate Removed Equipment Type
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateRemovedEquipmentType(String fieldname, String value, String removedEquipmentFieldname, String removedEquipmentValue){
	 //Mandatory when a matching Removed Equipment Number exists
 	   if (validator.exists(removedEquipmentValue) && !validator.validateMandatory(events, category, fieldname, value)) {
 	  	 return false;
 	   }
     //Can only exist if a matching Removed Equipment Number exists
 	   return !validator.exists(value) || (
 	  		 validator.validateOneWayDependancy(events, category, fieldname, removedEquipmentFieldname, value, removedEquipmentValue)
 	  		 && validator.validateEnumeratedValue(events, category, fieldname, "RemovedEquipmentType",RemovedEquipmentType.REMOVED_EQUIPMENT_TYPES,value));
   }
   /**
    * Validate Removed Equipment Type
    * @return boolean
    */
   protected boolean validateRemovedEquipmentType(){
  	 return validateRemovedEquipmentType(NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE1,transaction.getRemovedEquipType1(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER1,transaction.getRemovedEquipNumber1())
  			 & validateRemovedEquipmentType(NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE2,transaction.getRemovedEquipType2(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER2,transaction.getRemovedEquipNumber2());
   }

   /**
    * Validate Removed Register
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateRemovedRegister(String fieldname, String value, String removedEquipmentFieldname, String removedEquipmentValue, String removedEquipmentTypeFieldname, String removedEquipmentTypeValue){
	  //Mandatory when Removed Equipment Type is Basic Meter
      if(!validator.validateMandatoryConditionally1(events, category, fieldname, value, removedEquipmentTypeFieldname, removedEquipmentTypeValue, RemovedEquipmentType.BASIC_METER.getCode())){
      	return false;
      }
      //Can only exist if a matching Removed Equipment Number exists
      if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, removedEquipmentFieldname, value, removedEquipmentValue)) {
        return false;
      }
      return validator.validateLengthRange(events, category, fieldname, value,0, 10);
   }
   /**
    * Validate Removed  Register
    * @return boolean
    */
   protected boolean validateRemovedRegister(){
  	 return validateRemovedRegister(NoticeOfMeteringWorks.REMOVED_REGISTER1,transaction.getRemovedReg1(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER1,transaction.getRemovedEquipNumber1(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE1, transaction.getRemovedEquipType1())
  			 & validateRemovedRegister(NoticeOfMeteringWorks.REMOVED_REGISTER2,transaction.getRemovedReg2(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_NUMBER2,transaction.getRemovedEquipNumber2(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE2, transaction.getRemovedEquipType2());
   }

      /**
    * Validate Removed Meter Reading
    * @param fieldname,  value
    * @return boolean
    */
   protected boolean validateRemovedMeterReading(String fieldname, String value, String removedEquipmentTypeFieldname, String removedEquipmentType){
	  //Mandatory when Removed Equipment Type is Basic Meter
	 if(!validator.validateMandatoryConditionally1(events, category, fieldname, value, removedEquipmentTypeFieldname, removedEquipmentType, RemovedEquipmentType.BASIC_METER.getCode())){
         return false;
      }
     //Can only exist if a matching Removed Equipment Number exists
     if (validator.exists(value) && !validator.validateOneWayDependancy(events, category, fieldname, removedEquipmentTypeFieldname, value, removedEquipmentType)) {
       return false;
     }
      return validator.validateLengthRange(events, category, fieldname, value,0, 15);
   }
   /**
    * Validate Removed  Meter Reading
    * @return boolean
    */
   protected boolean validateRemovedMeterReading(){
  	 return validateRemovedMeterReading(NoticeOfMeteringWorks.REMOVED_METER_READING1,transaction.getRemovedMeterRead1(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE1,transaction.getRemovedEquipType1())
  			 & validateRemovedMeterReading(NoticeOfMeteringWorks.REMOVED_METER_READING2,transaction.getRemovedMeterRead2(), NoticeOfMeteringWorks.REMOVED_EQUIPMENT_TYPE2,transaction.getRemovedEquipType2());
   }

   /**
    * Validate Total removed Meters
    * @return boolean
    */

   protected boolean validateRemovedMeters(){
      String fieldName = NoticeOfMeteringWorks.TOTAL_REMOVED_METERS;
      if(validator.exists(transaction.getRemovedMeter())){
         return validator.validateNumeric(events, category, fieldName, transaction.getRemovedMeter())
        		 && validator.validateLengthRange(events, category, fieldName, transaction.getRemovedMeter(),0, 2);
      }
    return true;

   }

   /**
    * Validate Total removed Others
    * @return boolean
    */

   protected boolean validateRemovedOthers(){
      String fieldName = NoticeOfMeteringWorks.TOTAL_REMOVED_OTHERS;
      if(validator.exists(transaction.getRemovedOther())){
         return validator.validateNumeric(events, category, fieldName, transaction.getRemovedOther())
        		 && validator.validateLengthRange(events, category, fieldName, transaction.getRemovedOther(),0, 2);
      }
    return true;

   }
   /**
    * Validate notes
    * @return boolean
    */
   protected boolean validateNotes(){
      String fieldname =NoticeOfMeteringWorks.NOTES;
      return (validator.validateLengthRange(events, category, fieldname, transaction.getNotes(),0, 240));
   }

   /**
    * Validate notes
    * @return boolean
    */
   protected boolean validateParticipantRole(){
      String fieldname =NoticeOfMeteringWorks.PART_ROLE;
	  String value = transaction.getPartRole();
	  return validator.validateMandatory(events, category, fieldname, value)
	    && validator.validateAllowedValue(events, category, fieldname, value, SORRole.MPB.getCode());
   }

}
