/*
 * OneWayNotificationBasicValidator.java
 *
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the 
 * National Electricity Market Management Company Limited.
 *
 * Copyright National Electricity Market Management Company Limited. 2009
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 23-Jan-2009    Eddie Graham      Created.
 * 18-Mar-2011    Eddie Graham      Defect 7087. Modify method validateName() to
 *	                                   a. Remove call to method validateAlphaNumeric().
 *	                                   b. Remove call to method validateLengthRange().
 *	                                   c. Add call to method validateOneWayNotificationName().
 */
  
package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.OneWayNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;

public class OneWayNotificationBasicValidator extends Validator
{
  protected OneWayNotification transaction;

  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * 
   * @param transaction Transaction to use in the validator.
   */
  public OneWayNotificationBasicValidator(Transaction transaction) {
    super(transaction);
    this.transaction = (OneWayNotification) transaction;
  }

  /**
   * Validate the transaction
   *
   * @return boolean result of validation
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID()
        & validateTransactionDate()
        & validateName()
        & validateCSVNotificationDetail();

    return result;
  }

   /**
    * Validate CSV Notification Detai
    *
    * @return boolean result of validation
   */
  protected boolean validateCSVNotificationDetail()
  {
    String field = OneWayNotification.CSV_NOTIFICATION_DETAIL;
    return validator.validateMandatory(events, category, field, transaction.getCSVNotificationDetail());
  }

   /**
    * Validate Name
    *
    * @return boolean result of validation
   */
  protected boolean validateName()
  {
    String field = OneWayNotification.NAME;
    return validator.validateMandatory(events, category, field, transaction.getName())
        && validator.validateOneWayNotificationName(events, category, field, transaction.getName());         
  }
}

