/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/PersonName.java-arc   1.17   Sep 15 2005 09:44:22   adij  $
 * $Revision:   1.17  $
 * $Date:   Sep 15 2005 09:44:22  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.ArrayList;
import java.util.List;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.NameType;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.enums.NameTitle;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.utils.EventObject;

/**
 * Defines Person Name
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.17  $ $Date:   Sep 15 2005 09:44:22  $
 */
public class PersonName extends XMLObject implements ValidatorInterface {
  public static String FAMILY_NAME = "familyName";
  public static String GIVEN_NAME = "givenName";
  public static String NAME_SUFFIX = "nameSuffix";
  public static String NAME_TITLE = "nameTitle";
  public static String NAME_TYPE = "nameType";
  public static String PERSONNAME_CODE = "PersonName";
  
  // As this class extends transaction an event list is required which is usually found in validator.
  private List<EventObject> events = new ArrayList<EventObject>();
  private String familyName = "";
  private String givenName = "";
  private String nameSuffix = "";
  private String nameTitle = "";
  private String nameType = "";
  private String parentCategory;
  private String parentField;

  private BasicValidationUtility validator;

  public PersonName() {
    super(XMLObjectType.PERSONNAME);
    validator = BasicValidationUtility.getBasicInstance();
  }
  
  public PersonName(XMLObjectType xmlObjectType) {
    super(xmlObjectType);
    validator = BasicValidationUtility.getBasicInstance();
  }

  public boolean fieldsExists() {
    return validator.exists(nameType) 
    		|| validator.exists(nameTitle) 
    		|| validator.exists(givenName) 
    		|| validator.exists(familyName)
        || validator.exists(nameSuffix);
  }

  /**
   * @see com.capgemini.nemmco.b2bb.validators.ValidatorInterface#getEvents()
   */
  public List<EventObject> getEvents() {
    return events;
  }

  /**
   * @return Returns the familyName.
   */
  public String getFamilyName() {
    return familyName;
  }

  /**
   * @return Returns the givenName.
   */
  public String getGivenName() {
    return givenName;
  }

  /**
   * @return Returns the nameSuffix.
   */
  public String getNameSuffix() {
    return nameSuffix;
  }

  /**
   * @return Returns the nameTitle.
   */
  public String getNameTitle() {
    return nameTitle;
  }

  public List<? extends Object> getNameTitles() {
    return NameTitle.NAME_TITLES;
  }

  /**
   * @return Returns the nameType.
   */
  public String getNameType() {
    return nameType;
  }

  public List<? extends Object> getNameTypes() {
    return NameType.NAME_TYPES;
  }

  /**
   * @param event 
   */
  public void setEvents(List<EventObject> event) {
    events = event;
  }

  /**
   * @param familyName The familyName to set.
   */
  public void setFamilyName(String familyName) {
    this.familyName = BasicValidationUtility.trim(familyName);
  }

  /**
   * @param givenName The givenName to set.
   */
  public void setGivenName(String givenName) {
    this.givenName = BasicValidationUtility.trim(givenName);
  }

  /**
   * @param nameSuffix The nameSuffix to set.
   */
  public void setNameSuffix(String nameSuffix) {
    this.nameSuffix = BasicValidationUtility.trim(nameSuffix);
  }

  /**
   * @param nameTitle The nameTitle to set.
   */
  public void setNameTitle(String nameTitle) {
    this.nameTitle = BasicValidationUtility.trim(nameTitle);
  }

  /**
   * @param nameType The nameType to set.
   */
  public void setNameType(String nameType) {
    this.nameType = BasicValidationUtility.trim(nameType);
  }

  /**
   * @param parentCategory The parentCategory to set.
   */
  public void setParentCategory(String parentCategory) {
    this.parentCategory = parentCategory;
  }

  /**
   * @param parentField The parentField to set.
   */
  public void setParentField(String parentField) {
    this.parentField = parentField;
  }

  public boolean validate() {
    boolean result = true;
    result = validateFamilyName() && result;
    result = validateGivenName() && result;
    result = validateNameSuffix() && result;
    result = validateNameTitle() && result;
    result = validateNameType() && result;
    return result;
  }

  protected boolean validateFamilyName() {
    String fieldname = parentField + "." + FAMILY_NAME;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getFamilyName());
    return tmpresult && validator.validateLengthRange(events, parentCategory, fieldname, getFamilyName(), 0, 40);
  }

  protected boolean validateGivenName() {
    String fieldname = parentField + "." + GIVEN_NAME;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getGivenName());
    return tmpresult && validator.validateLengthRange(events, parentCategory, fieldname, getGivenName(), 0, 40);
  }

  protected boolean validateNameSuffix() {
    String fieldname = parentField + "." + NAME_SUFFIX;
    if (getNameSuffix() != null && !getNameSuffix().equals("")) {
      return validator.validateLengthRange(events, parentCategory, fieldname, getNameSuffix(), 0, 12);
    }
    return true;
  }

  protected boolean validateNameTitle() {
    String fieldname = parentField + "." + NAME_TITLE;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getNameTitle());
    //(Justina Tran) defect 3511: Commented out the call to validator.validateNameTitle(events, parentCategory, fieldname, getNameTitle());
    // and added the call to validateLength().
    return tmpresult && validator.validateLengthRange(events, parentCategory, fieldname, getNameTitle(), 0, 12);

  }

  protected boolean validateNameType() {
    String fieldname = parentField + "." + NAME_TYPE;
    //HM: NameType has a default value of LGL in the schema
    return (!validator.exists(nameType) ||
    	    validator.validateNameType(events, parentCategory, fieldname, getNameType()));
  }
}

