/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/ProvideMeterDataRequestBasicValidator.java-arc   1.8   11 Apr 2005 13:40:00   hendyl  $
 * $Revision:   1.8  $
 * $Date:   11 Apr 2005 13:40:00  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.Date;

import com.capgemini.nemmco.b2bb.ProvideMeterDataRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;

/**
 * Basic validator for Provide Meter Data Request transaction
 *
 * @author $Author:   hendyl  $
 * @version $Revision:   1.8  $ $Date:   11 Apr 2005 13:40:00  $
 */
public class ProvideMeterDataRequestBasicValidator extends Validator {
  private ProvideMeterDataRequest transaction = null;
  private String transactionType = "";

  BasicValidationUtility validator = null;

  /**
   * Default constructor
   * @param trans The transaction to set
   */
  public ProvideMeterDataRequestBasicValidator(Transaction trans) {
    super(trans);
    transaction = (ProvideMeterDataRequest) trans;
    transactionType = transaction.getTransactionType().getCode();
    validator = BasicValidationUtility.getBasicInstance();
  }

  /**
   * 
   * @return Returns true if transaction is valid
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID() 
        & validateTransactionDate() 
        & validateNMI() 
        & validateNMIChecksum() 
        & validateStartReadDate() 
        & validateEndReadDate() 
        & validateParticipantRole();

    return result;
  }

  /**
   * 
   * @return Returns true if endReadDate is valid
   */
  protected boolean validateEndReadDate() {
    String fieldname = ProvideMeterDataRequest.END_READ_DATE;

    if (transaction.isEndReadDateSet()) {      
        if(transaction.getEndReadDate()!=null && !transaction.getEndReadDate().equals(""))
        {
          try {
            if (transaction.getStartReadDate() != null && !transaction.getStartReadDate().equals("")) {
              B2BDateFormatter df = new B2BDateFormatter(transaction.getEndReadDate(), B2BDateFormatter.XMLDATE);
    
              Date endDate = df.getDate();
              df = new B2BDateFormatter(transaction.getStartReadDate(), B2BDateFormatter.XMLDATE);
              Date startDate = df.getDate();
              if (startDate.after(endDate)) {
                Object[] args = { "End Date" };
                validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202, "invalidEndDateRange");
                return false;
              }
            }
          } catch (Exception e) {            
          } 
          return validator.validateXMLDate(events, category, fieldname, transaction.getEndReadDate());          
        }
      return true;      
    } else {

      if (transaction.getEndReadDisplayDate() != null && !transaction.getEndReadDisplayDate().equals("")) {
        try {
          if (transaction.getStartReadDisplayDate() != null && !transaction.getStartReadDisplayDate().equals("")) {
            B2BDateFormatter df = new B2BDateFormatter(transaction.getEndReadDisplayDate(), B2BDateFormatter.B2BDATE);

            Date endDate = df.getDate();
            df = new B2BDateFormatter(transaction.getStartReadDisplayDate(), B2BDateFormatter.B2BDATE);
            Date startDate = df.getDate();
            if (startDate.after(endDate)) {
              Object[] args = { "End Date" };
              validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202, "invalidEndDateRange");
              return false;
            }
          }
        } catch (Exception e) {
        }

        return validator.validateB2BDate(events, category, fieldname, transaction.getEndReadDisplayDate());
      }
      return true;
    }
  }

  /**
   * 
   * @return Returns true if participantRole is valid
   */
  protected boolean validateParticipantRole() {
    String fieldname = ProvideMeterDataRequest.PARTICIPANT_ROLE;
    return validator.validateMandatory(events, transactionType, fieldname, transaction.getParticipantRole())
    		&& validator.validateLengthRange(events, category, fieldname, transaction.getParticipantRole(), 1, 4);
  }

  /**
   * 
   * @return Returns true if startReadDate is valid
   */
  protected boolean validateStartReadDate() {
    String fieldname = ProvideMeterDataRequest.START_READ_DATE;
    if (transaction.isStartReadDateSet()) {

      return validator.validateMandatory(events, category, fieldname, transaction.getStartReadDate())
          && validator.validateXMLDate(events, category, fieldname, transaction.getStartReadDate());
    } else {
      boolean tempresult = validator.validateMandatory(events, category, fieldname, transaction.getStartReadDisplayDate());
      // if(transaction.getStartReadDisplayDate()!=null && !transaction.getEndReadDisplayDate().equals(""))
      // {
      tempresult = tempresult && validator.validateB2BDate(events, category, fieldname, transaction.getStartReadDisplayDate());
      //}      
      return tempresult;
    }
  }
}

