/**
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 19-Jun-2017    SP                SMP work pakg7b- new screens
 */
package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.RemoteServiceRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

import java.util.Date;

public class RemoteServiceRequestBasicValidator extends Validator {
   protected RemoteServiceRequest transaction;

   public RemoteServiceRequestBasicValidator(Transaction transaction) {
      super(transaction);
      this.transaction = (RemoteServiceRequest)transaction;
   }

   @Override
   public boolean validate() throws TransactionNotSetException {

       boolean result = validateTransactionID()&
               validateTransactionDate()&
               validateNMI()&
               validateNMIChecksum()&
               validateFromDateTime()&
               validateToDateTime()&
                validateRequestID() &
                validateRequestCode()&
               validateServiceType() &
               validateResponseFormat()&
          validateMeterSerialNumber() &
          validateUserDefined();

      return result;
   }

   /**
    * Validate from date and from time
    * @return boolean
    */
   protected boolean validateFromDateTime() {
       boolean tmpResult=true;
       String fieldFromDateTime = RemoteServiceRequest.FROM_DATETIME;
       String fieldDateDisplay = RemoteServiceRequest.DISPLAY_FROM_DATE;
       String fieldTimeDisplay = RemoteServiceRequest.DISPLAY_FROM_TIME;
       String valueDateDisplay = transaction.getFromDateDisplay();
       String valueTimeDisplay = transaction.getFromTimeDisplay();

       if((!validator.exists(valueDateDisplay) && validator.exists(valueTimeDisplay)) ||(validator.exists(valueDateDisplay) && !validator.exists(valueTimeDisplay) )){
          tmpResult = validator.validateDependancy(events, category, fieldDateDisplay, fieldTimeDisplay, valueDateDisplay,valueTimeDisplay);

       }else if(validator.exists(valueDateDisplay) && validator.exists(valueTimeDisplay)){
          tmpResult =  validator.validateXMLDateTime(events, category, fieldFromDateTime, transaction.getFromDateTime());
       }
         if(tmpResult && validator.exists(transaction.getToDateDisplay()) ){
                      B2BDateFormatter df;
                     try {
                        df = new B2BDateFormatter(transaction.getToDateDisplay(), B2BDateFormatter.B2BDATE);
                        Date toDate = df.getDate();
                        df = new B2BDateFormatter(transaction.getFromDateDisplay(), B2BDateFormatter.B2BDATE);
                        Date fromDate = df.getDate();
                        if (fromDate.after(toDate)) {
                              Object[] args = {transaction.getFromDateDisplay(),transaction.getToDateDisplay()};
                              validator.addEvent(events, category, fieldDateDisplay, args, EventCode.CODE_0202,"invalidDateRange");
                           tmpResult= false;

                        }
                     } catch (Exception e ) {
                      return false;
                     }
                   }
         return tmpResult;

      
     }
   /**
    * Validate to date and to time
    * @return boolean
    */
   protected boolean validateToDateTime() {
       boolean tmpResult=true;
       String fieldToDateTime = RemoteServiceRequest.TO_DATETIME;
       String fieldDateDisplay = RemoteServiceRequest.DISPLAY_TO_DATE;
       String fieldTimeDisplay = RemoteServiceRequest.DISPLAY_TO_TIME;
       String valueDateDisplay = transaction.getToDateDisplay();
       String valueTimeDisplay = transaction.getToTimeDisplay();

        if((!validator.exists(valueDateDisplay) && validator.exists(valueTimeDisplay)) ||(validator.exists(valueDateDisplay) && !validator.exists(valueTimeDisplay) )){
          tmpResult = validator.validateDependancy(events, category, fieldDateDisplay, fieldTimeDisplay, valueDateDisplay,valueTimeDisplay);
       }else if(validator.exists(valueDateDisplay) && validator.exists(valueTimeDisplay)){
          tmpResult =  validator.validateXMLDateTime(events, category, fieldToDateTime, transaction.getToDateTime());

       }

       return tmpResult;
     }

   /**
    * Validate Request ID
    * @return boolean
    */
   protected boolean validateRequestID(){
      String fieldname = RemoteServiceRequest.REQUEST_ID;
      return validator.validateMandatory(events, category, fieldname, transaction.getRequestId())
      		&& validator.validateLengthRange(events, category, fieldname, transaction.getRequestId(),1, 15);

   }

   /**
    * Validate Request Code
    * @return boolean
    */
   protected boolean validateRequestCode(){
      String fieldname = RemoteServiceRequest.REQUEST_CODE;
      return validator.validateMandatory(events, category, fieldname, transaction.getRequestCode())
      		&& validator.validateLengthRange(events, category, fieldname, transaction.getRequestCode(),1, 40);
   }

   /**
    * Validate Service Type
    * @return boolean
    */
   protected boolean validateServiceType(){
      String fieldname = RemoteServiceRequest.SERVICE_TYPE;
      return validator.validateMandatory(events, category, fieldname, transaction.getServiceType())
      		&& validator.validateLengthRange(events, category, fieldname, transaction.getServiceType(), 1,40);
   }

   /**
    * Validate Response Format
    * @return boolean
    */
   protected boolean validateResponseFormat(){
      String fieldname = RemoteServiceRequest.RESPONSE_FORMAT;
      return validator.validateMandatory(events, category, fieldname, transaction.getResponseFormat())
      		&& validator.validateLengthRange(events, category, fieldname, transaction.getResponseFormat(),1, 20);
   }

   /**
    * Validate Meter Serial Number
    * @param fieldname, value
   * @return boolean
    */
   protected boolean validateMeterSerialNumber(String fieldname, String value){
      return validator.validateLengthRange(events, category, fieldname, value,0, 12);
   }

   /**
    * Validate Meter Serial Number
    * @return boolean
    */
   protected boolean validateMeterSerialNumber(){
  	 return validateMeterSerialNumber(RemoteServiceRequest.METER_SERIAL_NUMBER1,transaction.getMeterSerialNumber1())
  			 & validateMeterSerialNumber(RemoteServiceRequest.METER_SERIAL_NUMBER2,transaction.getMeterSerialNumber2())
  			 & validateMeterSerialNumber(RemoteServiceRequest.METER_SERIAL_NUMBER3,transaction.getMeterSerialNumber3());
   }

   /**
    * Validate User Defined
    * @return boolean
    */
   protected boolean validateUserDefined(){
      return validateUserDef(RemoteServiceRequest.USER_DEF1, transaction.getUserDef1())
        & validateUserDef(RemoteServiceRequest.USER_DEF2, transaction.getUserDef2())
        & validateUserDef(RemoteServiceRequest.USER_DEF3, transaction.getUserDef3())
        & validateUserDef(RemoteServiceRequest.USER_DEF4, transaction.getUserDef4())
        & validateUserDef(RemoteServiceRequest.USER_DEF5, transaction.getUserDef5())
        & validateUserDef(RemoteServiceRequest.USER_DEF6, transaction.getUserDef6())
        & validateUserDef(RemoteServiceRequest.USER_DEF7, transaction.getUserDef7())
        & validateUserDef(RemoteServiceRequest.USER_DEF8, transaction.getUserDef8())
        & validateUserDef(RemoteServiceRequest.USER_DEF9, transaction.getUserDef9())
        & validateUserDef(RemoteServiceRequest.USER_DEF10, transaction.getUserDef10());
   }

   /**
    * Validate User Defined
    * @param fieldname, value
   * @return boolean
    */
   protected boolean validateUserDef(String fieldname, String value){
      return  validator.validateLengthRange(events, category, fieldname, value, 0,240);
   }




}
