/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/SORAllocateNMIBasicValidator.java-arc   1.9   03 May 2005 10:47:38   RayanS  $
 * $Revision:   1.9  $
 * $Date:   03 May 2005 10:47:38  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 21-Aug-2008    Bob Jones         Defect 4985 - increase each telephone number from one to three.
 * 22-Jun-2011    Bob Jones         7137 Add Customer Details fields for Person Name and Phone Numbers.
 * 03-Aug-2017    NG                7135 - PoC SMP Changes -- NMI and Checksum in Allocate NMI should be SKIPPED
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.ValidationSetting;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;

/**
 * @author $Author:   RayanS  $
 * @version $Revision:   1.9  $ $Date:   03 May 2005 10:47:38  $
 */

public class SORAllocateNMIBasicValidator extends ServiceOrderRequestBasicValidator
{

  private BasicValidationUtility validator = null;

  public SORAllocateNMIBasicValidator(Transaction trans)
  {
    super(trans);
    validator = BasicValidationUtility.getBasicInstance();
  }

  //Explicitly define all validations
  //This is listed in the order as prescribed in the B2B procedures
  protected boolean validateActionType() { return validateActionType(ValidationSetting.MANDATORY); }
  protected boolean validateRetServiceOrder() { return validateRetServiceOrder(ValidationSetting.MANDATORY); }
  protected boolean validateNotifyParty() { return validateNotifyParty(ValidationSetting.SKIPPED); }
  protected boolean validateServiceOrderType() { return validateServiceOrderType(ValidationSetting.MANDATORY); }
  protected boolean validateServiceOrderSubType() { return validateServiceOrderSubType(ValidationSetting.MANDATORY); }
  protected boolean validateDeenergisationReason() { return validateDeenergisationReason(ValidationSetting.SKIPPED); }
  protected boolean validateConfirmDeenergisation() { return validateConfirmDeenergisation(ValidationSetting.SKIPPED); }
  protected boolean validateServiceTime() { return validateServiceTime(ValidationSetting.SKIPPED); }
  protected boolean validateNMI() { return validateNMI(ValidationSetting.SKIPPED); }
  protected boolean validateNMIChecksum() { return validateNMIChecksum(ValidationSetting.SKIPPED); }
  protected boolean validateMeterSerialNumber() { return validateMeterSerialNumber(ValidationSetting.SKIPPED); }
  protected boolean validateServiceOrderAddress() { return validateServiceOrderAddress(ValidationSetting.MANDATORY); }
  protected boolean validateHazardDescription1() { return validateHazardDescription1(ValidationSetting.SKIPPED); }
  protected boolean validateHazardDescription2() { return validateHazardDescription2(ValidationSetting.SKIPPED); }
  protected boolean validateHazardDescription3() { return validateHazardDescription3(ValidationSetting.SKIPPED); }
  protected boolean validateAccessDetails() { return validateAccessDetails(ValidationSetting.SKIPPED); }
  protected boolean validateLifeSupport() { return validateLifeSupport(ValidationSetting.MANDATORY); }
  protected boolean validateFormReference() { return validateFormReference(ValidationSetting.OPTIONAL); }
  protected boolean validateFormNumber() { return validateFormNumber(ValidationSetting.OPTIONAL); }
  protected boolean validateSafetyCertificateId() { return validateSafetyCertificateId(ValidationSetting.OPTIONAL); }
  protected boolean validateSafetyCertificateIdMethodSent() { return validateSafetyCertificateIdMethodSent(ValidationSetting.OPTIONAL); }
  protected boolean validateMeteringSafetyCertificateId() { return validateMeteringSafetyCertificateId(ValidationSetting.SKIPPED); }
  protected boolean validateMeteringSafetyCertificateMethodSent() { return validateMeteringSafetyCertificateMethodSent(ValidationSetting.SKIPPED); }
  protected boolean validateSpecialInstructions1() { return validateSpecialInstructions1(ValidationSetting.OPTIONAL); } //Conditional Mandatory
  protected boolean validateSpecialInstructions2() { return validateSpecialInstructions2(ValidationSetting.OPTIONAL); }
  protected boolean validateSpecialInstructions3() { return validateSpecialInstructions3(ValidationSetting.OPTIONAL); }
  protected boolean validateCoOrdinationRequired() { return validateCoOrdinationRequired(ValidationSetting.SKIPPED); }
  protected boolean validateCoOrdinatingContactName() { return validateCoOrdinatingContactName(ValidationSetting.SKIPPED); }
  protected boolean validateCoOrdinatingContactPhone1() { return validateCoOrdinatingContactPhone1(ValidationSetting.SKIPPED); }
  protected boolean validateCoOrdinatingContactPhone2() { return validateCoOrdinatingContactPhone2(ValidationSetting.SKIPPED); }
  protected boolean validateCoOrdinatingContactPhone3() { return validateCoOrdinatingContactPhone3(ValidationSetting.SKIPPED); }
  protected boolean validateCustomerConsultationRequired() { return validateCustomerConsultationRequired(ValidationSetting.SKIPPED); }
  protected boolean validateContactName() { return validateContactName(ValidationSetting.SKIPPED); }
  protected boolean validateContactPhone1() { return validateContactPhone1(ValidationSetting.SKIPPED); }
  protected boolean validateContactPhone2() { return validateContactPhone2(ValidationSetting.SKIPPED); }
  protected boolean validateContactPhone3() { return validateContactPhone3(ValidationSetting.SKIPPED); }
  protected boolean validateRetailerName() { return validateRetailerName(ValidationSetting.OPTIONAL); }
  protected boolean validateRetailerPhone1() { return validateRetailerPhone1(ValidationSetting.OPTIONAL); } //Conditional Mandatory
  protected boolean validateRetailerPhone2() { return validateRetailerPhone2(ValidationSetting.OPTIONAL); }
  protected boolean validateRetailerPhone3() { return validateRetailerPhone3(ValidationSetting.OPTIONAL); }
  protected boolean validateScheduleDate() { return validateScheduleDate(ValidationSetting.SKIPPED); }
  protected boolean validateAppointmentReference() { return validateAppointmentReference(ValidationSetting.SKIPPED); }
  protected boolean validatePreferredDate() { return validatePreferredDate(ValidationSetting.SKIPPED); }
  protected boolean validatePreferredTime() { return validatePreferredTime(ValidationSetting.SKIPPED); }
  protected boolean validateRoleAssignments() { return validateRoleAssignments(ValidationSetting.MANDATORY); }
  protected boolean validateNMIStatusCode() { return validateNMIStatusCode(ValidationSetting.SKIPPED); }
  protected boolean validateEmbeddedNetworkParentName() { return validateEmbeddedNetworkParentName(ValidationSetting.OPTIONAL); }
  protected boolean validateCustomerType() { return validateCustomerType(ValidationSetting.MANDATORY); }
  protected boolean validateAverageDailyLoad() { return validateAverageDailyLoad(ValidationSetting.MANDATORY); }
  protected boolean validateMaximumDemand() { return validateMaximumDemand(ValidationSetting.OPTIONAL); }
  protected boolean validateLicenseeName() { return validateLicenseeName(ValidationSetting.OPTIONAL); }
  protected boolean validateLicenseeBusinessName() { return validateLicenseeBusinessName(ValidationSetting.OPTIONAL); }
  protected boolean validateLicenseePhone1() { return validateLicenseePhone1(ValidationSetting.OPTIONAL); }
  protected boolean validateLicenseePhone2() { return validateLicenseePhone2(ValidationSetting.OPTIONAL); }
  protected boolean validateLicenseePhone3() { return validateLicenseePhone3(ValidationSetting.OPTIONAL); }
  protected boolean validateRecId() { return validateRecId(ValidationSetting.OPTIONAL); }
  protected boolean validateRECAttendanceRequired() { return validateRECAttendanceRequired(ValidationSetting.OPTIONAL); }
  protected boolean validateInstallationType() { return validateInstallationType(ValidationSetting.OPTIONAL); }
  protected boolean validateSupplyPhases() { return validateSupplyPhases(ValidationSetting.MANDATORY); }
  protected boolean validateMeteringRequired() { return validateMeteringRequired(ValidationSetting.SKIPPED); }
  protected boolean validateOffPeakRequirement1() { return validateOffPeakRequirement1(ValidationSetting.OPTIONAL); }
  protected boolean validateOffPeakRequirement2() { return validateOffPeakRequirement2(ValidationSetting.OPTIONAL); }
  protected boolean validateOffPeakRequirement3() { return validateOffPeakRequirement3(ValidationSetting.OPTIONAL); }
  protected boolean validateMeterInstallCode() { return validateMeterInstallCode(ValidationSetting.SKIPPED); }
  protected boolean validateSwitchingServiceRequired() { return validateSwitchingServiceRequired(ValidationSetting.SKIPPED); }
  protected boolean validateProposedTariff() { return validateProposedTariff(ValidationSetting.SKIPPED); }
  protected boolean validateRequestData() { return validateRequestData(ValidationSetting.SKIPPED); }

}

