/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/SORResponseBasicValidator.java-arc   1.14   Aug 23 2005 15:31:30   adij  $
 * $Revision:   1.14  $
 * $Date:   Aug 23 2005 15:31:30  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 *                     Modification History
 *                     ====================
 *  Date Changed      Changed By     Comments
 *  ------------      ----------     ------------------------------------------
 *  21-Aug-2008       Bob Jones      Defect 4985 - increase each telephone number from one to three.
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.SORResponse;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.ServiceOrderRequest;
import com.capgemini.nemmco.b2bb.TransactionFactory;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.ExceptionCode;
import com.capgemini.nemmco.b2bb.enums.ResponseType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderStatus;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderSubType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderType;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.exceptions.UnableToInstantiateClassException;
import java.util.Date;
import java.util.List;

/**
 * Create response transaction Service Order Response basic validator
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.14  $   $Date:   Aug 23 2005 15:31:30  $
 * @version $Revision:   1.14  $   $Date:   Aug 23 2005 15:31:30  $    $Author:   adij  $
 *          $Changes for CR100. Changed validateResponseCode() to allow additional codes when the . $
 * @version $Revision:   1.14  $   $Date:   Aug 23 2005 15:31:30  $    $Author:   adij  $
 *          $Defect 3481: Changed validateResponseCode(). Removed the mandatory check when the ServiceOrderStatus
 *          is 'Complete'.$
 *  Defect 3604 : NG : 09-11-2005 Changed validateActualDateTime not raise error only if actual time is greater than transaction time 
 *
 */
public class SORResponseBasicValidator extends Validator {
  BasicValidationUtility validator = null;
  private SORResponse transaction = null;
  private String transactionType = "";

  /**
   * Default constructor
   * @param trans The transaction to set
   */
  public SORResponseBasicValidator(Transaction trans) {
    super(trans);
    transaction = (SORResponse) trans;
    transactionType = transaction.getTransactionType().getCode();
    validator = BasicValidationUtility.getBasicInstance();
  }

  /**
   *
   * @return Returns true if the transaction is valid
   */
  public boolean validate() {
    boolean result = true;
    result &= validateTransactionID()
        & validateTransactionDate()
        & validateInitiatingTransactionID()
        & validateResponseType()
        & validateNMI()
        & validateNMIChecksum()
        & validateAddress()
        & validateNMIAddress()
        & validateServiceOrderNum()
        & validateServiceProviderReference()
        & validatePersonName()
        & validateTelephone1()
        & validateTelephone2()
        & validateTelephone3()
        & validateTelephonePersonName()
        & validateSpecialNote1()
        & validateSpecialNote2()
        & validateSpecialNote3()
        & validateServiceOrderStatus()
        & validateResponseCode()
        & validateActualDateTime()
        & validateProductCode1()
        & validateProductCode2()
        & validateProductCode3();

    return result;
  }

  /**
   *
   * @return Returns true if the initiatingTransactionID is valid
   */
  protected boolean validateInitiatingTransactionID()
  {
    String field = Transaction.INIT_TRANSACTION_ID;
    return validator.validateMandatory(events, category, field, transaction.getInitiatingTransactionID())
      && validator.validateTransactionId(events, category, field, transaction.getInitiatingTransactionID());
  }

  /**
   *
   * @return Returns true if the responseType is valid
   */
  protected boolean validateResponseType() {
    String field = SORResponse.RESPONSE_TYPE;
    return validator.validateMandatory(events, category, field, transaction.getResponseType())
        && validator.validateAllowedValue(events,category,field,transaction.getResponseType(), ResponseType.CLOSURE.getCode());
  }

  /**
   *
   * @return Returns true if the NMI is valid
   */
  protected boolean validateNMI() {
    String fieldname = SORResponse.NMI;

    if (transaction.getServiceOrderRequest() == null) return true;

    if (transaction.getServiceOrderRequest().getTransactionType() == TransactionType.SOR_ALLOCATE_NMI
    		&& transaction.getServiceOrderStatus().equals(ServiceOrderStatus.NOT_COMPLETE.getCode())) {
    	//NMI optional
    	return !validator.exists(transaction.getNMI())
    			|| validator.validateNMI(events, transactionType, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
    }
    //NMI mandatory
    return validator.validateMandatory(events, transactionType, fieldname, transaction.getNMI())
          && validator.validateNMI(events, transactionType, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
  }

  /**
   *
   * @return Returns true if address object is valid
   */
  protected boolean validateAddress() {
    try {
      String fieldname = SORResponse.ADDRESS;

      if (transaction.getNMI() != null && !transaction.getNMI().equals("")) {
        if (transaction.getAddress() == null) {
          return true;
        } else if (!transaction.getAddress().fieldsExists()) {
          return true;
        }
      } else {
        if (transaction.getAddress() == null) {
          Object[] args = { "Address" };
          validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0201, "mandatoryMissing");
          return false;
        }
      }
      Address address = transaction.getAddress();
      address.setEvents(events);

      address.setParentCategory(category);
      address.setParentField(fieldname);
      boolean result = address.validate();
      return result;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }

  /**
   *
   * @return Returns true if only either NMI or Address exists, not both
   */
  protected boolean validateNMIAddress()
  {
    try
    {
      String fieldname = SORResponse.ADDRESS;
      Address address = transaction.getAddress();
      if(transaction.getNMI()!=null && !transaction.getNMI().equals("") &&
         address.fieldsExists())
      {
        Object[] args = { "NMI or Address" };
        validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202, "invalidChoiceMandatory");
        return false;
      }
      return true;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }

  /**
   *
   * @return Returns true if serviceOrderNumber is valid
   */
  protected boolean validateServiceOrderNum() {
    String fieldname = SORResponse.SERVICE_ORDER_NUM;
    boolean tmpresult = validator.validateMandatory(events, category, fieldname, transaction.getServiceOrderNum());
    return tmpresult && validator.validateLengthRange(events, category, fieldname, transaction.getServiceOrderNum(), 1, 15);
  }

  /**
   *
   * @return Returns true if serviceProviderReference is valid
   */
  protected boolean validateServiceProviderReference() {
    String fieldname = SORResponse.SERVICE_PROVIDER_REFERENCE;
    boolean tmpresult = validator.validateLengthRange(events, category, fieldname, transaction.getServiceProviderReference(), 0, 15);
    return tmpresult;
  }

  /**
   *
   * @return Returns true if the personName object is valid
   */
  protected boolean validatePersonName() {
    String fieldname = SORResponse.PERSON_NAME;
    boolean result = true;
    try {
      if (transaction.getPersonName() != null && transaction.getPersonName().fieldsExists()) {
        PersonName personName = transaction.getPersonName();
        personName.setEvents(events);

        personName.setParentCategory(category);
        personName.setParentField(fieldname);
        result = personName.validate();
        return result;
      }
      return result;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }

  /**
   *
   * @return Returns true if telephone object is valid
   */
  protected boolean validateTelephone1() {
    String fieldname = SORResponse.TELEPHONE1;
    boolean result = true;
    try {
      if (transaction.getTelephone1() != null && transaction.getTelephone1().fieldsExists()) {
        Telephone telephone = transaction.getTelephone1();
        telephone.setEvents(events);

        telephone.setParentCategory(category);
        telephone.setParentField(fieldname);
        result = telephone.validate();
        return result;
      }
      return result;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }
  
  protected boolean validateTelephone2() {
    String fieldname = SORResponse.TELEPHONE2;
    boolean result = true;
    try {
      if (transaction.getTelephone2() != null && transaction.getTelephone2().fieldsExists()) {
        Telephone telephone = transaction.getTelephone2();
        telephone.setEvents(events);

        telephone.setParentCategory(category);
        telephone.setParentField(fieldname);
        result = telephone.validate();
        return result;
      }
      return result;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }
  
  protected boolean validateTelephone3() {
    String fieldname = SORResponse.TELEPHONE3;
    boolean result = true;
    try {
      if (transaction.getTelephone3() != null && transaction.getTelephone3().fieldsExists()) {
        Telephone telephone = transaction.getTelephone3();
        telephone.setEvents(events);

        telephone.setParentCategory(category);
        telephone.setParentField(fieldname);
        result = telephone.validate();
        return result;
      }
      return result;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }  

  /**
   *
   * @return Returns true when both telephone and personname exist, false if only telephone exists
   */
  protected boolean validateTelephonePersonName()
  {
    try
    {
      String fieldname = SORResponse.PERSON_NAME;
      PersonName personName = transaction.getPersonName();
      Telephone telephone1 = transaction.getTelephone1();
      Telephone telephone2 = transaction.getTelephone2();
      Telephone telephone3 = transaction.getTelephone3();
      if(telephone1 == null && telephone2 == null && telephone3 == null)
      {
        return true;
      }
      if((telephone1.fieldsExists() || telephone2.fieldsExists() || telephone3.fieldsExists()) && !personName.fieldsExists())
      {
        Object[] args = { "Person name details" };
        validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0201, "mandatoryMissing");
        return false;
      }
      return true;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }

  /**
   *
   * @return Returns true if specialNote1, specialNote2, specialNote3 are valid
   */
  protected boolean validateSpecialNote1() {
    String fieldname1 = SORResponse.SPECIAL_NOTE1;
    String fieldname2 = SORResponse.SPECIAL_NOTE2;
    String fieldname3 = SORResponse.SPECIAL_NOTE3;
    boolean tmpresult = true;
    boolean mandatoryCheck = false;

    if (transaction.getResponseCode() != null && !transaction.getResponseCode().equals("")) {
      if (transaction.getResponseCode().equals(ExceptionCode.OTHER.getCode())
          || transaction.getResponseCode().equals(ExceptionCode.RECIPIENT_CANCELLATION.getCode())
          || transaction.getResponseCode().equals(ExceptionCode.DOC_NOT_PROVIDED.getCode())
          ) {
        tmpresult = validator.validateMandatory(events, category, fieldname1, transaction.getSpecialNote1());
        mandatoryCheck = true;
      }
    }
    if (transaction.getServiceOrderStatus() != null && !transaction.getServiceOrderStatus().equals("") && !mandatoryCheck) {
      if (transaction.getServiceOrderStatus().equals(ServiceOrderStatus.PARTIALLY_COMPLETE.getCode())) {
        tmpresult = validator.validateMandatory(events, category, fieldname1, transaction.getSpecialNote1());
      }
    }

    if(transaction.getSpecialNote3()!=null && !transaction.getSpecialNote3().equals(""))
    {
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname1, transaction.getSpecialNote1(), 1, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname2, transaction.getSpecialNote2(), 1, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname3, transaction.getSpecialNote3(), 0, 80);
    }else
    if(transaction.getSpecialNote2()!=null && !transaction.getSpecialNote2().equals(""))
    {
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname1, transaction.getSpecialNote1(), 1, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname2, transaction.getSpecialNote2(), 0, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname3, transaction.getSpecialNote3(), 0, 80);
    }else
    {
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname1, transaction.getSpecialNote1(), 0, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname2, transaction.getSpecialNote2(), 0, 80);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname3, transaction.getSpecialNote3(), 0, 80);
    }

    return tmpresult;
  }

  /**
   *
   * @return Returns true
   */
  protected boolean validateSpecialNote2() {
    return true;
  }

  /**
   *
   * @return Returns true
   */
  protected boolean validateSpecialNote3() {
    return true;
  }

  /**
   *
   * @return Returns true if serviceOrderStatus is valid
   */
  protected boolean validateServiceOrderStatus() {
    String fieldname = SORResponse.SERVICE_ORDER_STATUS;
    boolean tmpresult = validator.validateMandatory(events, transactionType, fieldname, transaction.getServiceOrderStatus());
    return tmpresult && validator.validateServiceOrderStatus(events, transactionType, fieldname, transaction.getServiceOrderStatus());
  }

  /**
   *
   * @return Returns true if responseCode is valid
   */
  protected boolean validateResponseCode() {
    String fieldname = SORResponse.RESPONSE_CODE;
    String tempServiceOrderStatus = transaction.getServiceOrderStatus();
    
    //Can't continue with out a valid Order status
    if (!validator.exists(tempServiceOrderStatus)) return true;

    if (transaction.getServiceOrderRequest() == null) {
        if (validator.exists(transaction.getServiceOrderType())) {
            //try and match to a Transaction Type
            String code = TransactionType.getCodeForDescSubDesc(transaction.getServiceOrderType(), transaction.getServiceOrderSubType());
            if (code != null) {
                try {
                  TransactionType tt = TransactionType.getTransactionType(code);
                  Transaction trans = TransactionFactory.getTransaction(tt);
                  ServiceOrderRequest request = (ServiceOrderRequest) trans;
                  request.setServiceOrderType(transaction.getServiceOrderType());
                  request.setServiceOrderSubType(transaction.getServiceOrderSubType());
                  request.setNMI(transaction.getNMI());
                  request.setNMIChecksum(transaction.getNMIChecksum());
                  request.setServiceOrderAddress(transaction.getAddress());
                  request.setRetServiceOrder(transaction.getServiceOrderNum());
                  transaction.setServiceOrderRequest(request);
                }
				catch (UnableToInstantiateClassException e) {
					//nothing to do
				}
				catch (ClassCastException e) {
					//nothing to do
				}
            }
        }
    }

    //Validate ExceptionCode values allowed for a COMPLETE response
    if (tempServiceOrderStatus.equals(ServiceOrderStatus.COMPLETE.getCode())) {
      //Response code is optional
      if (!validator.exists(transaction.getResponseCode())) {
        return true;
      }
      List<? extends Object> allowedExceptionCodes = ExceptionCode.getAllCompletedCodes();
      return validator.validateEnumeratedValue(events, transactionType, fieldname, "Exception Code", allowedExceptionCodes, transaction.getResponseCode());
    }
    
    //Validate ExceptionCode values allowed for a NOT COMPLETE response
    if (tempServiceOrderStatus.equals(ServiceOrderStatus.NOT_COMPLETE.getCode())) {
      if (!validator.validateMandatory(events, transactionType, fieldname, transaction.getResponseCode())) return false;

      List<? extends Object> allowedExceptionCodes = null;
      //JTNT 02-Aug-05 CR100 added.
      // if the Response does not know about its corresponding Request,
      // then allow all the valid Not Complete ExceptionCodes.
      if (transaction.getServiceOrderRequest() == null) {
        allowedExceptionCodes = ExceptionCode.getAllNotCompletedCodes();
      }
      else {
        allowedExceptionCodes = ExceptionCode.getNotCompletedCodes(transaction.getServiceOrderRequest());
      }
      return validator.validateEnumeratedValue(events, transactionType, fieldname, "Exception Code", allowedExceptionCodes, transaction.getResponseCode());
    }
      
      //Validate ExceptionCode values allowed for a PARTIALLY COMPLETE response
    if (tempServiceOrderStatus.equals(ServiceOrderStatus.PARTIALLY_COMPLETE.getCode())) {
      if (!validator.validateMandatory(events, transactionType, fieldname, transaction.getResponseCode())) return false;
      List<? extends Object> allowedExceptionCodes = null;
      if (transaction.getServiceOrderRequest() == null) {
          allowedExceptionCodes = ExceptionCode.getAllPartiallyCompletedCodes();
      }
      else {
        allowedExceptionCodes = ExceptionCode.getPartiallyCompletedCodes(transaction.getServiceOrderRequest());
      }
      return validator.validateEnumeratedValue(events, transactionType, fieldname, "Exception Code", allowedExceptionCodes, transaction.getResponseCode());
    }
    
    //It shouldn't be possible to get here....
    return true;
  }

  /**
   *
   * @return Returns true if actualDateTime is valid
   */
  protected boolean validateActualDateTime() {
    String field = SORResponse.ACTUAL_DATE_TIME;
    boolean tmpresult = true;

    if (transaction.isActualDateTimeSet()) {
      tmpresult = tmpresult && validator.validateMandatory(events, category, field, transaction.getActualDateTime());
      tmpresult = tmpresult && validator.validateXMLDateTime(events, category, field, transaction.getActualDateTime());

      try
      {
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(transaction.getActualDateTime(), B2BDateTimeFormatter.XMLDATE);
        Date mActualDate = df.getDate();

        df = new B2BDateTimeFormatter(transaction.getTransactionDate(), B2BDateTimeFormatter.XMLDATE);
        Date mTransDate = df.getDate();
        if(!mActualDate.after(mTransDate))
        {
        }else
        {
          tmpresult = false;
          Object[] args = { transaction.getActualDateTime(), transaction.getTransactionDate() };
          validator.addEvent(events, transactionType, field, args, EventCode.CODE_1921, "invalidActualDateTimeRange");
        }
      }catch(Exception e)
      {
        tmpresult = false;
      }


    } else {
      tmpresult = tmpresult && validator.validateMandatory(events, category, field, transaction.getActualDisplayDateTime());
      tmpresult = tmpresult && validator.validateB2BDateTime(events, category, field, transaction.getActualDisplayDateTime());

      try
      {
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(transaction.getActualDisplayDateTime(), B2BDateTimeFormatter.B2BDATE);
        Date mActualDate = df.getDate();

        df = new B2BDateTimeFormatter(transaction.getTransactionDisplayDate(), B2BDateTimeFormatter.B2BDATE);
        Date mTransDate = df.getDate();
        if(!mActualDate.after(mTransDate))
        {
        }else
        {
          tmpresult = false;
          Object[] args = { transaction.getActualDisplayDateTime(), transaction.getTransactionDisplayDate() };
          validator.addEvent(events, transactionType, field, args, EventCode.CODE_1921, "invalidActualDateTimeRange");
        }
      }catch(Exception e)
      {
        tmpresult = false;
      }
    }
    return tmpresult;
  }

  /**
   *
   * @return Returns true if productCode1, productCode2, productCode3 are valid
   */
  protected boolean validateProductCode1() {
    String fieldname1 = SORResponse.PRODUCT_CODE1;
    String fieldname2 = SORResponse.PRODUCT_CODE2;
    String fieldname3 = SORResponse.PRODUCT_CODE3;
    boolean tmpresult = true;

    tmpresult = validator.validateMandatory(events, category, fieldname1, transaction.getProductCode1());

    if( (transaction.getProductCode1() !=null && transaction.getProductCode1().equalsIgnoreCase("Cost TBA")) ||
        (transaction.getProductCode2() !=null && transaction.getProductCode2().equalsIgnoreCase("Cost TBA")) ||
        (transaction.getProductCode3() !=null && transaction.getProductCode3().equalsIgnoreCase("Cost TBA")))
    {
      if(transaction.getServiceOrderRequest()!=null)
      {
        if(transaction.getServiceOrderRequest().getServiceOrderType().equals(ServiceOrderType.RE_ENERGISATION.getCode()) ||
           transaction.getServiceOrderRequest().getServiceOrderType().equals(ServiceOrderType.DE_ENERGISATION.getCode()) ||
           transaction.getServiceOrderRequest().getServiceOrderType().equals(ServiceOrderType.SPECIAL_READ.getCode()))
        {
          try {
          Object[] args= {transaction.getProductCode1()};
          if(transaction.getProductCode1() !=null && transaction.getProductCode1().equalsIgnoreCase("Cost TBA"))
          {
            args[0] = transaction.getProductCode1();
          }else
          if(transaction.getProductCode2() !=null && transaction.getProductCode2().equalsIgnoreCase("Cost TBA"))
          {
            args[0] = transaction.getProductCode2();
          }else
          if(transaction.getProductCode3() !=null && transaction.getProductCode3().equalsIgnoreCase("Cost TBA"))
          {
            args[0] = transaction.getProductCode3();
          }

          validator.addEvent(events, transactionType, fieldname1, args, EventCode.CODE_0202, "valueNotAllowed");
          return false;
          } catch (Exception e) {
          validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
          return false;
          }
        }
      }
    }

    if(transaction.getProductCode3()!=null && !transaction.getProductCode3().equals(""))
    {
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname1, transaction.getProductCode1(), 1, 10);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname2, transaction.getProductCode2(), 1, 10);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname3, transaction.getProductCode3(), 0, 10);
    }else
    {
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname1, transaction.getProductCode1(), 1, 10);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname2, transaction.getProductCode2(), 0, 10);
      tmpresult = tmpresult && validator.validateLengthRange(events, category, fieldname3, transaction.getProductCode3(), 0, 10);
    }

    return tmpresult;
  }

  /**
   *
   * @return Return true
   */
  protected boolean validateProductCode2() {
    return true;
  }

  /**
   *
   * @return Return true
   */
  protected boolean validateProductCode3() {
    return true;
  }
}

