/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/ServiceOrderAppointmentNotificationBasicValidator.java-arc   1.6   11 Apr 2005 11:11:10   RayanS  $
 * $Revision:   1.6  $
 * $Date:   11 Apr 2005 11:11:10  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */
package com.capgemini.nemmco.b2bb.validators;

/**
 * The class <code>ServiceOrderAppointmentNotificationBasicValidator</code> is the Basic Validator 
 * object of the Transaction.
 *
 * @author  $Author:   RayanS  $
 * @version $Revision:   1.6  $
 */
import com.capgemini.nemmco.b2bb.ServiceOrderAppointmentNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.ResponseType;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;


public class ServiceOrderAppointmentNotificationBasicValidator extends Validator
{
  protected ServiceOrderAppointmentNotification transaction;

  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * @param transaction Transaction to use in the validator.
   */
  public ServiceOrderAppointmentNotificationBasicValidator(Transaction transaction) {
    super(transaction);
    this.transaction = (ServiceOrderAppointmentNotification) transaction;
  }

  /**
   * Validate the Transaction
   * 
   * @return boolean result of validation
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID()
      & validateTransactionDate()
      & validateResponseType()
      & validateRetServiceOrder()
      & validateNMI()
      & validateNMIChecksum()
      & validateServiceOrderAddress()
      & validateNMIAddress()
      & validateAppointmentReference()
      & validateCustomersPreferredDate()
      & validateCustomersPreferredTime()
      & validateServiceProviderReference();

    return result;
  }
  
  protected boolean validateNMI() {
    String fieldname = ServiceOrderAppointmentNotification.NMI_FIELD;
    boolean tmpresult = true;

    if (validator.exists(transaction.getNMI())) {
      tmpresult = validator.validateNMI(events, category, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
    }
    
    return tmpresult;
  }
  
  protected boolean validateResponseType() {
    String fieldname = ServiceOrderAppointmentNotification.RESPONSE_TYPE;
    return validator.validateMandatory(events, category,fieldname,transaction.getResponseType())
      && validator.validateAllowedValue(events,category,fieldname,transaction.getResponseType(), ResponseType.APPOINTMENT.getCode());
  }
  
  protected boolean validateCustomersPreferredDate() {
    String fieldname = ServiceOrderAppointmentNotification.CUSTOMERS_PREFERRED_DATE;
    return validator.validateMandatory(events, category, fieldname, transaction.getCustomersPreferredDate())
      && validator.validateXMLDate(events,category,fieldname,transaction.getCustomersPreferredDate());
  }

  protected boolean validateCustomersPreferredTime() {
    String fieldname = ServiceOrderAppointmentNotification.CUSTOMERS_PREFERRED_TIME;
    return validator.validateMandatory(events, category, fieldname, transaction.getCustomersPreferredTime())
      && validator.validateXMLTime(events,category,fieldname,transaction.getCustomersPreferredTime());
  }
  
  protected boolean validateRetServiceOrder() {
    String fieldname = ServiceOrderAppointmentNotification.RET_SERVICE_ORDER;
    return validator.validateMandatory(events, category, fieldname, transaction.getRetServiceOrder())
      && validator.validateUniqueIdentifier(events, category, fieldname, transaction.getRetServiceOrder());
  }
  
  protected boolean validateAppointmentReference() 
  {
    String fieldname = ServiceOrderAppointmentNotification.APPOINTMENT_REFERENCE;
    return validator.validateMandatory(events, category, fieldname, transaction.getAppointmentReference())
      && validator.validateUniqueIdentifier(events, category, fieldname, transaction.getAppointmentReference());
  }
  
  protected boolean validateServiceProviderReference() 
  {
    String fieldname = ServiceOrderAppointmentNotification.SERVICE_PROVIDER_REFERENCE;
    String serviceProviderReference = transaction.getServiceProviderReference();
    if (validator.exists(serviceProviderReference)) {
      return validator.validateUniqueIdentifier(events, category, fieldname, transaction.getServiceProviderReference());
    }
    return true;
  }  
  
  protected boolean validateServiceOrderAddress() {
    try {
      String fieldname = ServiceOrderAppointmentNotification.SERVICE_ORDER_ADDRESS;

      if (validator.exists(transaction.getNMI())) {
        if (transaction.getServiceOrderAddress() == null) {
          return true;
        } else if (!transaction.getServiceOrderAddress().fieldsExists()) {
          return true;
        }
      } else {
        if (transaction.getServiceOrderAddress() == null) {
          Object[] args = { "Address" };
          validator.addEvent(events, category, fieldname, args, EventCode.CODE_0201, "mandatoryMissing");
          return false;
        }
        
        Address address = transaction.getServiceOrderAddress();
        address.setEvents(events);
        address.setParentCategory(category);
        address.setParentField(fieldname);

        if (address.fieldsExists()) {
          return address.validate();
        }
      }
      
      return true;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }
  
  protected boolean validateNMIAddress() 
  {
    try
    {
      String nmiField = ServiceOrderAppointmentNotification.NMI_FIELD;
      String addressField = ServiceOrderAppointmentNotification.SERVICE_ORDER_ADDRESS;
      String nmiData = transaction.getNMI();
      Address addressData = transaction.getServiceOrderAddress();

      boolean nmiExists = validator.exists(nmiData);
      boolean addressExists = (addressData != null && addressData.fieldsExists());
      if(!((nmiExists || addressExists) && (!nmiExists || !addressExists)))  //one or the other but not both
      {
        Object[] args = { "[" + nmiField +", " + addressField + "]" };
        EventCode eventCode = (!nmiExists && !addressExists) ? EventCode.CODE_0201 : EventCode.CODE_0202;
        validator.addEvent(events, category, addressField, args, eventCode, "invalidChoiceMandatory");
        return false;
      }
      return true;
    } catch (Exception e) {
      validator.unexpectedException(events, transaction.getTransactionType().getCode(), e.getMessage());
      return false;
    }
  }  
  
  
  
  
}

