/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/ServiceOrderRequestBasicValidator.java-arc   1.33   Sep 15 2005 15:15:04   adij  $
 * $Revision:   1.33  $
 * $Date:   Sep 15 2005 15:15:04  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Modification History
 * 
 * Defect 3701 : 28-03-06 : BJ : If Preferred Time is entered, Preferred Date
 * must also be entered.
 * Defect 4300 : 01-05-07 : EG : Change REC-AttendanceRequired,CustomersPreferredDateAndTime to optional.
 * 
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 21-Aug-2008    Bob Jones      Defect 4985 - increase each telephone number from one to three.
 * 22-May-2009    Tushar Gupta   Defect 5439 - Modify method validateServiceTime to varaible to true if ServiceTime  is Non Business hours
 * 10-Dec-2010    Oleg Pevzner   Def 6675 EVM for b2b does not validate length for role id. Allow <BusinessName> and <ContactName> when <PhoneNumber> is provided.
 * 21-Aug-2008    Bob Jones      7479 Change the maximum allowed length of the Business name field from 60 to 200 to conform with the schema.
 * 16-Jul-2017    Hamish McNeish Changes to support PoC changes
 * 20-Jul-2017    Harpreet kaur  d7135 pkg7b : Changes for CustomerContactName as part of SpecialRead screen
 * 28-Jul-2017    Harpreet kaur  d7135 pkg7b : Changes for MeterSerialNumber and Proposed Tariff as part of MeterServiceWorks screens
 */

package com.capgemini.nemmco.b2bb.validators;

/**
 * Defines all the fields for every type of Service Order Request (SOR). As SOR
 * messages all use common fields one parent class defini
 * all these fields
 * is needed.
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.33  $ $Date:   Sep 15 2005 15:15:04  $
 */
import com.capgemini.nemmco.b2bb.ServiceOrderRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.ActionType;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.MeteringType;
import com.capgemini.nemmco.b2bb.enums.SORRole;
import com.capgemini.nemmco.b2bb.enums.ServiceTime;
import com.capgemini.nemmco.b2bb.enums.SupplyPhase;
import com.capgemini.nemmco.b2bb.enums.TrueFalse;
import com.capgemini.nemmco.b2bb.enums.ValidationSetting;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.lookup.ScreenName;

public class ServiceOrderRequestBasicValidator extends Validator
{
  private BasicValidationUtility validator = null;
  //casts the transaction to a SiteAccessNotification type for ease of access
  //This is a reference to the transaction Object in the parent class
  protected ServiceOrderRequest transaction = null;
  // true if special instruction is required
  protected boolean specialInstructionRequired = false;
  // used for role assignment validation
  private String ROLE_ASSIGNMENT_LOOKUP = "roleAssignment";
  private String ROLE_ASSIGNMENT_TYPE = "Generic";
  
  /**
  * Constructor. Sets a transaction with the objects initialisation.
  * @param trans Transaction to use in the validator.
  */
  public ServiceOrderRequestBasicValidator(Transaction trans)
  {
    super(trans);
    transaction = (ServiceOrderRequest) trans;
    validator = BasicValidationUtility.getBasicInstance();
  }
  
  /**
   * Validate the Transaction
   * 
   * @return boolean result of validation
   * @throws TransactionNotSetException is a transaction was not set before 
   * validation is called.
   */
  public boolean validate() throws TransactionNotSetException
  {
    //Aligned to match validation matrix in the procedures 
    return validateTransactionDate()
      & validateTransactionID()
      & validateActionType()
      & validateRetServiceOrder()
      //From
        //To
      & validateNotifyParty()
      & validateServiceOrderType()
      & validateServiceOrderSubType()
      & validateDeenergisationReason()
      & validateConfirmDeenergisation()
      & validateServiceTime()
      & validateNMI()
      & validateNMIChecksum()
      & validateMeterSerialNumber()
      & validateMeterSerialNumber1()
      & validateMeterSerialNumber2()
      & validateMeterSerialNumber3()
      & validateServiceOrderAddress()
      & validateHazardDescription1()
      & validateHazardDescription2()
      & validateHazardDescription3()
      & validateAccessDetails()
      & validateLifeSupport()
      & validateFormReference()
      & validateFormNumber()
      & validateSafetyCertificateId()
      & validateSafetyCertificateIdMethodSent()
      & validateMeteringSafetyCertificateId()
      & validateMeteringSafetyCertificateMethodSent()
      //Special Instructions... is special.. and at the bottom
      & validateCoOrdinationRequired()
      & validateCoOrdinatingContactName()
      & validateCoOrdinatingContactPhone1()
      & validateCoOrdinatingContactPhone2()
      & validateCoOrdinatingContactPhone3()
      & validateCustomerConsultationRequired()
      
      & validateContactName()
      & validateContactPhone1()
      & validateContactPhone2()
      & validateContactPhone3()

      & validateRetailerName()
      & validateRetailerPhone1()
      & validateRetailerPhone2()
      & validateRetailerPhone3()

      & validateScheduleDate()
      & validateAppointmentReference()
      & validatePreferredDate()
      & validatePreferredTime()
      & validateRoleAssignments()
      & validateNMIStatusCode()
      & validateEmbeddedNetworkParentName()
      & validateCustomerType()
      & validateAverageDailyLoad()
      & validateMaximumDemand()
      & validateLicenseeName()
      & validateLicenseeBusinessName()
      & validateLicenseePhone1()
      & validateLicenseePhone2()
      & validateLicenseePhone3()
      & validateRecId()
      & validateRECAttendanceRequired()
      & validateInstallationType()
      & validateSupplyPhases()
      & validateMeteringRequired()
      & validateOffPeakRequirement1()
      & validateOffPeakRequirement2()
      & validateOffPeakRequirement3()
      & validateMeterInstallCode()
      & validateSwitchingServiceRequired()
      & validateProposedTariff()
      & validateProposedTariff1()
      & validateProposedTariff2()
      & validateProposedTariff3()
      & validateRequestData()
  // always validate SpecialInstructions last as they are dependent on
  // other validations
      & validateSpecialInstructions1()
      & validateSpecialInstructions2()
      & validateSpecialInstructions3();
  }

  // expected ordering of the roles
  protected String[] order = { SORRole.RP.getCode()
                    ,SORRole.MDP.getCode()
                    ,SORRole.MPB.getCode()
                    ,SORRole.MPC.getCode()
                   };
  int location  = 0;
  
  
  protected boolean validateRoleAssignments() {
    return validateRoleAssignments(ValidationSetting.SKIPPED);
  }
  protected boolean validateRoleAssignments(ValidationSetting setting){
	if (setting == ValidationSetting.SKIPPED) return true;
	//The fieldnames allow linking to the right onscreen discription
	//The validation doesn't actually care about the order - as long as the role exists
    boolean result = validateParty(setting, SORRole.RP.getCode(), ServiceOrderRequest.PARTY1, ServiceOrderRequest.ROLE1) &
                     validateParty(setting, SORRole.MDP.getCode(), ServiceOrderRequest.PARTY2, ServiceOrderRequest.ROLE2) &
                     validateParty(setting, SORRole.MPB.getCode(), ServiceOrderRequest.PARTY3, ServiceOrderRequest.ROLE3) &
                     validateParty(setting, SORRole.MPC.getCode(), ServiceOrderRequest.PARTY4, ServiceOrderRequest.ROLE4);

    if (!result){
      // invalid roles
      Object[] args = {};
      try {
        validator.addEvent(events,ROLE_ASSIGNMENT_TYPE,ROLE_ASSIGNMENT_LOOKUP,args,EventCode.CODE_0202,"invalidRoleAssignment");
      } catch (UnableToLookupResourceException e){
        validator.unexpectedException(events, ROLE_ASSIGNMENT_LOOKUP, e.getMessage());
      }
    } else {
      // validate party dependencies
      result = result && validatePartyDependancies();
    }
    return result;
  }
  
  private boolean validateParty(ValidationSetting setting, String roleName, String partyFieldname, String roleFieldname) {
	String roleValue = "";
	String partyValue = "";
	
	//Match role
	if (roleName.equals(transaction.getRole1())) {
		roleValue = transaction.getRole1();
		partyValue = transaction.getParty1();
	}
	else if (roleName.equals(transaction.getRole2())) {
		roleValue = transaction.getRole2();
		partyValue = transaction.getParty2();
	}
	else if (roleName.equals(transaction.getRole3())) {
		roleValue = transaction.getRole3();
		partyValue = transaction.getParty3();
	}
	else if (roleName.equals(transaction.getRole4())) {
		roleValue = transaction.getRole4();
		partyValue = transaction.getParty4();
	}

	if (!checkMandatory(setting, roleFieldname, roleValue) | !checkMandatory(setting, partyFieldname, partyValue)) return false;
    
    //optional, if both don't exist
    if (!validator.exists(roleValue) && !validator.exists(partyValue)) return true;
    
    //if any in the pair do not exist
    if (!validator.exists(roleValue) || !validator.exists(partyValue)){
      return false;
    }
    
    return validator.validateParticipant(events,category,partyFieldname,partyValue);
  }
  
  protected boolean validateRole(String fieldname, String roleValue) {
    return validator.validateSORRole(events,category,fieldname,roleValue);
  }

  protected boolean validatePartyDependancies() {
    return true;
  }

  protected boolean validateLicenseeBusinessName() {
    return validateLicenseeBusinessName(ValidationSetting.SKIPPED);
  }
  protected boolean validateLicenseeBusinessName(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.LICENSEE_BUSINESS_NAME;
    String value = transaction.getLicenseeBusinessName();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getLicenseeBusinessName(),0,200);
  }
  
  private boolean checkMandatory(ValidationSetting setting, String fieldname, String value) {
	  if (setting == ValidationSetting.MANDATORY) {
    	return validator.validateMandatory(events,category,fieldname,value);
    }
	  return true; 
  }
  
  private boolean checkMandatory(ValidationSetting setting, String fieldname, XMLObject obj) {
	  if (setting == ValidationSetting.MANDATORY && !validator.exists(obj)) {
	  	//Force mandatory error passing null value
    	return validator.validateMandatory(events,category,fieldname,null);
    }
	  return true; 
  }
    
  protected boolean validateAccessDetails() {
	  return validateAccessDetails(ValidationSetting.SKIPPED);
  }
  protected boolean validateAccessDetails(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.ACCESS_DETAILS;
    String value = transaction.getAccessDetails();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,1,160);
  }
  
  protected boolean validateActionType() {
	  return validateActionType(ValidationSetting.SKIPPED);
  }
  protected boolean validateActionType(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.ACTIONTYPE;
    String value = transaction.getActionType();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    boolean tmpresult = validator.validateActionType(events,category,fieldname,value);
    // if ActionType is replace then special instruction is mandatory
    if (tmpresult && value.equals(ActionType.REPLACE.getCode())){
      specialInstructionRequired = true;
    }
    return tmpresult;
  }
  
  protected boolean validateAppointmentReference() {
	  return validateAppointmentReference(ValidationSetting.SKIPPED);
  }
  protected boolean validateAppointmentReference(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.APPOINTMENT_REFERENCE;
    String value = transaction.getAppointmentReference();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,15);
  }
  
  protected boolean validateAverageDailyLoad() {
	  return validateAverageDailyLoad(ValidationSetting.SKIPPED);
  }
  protected boolean validateAverageDailyLoad(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.AVERAGE_DAILY_LOAD;
    String value = transaction.getAverageDailyLoad();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return (validator.validateNumeric(events,category,fieldname,transaction.getAverageDailyLoad())
    		&& validator.validateLengthRange(events,category,fieldname,transaction.getAverageDailyLoad(),0,10));
  }
  
  protected boolean validateConfirmDeenergisation() {
	  return validateConfirmDeenergisation(ValidationSetting.SKIPPED);
  }
  protected boolean validateConfirmDeenergisation(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CONFIRM_DEENERGISATION;
    String value = transaction.getConfirmDeenergisation();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateTrueFalse(events,category,fieldname,value);
  }
  
  protected boolean validateContactName() {
	  return validateContactName(ValidationSetting.SKIPPED);
  }
  protected boolean validateContactName(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
	
    if (transaction.getCustomerConsultationRequired().equals(TrueFalse.TRUE.getCode())){
      setting = ValidationSetting.MANDATORY;
    }

	//Either provide ContactName or BusinessName (with optional bizContactName)
    if (validator.exists(transaction.getCustomerContactName()) && (validator.exists(transaction.getBusinessName()) || validator.exists(transaction.getBusinessContactName()))){
      // show an error since you can only have 1 or the other
      validator.addConcurrencyError(events,category,ServiceOrderRequest.CUSTOMER_CONTACT_NAME,ServiceOrderRequest.BUSINESS_NAME);
      return false;
    }
	
	if (setting == ValidationSetting.MANDATORY) {
		//Must have at least one
		if (!validator.exists(transaction.getCustomerContactName()) && !validator.exists(transaction.getBusinessName())){
			String[] fieldnames = new String[] {ServiceOrderRequest.CUSTOMER_CONTACT_NAME, ServiceOrderRequest.BUSINESS_NAME};
		  try {
			String choices = "[";
			for (int i = 0; i < fieldnames.length; i++) {
			  choices += ScreenName.getScreenName(category, fieldnames[i]);
			  if (i < fieldnames.length - 1)
				choices += ", ";
			}
			choices += "]";
			validator.addEvent(events, category, fieldnames[0], new String[]{choices}, EventCode.CODE_0201, "invalidChoiceMandatory");
		  } catch (UnableToLookupResourceException e)
		  {
			validator.unexpectedException(events, fieldnames[0], e.getMessage());
		  }
		  return false;
		}
	}
	
    String fieldname = ServiceOrderRequest.CUSTOMER_CONTACT_NAME;
    PersonName contactName = transaction.getCustomerContactName();
	
	//If no contactName
	if (!validator.exists(contactName)) {
		return validateBusinessName(ValidationSetting.OPTIONAL)
			& validateBusinessContactName(ValidationSetting.OPTIONAL);
	}
	
    contactName.setEvents(events);
	contactName.setParentCategory(category);
  	contactName.setParentField(fieldname);
	return contactName.validate();
  }
  
  private boolean validateContactPhone(ValidationSetting setting, String fieldname, Telephone contactPhone) {
    if (!checkMandatory(setting, fieldname, contactPhone)) return false;
    if (!validator.exists(contactPhone)) return true;
   
    // if phone exists then either contact name or business name must exist
    // ( business name must exist in customer details and is validated by XML schema)
    boolean contactNameExists=validator.exists(transaction.getCustomerContactName());
    boolean businesNameExists=validator.exists(transaction.getBusinessName());
    if ( !(contactNameExists || businesNameExists) ){
      //generate error
      return validator.validateOneWayDependancy(events
                                               ,category
                                               ,fieldname
                                               ,ServiceOrderRequest.CUSTOMER_CONTACT_NAME
                                               ,"1"  // fake values to cause an error to be generated
                                               ,null);
     }
    contactPhone.setEvents(events);
    contactPhone.setParentCategory(category);
    contactPhone.setParentField(fieldname);
    return contactPhone.validate();
  }
  
  protected boolean validateContactPhone1() {
	  return validateContactPhone1(ValidationSetting.SKIPPED);
  }
  protected boolean validateContactPhone1(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CONTACT_PHONENUMBER1;
    Telephone contactPhone = transaction.getContactPhone1();
    if (transaction.getCustomerConsultationRequired().equals(TrueFalse.TRUE.getCode())){
      setting = ValidationSetting.MANDATORY;
    }
    return validateContactPhone(setting, fieldname, contactPhone);
  }

  protected boolean validateContactPhone2() {
	  return validateContactPhone2(ValidationSetting.SKIPPED);
  }
  protected boolean validateContactPhone2(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CONTACT_PHONENUMBER2;
    Telephone contactPhone = transaction.getContactPhone2();
    return validateContactPhone(setting, fieldname, contactPhone);
  }
  
  protected boolean validateContactPhone3() {
	  return validateContactPhone3(ValidationSetting.SKIPPED);
  }
  protected boolean validateContactPhone3(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CONTACT_PHONENUMBER3;
    Telephone contactPhone = transaction.getContactPhone3();
    return validateContactPhone(setting, fieldname, contactPhone);
  }

  protected boolean validateCustomerConsultationRequired() {
	  return validateCustomerConsultationRequired(ValidationSetting.SKIPPED);
  }
  protected boolean validateCustomerConsultationRequired(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CUST_CONSULTATION_REQUIRED;
    String value = transaction.getCustomerConsultationRequired();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    boolean tmpresult = validator.validateTrueFalse(events,category,fieldname,transaction.getCustomerConsultationRequired());
    // if Customer Consultation is TRUE then special instruction is mandatory
    if (tmpresult && transaction.getCustomerConsultationRequired().equals(TrueFalse.TRUE.getCode())){
      specialInstructionRequired = true;
    }
    return tmpresult;
  }

  protected boolean validateCustomerType() {
	  return validateCustomerType(ValidationSetting.SKIPPED);
  }
  protected boolean validateCustomerType(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.CUSTOMER_TYPE;
    String value = transaction.getCustomerType();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateCustomerType(events,category,fieldname,transaction.getCustomerType());
  }

  protected boolean validateFormReference() {
	  return validateFormReference(ValidationSetting.SKIPPED);
  }
  protected boolean validateFormReference(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.FORM_REF;
    String value = transaction.getFormReference();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getFormReference(),0,15);
  }

  protected boolean validateFormNumber() {
	  return validateFormNumber(ValidationSetting.SKIPPED);
  }
  protected boolean validateFormNumber(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.FORM_NUM;
    String value = transaction.getFormNumber();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getFormNumber(),0,15);
  }
  
  protected boolean validateEmbeddedNetworkParentName() {
	  return validateEmbeddedNetworkParentName(ValidationSetting.SKIPPED);
  }
  protected boolean validateEmbeddedNetworkParentName(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.EMBEDDED_NET_PARENT;
    String value = transaction.getEmbeddedNetworkParentName();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getEmbeddedNetworkParentName(),0,10);
  }
  
  protected boolean validateHazardDescription1() {
	  return validateHazardDescription1(ValidationSetting.SKIPPED);
  }
  protected boolean validateHazardDescription1(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.HAZARD_DESCRIPTION1;
    String value = transaction.getHazardDescription1();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getHazardDescription1(),0,80);
  }
  protected boolean validateHazardDescription2() {
	  return validateHazardDescription2(ValidationSetting.SKIPPED);
  }
  protected boolean validateHazardDescription2(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.HAZARD_DESCRIPTION2;
    String value = transaction.getHazardDescription2();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getHazardDescription2(),0,80);
  }
  protected boolean validateHazardDescription3() {
	  return validateHazardDescription3(ValidationSetting.SKIPPED);
  }
  protected boolean validateHazardDescription3(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.HAZARD_DESCRIPTION3;
    String value = transaction.getHazardDescription3();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getHazardDescription3(),0,80);
  }
  protected boolean validateInstallationType() {
	  return validateInstallationType(ValidationSetting.SKIPPED);
  }
  protected boolean validateInstallationType(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.INSTALLATION_TYPE;
    String value = transaction.getInstallationType();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateInstallationType(events,category,fieldname,transaction.getInstallationType());
  }
  
  protected boolean validateLicenseeName() {
	  return validateLicenseeName(ValidationSetting.SKIPPED);
  }
  protected boolean validateLicenseeName(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.LICENSEE_NAME;
    PersonName name = transaction.getLicenseeName();
    if (!checkMandatory(setting, fieldname, name)) return false;
    if (!validator.exists(name)) return true;
    name.setEvents(events);
    name.setParentCategory(category);
    name.setParentField(fieldname);
    return name.validate();
  }
  
  protected boolean validateLicenseePhone1() {
	  return validateLicenseePhone1(ValidationSetting.SKIPPED);
  }
  protected boolean validateLicenseePhone1(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.LICENSEE_PHONENUMBER1;
    Telephone phone = transaction.getLicenseePhone1();
    if (!checkMandatory(setting, fieldname, phone)) return false;
    if (!validator.exists(phone)) return true;
    phone.setEvents(events);
    phone.setParentCategory(category);
    phone.setParentField(fieldname);
    return phone.validate();
  }

  protected boolean validateLicenseePhone2() {
	  return validateLicenseePhone2(ValidationSetting.SKIPPED);
  }
  protected boolean validateLicenseePhone2(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.LICENSEE_PHONENUMBER2;
    Telephone phone = transaction.getLicenseePhone2();
    if (!checkMandatory(setting, fieldname, phone)) return false;
    if (!validator.exists(phone)) return true;
    phone.setEvents(events);
    phone.setParentCategory(category);
    phone.setParentField(fieldname);
    return phone.validate();
  }
  
  protected boolean validateLicenseePhone3() {
	  return validateLicenseePhone3(ValidationSetting.SKIPPED);
  }
  protected boolean validateLicenseePhone3(ValidationSetting setting) {
 	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.LICENSEE_PHONENUMBER3;
    Telephone phone = transaction.getLicenseePhone3();
    if (!checkMandatory(setting, fieldname, phone)) return false;
    if (!validator.exists(phone)) return true;
    phone.setEvents(events);
    phone.setParentCategory(category);
    phone.setParentField(fieldname);
    return phone.validate();
  }

  protected boolean validateMaximumDemand() {
	  return validateMaximumDemand(ValidationSetting.SKIPPED);
  }
  protected boolean validateMaximumDemand(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.MAXIMUM_DEMAND;
    String value = transaction.getMaximumDemand();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    boolean tmpresult = validator.validateNumeric(events,category,fieldname,transaction.getMaximumDemand());
    return tmpresult && validator.validateLengthRange(events,category,fieldname,transaction.getMaximumDemand(),0,4);
  }

  protected boolean validateMeteringRequired() {
	  return validateMeteringRequired(ValidationSetting.SKIPPED);
  }
  protected boolean validateMeteringRequired(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.METERING_REQUIRED;
    String value = transaction.getMeteringRequired();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    //boolean tmpresult = validator.validateMeteringRequired(events,category,fieldname,transaction.getMeteringRequired());
    // If Metering required is OTHER Special instruction is mandatory
    if (validator.exists(transaction.getMeteringRequired()) && transaction.getMeteringRequired().equals(MeteringType.OTHER.getCode())){
      specialInstructionRequired = true;
    }
    return validator.validateLengthRange(events,category,fieldname,value,0,240);
  }

  protected boolean validateMeterInstallCode() {
	  return validateMeterInstallCode(ValidationSetting.SKIPPED);
  }
  protected boolean validateMeterInstallCode(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.METER_INSTALL_CODE;
    String value = transaction.getMeterInstallCode();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateMeterInstallCode(events,category,fieldname,transaction.getMeterInstallCode());
  }

  protected boolean validateMeterSerialNumber() {
	  return validateMeterSerialNumber(ValidationSetting.SKIPPED);
  }
  protected boolean validateMeterSerialNumber(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.METER_SERIAL_NUMBER;
    String value = transaction.getMeterSerialNumber();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getMeterSerialNumber(),0,12);
  }

   protected boolean validateMeterSerialNumber1() {
      return validateMeterSerialNumber1(ValidationSetting.SKIPPED);
   }

   protected boolean validateMeterSerialNumber1(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.METER_SERIAL_NUMBER1;
     String value = transaction.getMeterSerialNumber1();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,category,fieldname,transaction.getMeterSerialNumber1(),0,12);
   }

   protected boolean validateMeterSerialNumber2() {
      return validateMeterSerialNumber2(ValidationSetting.SKIPPED);
   }
   protected boolean validateMeterSerialNumber2(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.METER_SERIAL_NUMBER2;
     String value = transaction.getMeterSerialNumber2();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,category,fieldname,transaction.getMeterSerialNumber3(),0,12);
   }

   protected boolean validateMeterSerialNumber3() {
      return validateMeterSerialNumber3(ValidationSetting.SKIPPED);
   }
   protected boolean validateMeterSerialNumber3(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.METER_SERIAL_NUMBER3;
     String value = transaction.getMeterSerialNumber3();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,category,fieldname,transaction.getMeterSerialNumber3(),0,12);
   }

  protected boolean validateNMI() {
	  return validateNMI(ValidationSetting.SKIPPED);
  }
  protected boolean validateNMI(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
	  String fieldname = ServiceOrderRequest.NMI_FIELD;
    String value = transaction.getNMI();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
	return validator.validateNMI(events, category, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
  }
  
  protected boolean validateNMIChecksum() {
	  return validateNMIChecksum(ValidationSetting.SKIPPED);
  }
  protected boolean validateNMIChecksum(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.NMI_CHECKSUM;
    String value = transaction.getNMIChecksum();
    if (!checkMandatory(setting, fieldname, value)) return false;
    // if both fields don't exist return true
    if (!validator.exists(transaction.getNMI()) && !validator.exists(transaction.getNMIChecksum())) return true;
    // if NMI checksum doesn't exist return true
    if (!validator.exists(transaction.getNMIChecksum())) return true;
    // Checksum can only exist if a NMI exists
    if (!validator.validateOneWayDependancy(events,category,fieldname,ServiceOrderRequest.NMI_FIELD,transaction.getNMI(),transaction.getNMIChecksum())) return false;
    return validator.validateNMIChecksum(events,category,fieldname,transaction.getNMI(),transaction.getNMIChecksum());
  }
  
  protected boolean validateNMIStatusCode() {
	  return validateNMIStatusCode(ValidationSetting.SKIPPED);
  }
  protected boolean validateNMIStatusCode(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.NMI_STATUS_CODE;
    String value = transaction.getNMIStatusCode();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateNMIStatusCode(events,category,fieldname,transaction.getNMIStatusCode());
  }

  protected boolean validateOffPeakRequirement1() {
	  return validateOffPeakRequirement1(ValidationSetting.SKIPPED);
  }
  protected boolean validateOffPeakRequirement1(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.OFF_PEAK_REQUIREMENT1;
    String value = transaction.getOffPeakRequirement1();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,80);
  }

  protected boolean validateOffPeakRequirement2() {
	  return validateOffPeakRequirement2(ValidationSetting.SKIPPED);
  }
  protected boolean validateOffPeakRequirement2(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.OFF_PEAK_REQUIREMENT2;
    String value = transaction.getOffPeakRequirement2();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,80);
  }  

  protected boolean validateOffPeakRequirement3() {
	  return validateOffPeakRequirement3(ValidationSetting.SKIPPED);
  }
  protected boolean validateOffPeakRequirement3(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.OFF_PEAK_REQUIREMENT3;
    String value = transaction.getOffPeakRequirement3();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,80);
  }

  protected boolean validatePreferredDate() {
	  return validatePreferredDate(ValidationSetting.SKIPPED);
  }
  protected boolean validatePreferredDate(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.PREFERRED_DATE;
    
    if (transaction.getIsPreferredDateSet()) {
      String value = transaction.getPreferredDate();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateXMLDate(events,category,fieldname,transaction.getPreferredDate());
    } else {
      String value = transaction.getPreferredDisplayDate();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateB2BDate(events,category,fieldname,transaction.getPreferredDisplayDate());
    }

  }

  protected boolean validatePreferredTime() {
	  return validatePreferredTime(ValidationSetting.SKIPPED);
  }
  protected boolean validatePreferredTime(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.PREFERRED_TIME;
    
    // Bob Jones 29/03/2006 3701: Preferred Time is dependent on Preferred Date being entered
    if (!transaction.getPreferredTime().equalsIgnoreCase("") && transaction.getPreferredDate().equalsIgnoreCase("")) {
      if (!validator.validateOneWayDependancy(events,category,fieldname,ServiceOrderRequest.PREFERRED_DATE,transaction.getPreferredTime(),transaction.getPreferredDate())) {
        return false;
      }
    }
    
    if (transaction.getIsPreferredTimeSet()) {
      String value = transaction.getPreferredTime();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateXMLTime(events,category,fieldname,value);
    } else {
      String value = transaction.getPreferredDisplayTime();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateB2BTime(events,category,fieldname,value);
    }
  }

  protected boolean validateProposedTariff() {
	  return validateProposedTariff(ValidationSetting.SKIPPED);
  }
  protected boolean validateProposedTariff(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.PROPOSED_TARIFF;
    String value = transaction.getProposedTariff();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,10);
  }

   protected boolean validateProposedTariff1() {
      return validateProposedTariff1(ValidationSetting.SKIPPED);
   }
   protected boolean validateProposedTariff1(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.PROPOSED_TARIFF1;
     String value = transaction.getProposedTariff1();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,category,fieldname,value,0,10);
   }

   protected boolean validateProposedTariff2() {
      return validateProposedTariff2(ValidationSetting.SKIPPED);
   }
   protected boolean validateProposedTariff2(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.PROPOSED_TARIFF2;
     String value = transaction.getProposedTariff2();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,category,fieldname,value,0,10);
   }

   protected boolean validateProposedTariff3() {
      return validateProposedTariff3(ValidationSetting.SKIPPED);
   }

   protected boolean validateProposedTariff3(ValidationSetting setting) {
      if (setting == ValidationSetting.SKIPPED)
         return true;
      String fieldname = ServiceOrderRequest.PROPOSED_TARIFF3;
      String value = transaction.getProposedTariff3();
      if (!checkMandatory(setting, fieldname, value))
         return false;
      if (!validator.exists(value))
         return true;
      return validator.validateLengthRange(events, category, fieldname, value, 0, 10);
   }

  protected boolean validateRECAttendanceRequired() {
	  return validateRECAttendanceRequired(ValidationSetting.SKIPPED);
  }
  protected boolean validateRECAttendanceRequired(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.REC_ATTENDANCE_REQUIRED;
    String value = transaction.getRecAttendanceRequired();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateTrueFalse(events,category,fieldname,value);    
  }
  
  protected boolean validateRecId() {
	  return validateRecId(ValidationSetting.SKIPPED);
  }
  protected boolean validateRecId(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.REC_ID;
    String value = transaction.getRecId();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,value,0,20);
  }
  
  protected boolean validateRetailerName() {
	  return validateRetailerName(ValidationSetting.SKIPPED);
  }
  protected boolean validateRetailerName(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.RETAILER_NAME;
    PersonName retailerName = transaction.getRetailerName();
    if (!checkMandatory(setting, fieldname, retailerName)) return false;
    if (!validator.exists(retailerName)) return true;

    retailerName.setEvents(events);
    retailerName.setParentCategory(category);
    retailerName.setParentField(fieldname);
    return retailerName.validate();
  }
  
  private boolean validateRetailerPhone(ValidationSetting setting, String fieldname, Telephone phone) {
  	if (!checkMandatory(setting, fieldname, phone)) return false;
  	if (!validator.exists(phone)) return true;
  
  	// if phone exists then RetailerName must also exist
  	if (!validator.exists(transaction.getRetailerName())){
  	  return validator.validateOneWayDependancy(events
  	                                   ,category
  	                                   ,fieldname
  	                                   ,ServiceOrderRequest.RETAILER_NAME
  	                                   ,"1"  // fake values to cause an error to be generated
  	                                   ,null);
  	}
  	phone.setEvents(events);
  	phone.setParentCategory(category);
  	phone.setParentField(fieldname);
  	return phone.validate();
  }
  protected boolean validateRetailerPhone1() {
	  return validateRetailerPhone1(ValidationSetting.SKIPPED);
  }
  protected boolean validateRetailerPhone1(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.RETAILER_PHONENUMBER1;
    Telephone phone = transaction.getRetailerPhone1();
    if (validator.exists(transaction.getRetailerName())){
      setting = ValidationSetting.MANDATORY;
    }
    return validateRetailerPhone(setting, fieldname, phone);
  }
  
  protected boolean validateRetailerPhone2() {
	  return validateRetailerPhone2(ValidationSetting.SKIPPED);
  }
  protected boolean validateRetailerPhone2(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.RETAILER_PHONENUMBER2;
    Telephone phone = transaction.getRetailerPhone2();
    return validateRetailerPhone(setting, fieldname, phone);
  }
  
  protected boolean validateRetailerPhone3() {
	  return validateRetailerPhone3(ValidationSetting.SKIPPED);
  }
  protected boolean validateRetailerPhone3(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.RETAILER_PHONENUMBER3;
    Telephone phone = transaction.getRetailerPhone3();
    return validateRetailerPhone(setting, fieldname, phone);
  }
  
  protected boolean validateRetServiceOrder() {
	  return validateRetServiceOrder(ValidationSetting.SKIPPED);
  }
  protected boolean validateRetServiceOrder(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.RET_SERVICE_ORDER;
    String value = transaction.getRetServiceOrder();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getRetServiceOrder(),1,15);
  }

  protected boolean validateSafetyCertificateId() {
	  return validateSafetyCertificateId(ValidationSetting.SKIPPED);
  }
  protected boolean validateSafetyCertificateId(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SAFETY_CERT_ID;
    String value = transaction.getSafetyCertificateId();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,category,fieldname,transaction.getSafetyCertificateId(),0,15);
  }

  protected boolean validateSafetyCertificateIdMethodSent() {
	  return validateSafetyCertificateIdMethodSent(ValidationSetting.SKIPPED);
  }
  protected boolean validateSafetyCertificateIdMethodSent(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SAFETY_CERT_METHOD_SENT;
    String value = transaction.getSafetyCertificateIdMethodSent();
    
    //If Safety Certificate ID exists then this is mandatory. (Required due to Schema enforcing this).
    if (validator.exists(transaction.getSafetyCertificateId())){
      setting = ValidationSetting.MANDATORY;
    }
    
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateSafetyCertMethodSent(events,
                                                  category,
                                                  fieldname,
                                                  transaction.getSafetyCertificateIdMethodSent());
  }

  protected boolean validateScheduleDate() {
	  return validateScheduleDate(ValidationSetting.SKIPPED);
  }
  protected boolean validateScheduleDate(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SCHEDULE_DATE;
    
    if (transaction.getIsScheduleDateSet()) {
      String value = transaction.getScheduleDate();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateScheduleDateRange(events
                                                 ,category
                                                 ,fieldname
                                                 ,transaction.getScheduleDate()
                                                 ,transaction.getTransactionDate()
                                                 ,B2BDateFormatter.XMLDATE);
    } else {
      String value = transaction.getScheduleDisplayDate();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateScheduleDateRange(events
                                                 ,category
                                                 ,fieldname
                                                 ,transaction.getScheduleDisplayDate()
                                                 ,transaction.getTransactionDisplayDate()
                                                 ,B2BDateFormatter.B2BDATE);
    }

  }

  protected boolean validateServiceOrderAddress() {
	  return validateServiceOrderAddress(ValidationSetting.SKIPPED);
  }
  protected boolean validateServiceOrderAddress(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SERVICE_ORDER_ADDRESS;
    Address address = transaction.getServiceOrderAddress();
	
    if (!checkMandatory(setting, fieldname, address)) return false;
    if (!validator.exists(address)) return true;
	
	address.setEvents(events);
	address.setParentCategory(category);
	address.setParentField(fieldname);
	return address.validate();
  }

  protected boolean validateRequestData() {
    return validateRequestData(ValidationSetting.SKIPPED);
  }
  // (Justina Tran) defect 3513: added method.
  protected boolean validateRequestData(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.REQUEST_DATA;
    RequestData requestData = transaction.getSimpleRequestData();

    if (requestData != null) {
	    requestData.setEvents(events);
	    requestData.setParentCategory(category);
	    requestData.setParentField(fieldname);
	    return requestData.validate();
    }
    // add an event if null - mandatory check
    if (setting == ValidationSetting.MANDATORY) {
      Object[] args = new Object[1];
      args[0]="RequestData";
      try {
        validator.addEvent(events,category,fieldname,args,EventCode.CODE_0201,"missingValue");
      } catch (Exception e){
        validator.unexpectedException(events,fieldname,"Unexpected Error on Missing Element");
      }
      return false;
    }
    return true; //optional
  }

  protected boolean validateServiceOrderSubType() {
    return validateServiceOrderSubType(ValidationSetting.SKIPPED);
  }
  protected boolean validateServiceOrderSubType(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SERVICE_ORDER_SUB_TYPE;
    String value = transaction.getServiceOrderSubType();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateServiceOrderSubType(events,
                                                  category,
                                                  fieldname,
                                                  transaction.getServiceOrderType(), 
                                                  transaction.getServiceOrderSubType());
  }
  
  protected boolean validateServiceOrderType() {
    return validateServiceOrderType(ValidationSetting.SKIPPED);
  }
  protected boolean validateServiceOrderType(ValidationSetting setting) {
	  if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SERVICE_ORDER_TYPE;
    String value = transaction.getServiceOrderType();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateServiceOrderType(events,
                                              category,
                                              fieldname,
                                              transaction.getServiceOrderType(), 
                                              transaction.getTransactionType().getDescription());
  }

  protected boolean validateServiceTime() {
    return validateServiceTime(ValidationSetting.SKIPPED);
  }
   protected boolean validateServiceTime(ValidationSetting setting) {
	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.SERVICE_TIME;
     String value = transaction.getServiceTime();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     // TG 27MAY2009 - for non business hours service time, make the special instructions required mandatory.
     if ( transaction.getServiceTime().equals( ServiceTime.NON_BUSINESS_HOURS.getCode() ) ) {
        specialInstructionRequired = true;
     }
     return validator.validateServiceTime( events, category, fieldname, transaction.getServiceTime() );
   }

   protected boolean validateSpecialInstructions1() {
     return validateSpecialInstructions1(ValidationSetting.SKIPPED);
   }
   protected boolean validateSpecialInstructions1(ValidationSetting setting) {
	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS1;
     if (specialInstructionRequired) setting = ValidationSetting.MANDATORY;
     String value = transaction.getSpecialInstructions1();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(
                                         events,
                                         category,
                                         fieldname,
                                         transaction.getSpecialInstructions1(),
                                         0,80);
  }
   protected boolean validateSpecialInstructions2() {
     return validateSpecialInstructions2(ValidationSetting.SKIPPED);
   }
  protected boolean validateSpecialInstructions2(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS2;
    String value = transaction.getSpecialInstructions2();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,
                                         category,
                                         fieldname,
                                         transaction.getSpecialInstructions2(),
                                         0,80);
  }
  protected boolean validateSpecialInstructions3() {
    return validateSpecialInstructions3(ValidationSetting.SKIPPED);
  }
  protected boolean validateSpecialInstructions3(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS3;
    String value = transaction.getSpecialInstructions3();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    return validator.validateLengthRange(events,
                                         category,
                                         fieldname,
                                         transaction.getSpecialInstructions3(),
                                         0,80);
  }
  
  protected boolean validateSupplyPhases() {
    return validateSupplyPhases(ValidationSetting.SKIPPED);
  }
  protected boolean validateSupplyPhases(ValidationSetting setting) {
    if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SUPPLY_PHASE;
    String value = transaction.getSupplyPhase();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
    boolean tmpresult = validator.validateSupplyPhase(events,category,fieldname,transaction.getSupplyPhase());
    // if Supply Phase is Multi Phase
    if (transaction.getSupplyPhase() != null && transaction.getSupplyPhase().equals(SupplyPhase.OTHER_MULTIPHASE.getCode())){
      specialInstructionRequired = true;
    }
    return tmpresult;
  }
  
  protected boolean validateSwitchingServiceRequired() {
    return validateSwitchingServiceRequired(ValidationSetting.SKIPPED);
  }
  protected boolean validateSwitchingServiceRequired(ValidationSetting setting) {
	if (setting == ValidationSetting.SKIPPED) return true;
    String fieldname = ServiceOrderRequest.SWITCHING_SERVICE_REQUIRED;
    String value = transaction.getSwitchingServiceRequired();
    if (!checkMandatory(setting, fieldname, value)) return false;
    if (!validator.exists(value)) return true;
	  return validator.validateLengthRange(events,
                                         category,
                                         fieldname,
                                         transaction.getSwitchingServiceRequired(),
                                         0,80);									   
  }


  protected boolean validateBusinessContactName() {
	  return validateBusinessContactName(ValidationSetting.SKIPPED);
  }
   protected boolean validateBusinessContactName(ValidationSetting setting) {
	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.BUSINESS_CONTACT_NAME;
     PersonName businessContactName = transaction.getBusinessContactName();
     if (!checkMandatory(setting, fieldname, businessContactName)) return false;
     if (!validator.exists(businessContactName)) return true;
     businessContactName.setEvents(events);
     businessContactName.setParentCategory(category);
     businessContactName.setParentField(fieldname);
     return businessContactName.validate();
   }

   protected boolean validateBusinessName() {
		  return validateBusinessName(ValidationSetting.SKIPPED);
	  }
   protected boolean validateBusinessName(ValidationSetting setting) {
	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.BUSINESS_NAME;
     String value = transaction.getBusinessName();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,
                                          category,
                                          fieldname,
                                          transaction.getBusinessName(),
                                          0,200);
   }
   // new business name section
   protected boolean validateCoordinatingBusinessContactName() {
           return validateCoordinatingBusinessContactName(ValidationSetting.SKIPPED);
   }
    protected boolean validateCoordinatingBusinessContactName(ValidationSetting setting) {
          if (setting == ValidationSetting.SKIPPED) return true;
      String fieldname = ServiceOrderRequest.COORDINATION_BUSINESS_CONTACT_NAME;
      PersonName coordinatingBusinessContactName = transaction.getCoordinatingBusinessContactName();
      if (!checkMandatory(setting, fieldname, coordinatingBusinessContactName)) return false;
      if (!validator.exists(coordinatingBusinessContactName)) return true;
      coordinatingBusinessContactName.setEvents(events);
      coordinatingBusinessContactName.setParentCategory(category);
      coordinatingBusinessContactName.setParentField(fieldname);
      return coordinatingBusinessContactName.validate();
    }

    protected boolean validateCoordinatingBusinessName() {
                   return validateCoordinatingBusinessName(ValidationSetting.SKIPPED);
           }
    protected boolean validateCoordinatingBusinessName(ValidationSetting setting) {
          if (setting == ValidationSetting.SKIPPED) return true;
      String fieldname = ServiceOrderRequest.BUSINESS_NAME;
      String value = transaction.getCoordinatingBusinessName();
      if (!checkMandatory(setting, fieldname, value)) return false;
      if (!validator.exists(value)) return true;
      return validator.validateLengthRange(events,
                                           category,
                                           fieldname,
                                           transaction.getCoordinatingBusinessName(),
                                           0,200);
    }
   protected boolean validateLifeSupport() {
     return validateLifeSupport(ValidationSetting.SKIPPED);
   }
   protected boolean validateLifeSupport(ValidationSetting setting) {
		if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.LIFE_SUPPORT;
     String value = transaction.getLifeSupport();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateYesNo(events,category,fieldname,transaction.getLifeSupport());
   }
   
   protected boolean validateNotifyParty() {
     return validateNotifyParty(ValidationSetting.SKIPPED);
   }
   protected boolean validateNotifyParty(ValidationSetting setting) {
	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.NOTIFY_PARTY1;
     String value = transaction.getNotifiedPartyId1();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     boolean tmpresult = validator.validateLengthRange(events,
                                          category,
                                          fieldname,
                                          transaction.getNotifiedPartyId1(),
                                          0,10);
     fieldname = ServiceOrderRequest.NOTIFY_PARTY2;
     tmpresult = tmpresult && validator.validateLengthRange(events,
                                          category,
                                          fieldname,
                                          transaction.getNotifiedPartyId2(),
                                          0,10);
     fieldname = ServiceOrderRequest.NOTIFY_PARTY3;
     tmpresult = tmpresult && validator.validateLengthRange(events,
                                          category,
                                          fieldname,
                                          transaction.getNotifiedPartyId3(),
                                          0,10);
	 return tmpresult;
   }

   protected boolean validateMeteringSafetyCertificateId() {
     return validateMeteringSafetyCertificateId(ValidationSetting.SKIPPED);
   }
   protected boolean validateMeteringSafetyCertificateId(ValidationSetting setting) {
 	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.METERING_SAFETY_CERTIFICATE_ID;
     String value = transaction.getMeteringSafetyCertificateId();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateLengthRange(events,
                                          category,
                                          fieldname,
                                          value,
                                          0,15);
   }

   protected boolean validateMeteringSafetyCertificateMethodSent() {
     return validateMeteringSafetyCertificateMethodSent(ValidationSetting.SKIPPED);
   }
   protected boolean validateMeteringSafetyCertificateMethodSent(ValidationSetting setting) {
  	 if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.METERING_SAFETY_CERT_METHOD_SENT;
     String value = transaction.getMeteringSafetyCertificateIdMethodSent();
	 
    // if Metering Safety Certificate ID exists then this is mandatory. (Required due to Schema enforcing this)
    if (validator.exists(transaction.getMeteringSafetyCertificateId())){
      setting = ValidationSetting.MANDATORY;
    }
	 
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateMeteringSafetyCertMethodSent(events,
                                                   category,
                                                   fieldname,
                                                   transaction.getMeteringSafetyCertificateIdMethodSent());
   }
   
   protected boolean validateCoOrdinationRequired() {
     return validateCoOrdinationRequired(ValidationSetting.SKIPPED);
   }
   protected boolean validateCoOrdinationRequired(ValidationSetting setting) {
	   if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.COORDINATION_REQUIRED;
     String value = transaction.getCoOrdinationRequired();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateYesNo(events,category,fieldname,value);
   }
   
   protected boolean validateCoOrdinatingContactName() {
     return validateCoOrdinatingContactName(ValidationSetting.SKIPPED);
   }
   protected boolean validateCoOrdinatingContactName(ValidationSetting setting) {
	   if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.COORDINATION_CONTACT_NAME;
     PersonName contactName = transaction.getCoOrdinatingContactName();
     if (transaction.getCoOrdinationRequired().equals(YesNo.YES.getCode())){
    	 setting = ValidationSetting.MANDATORY;
     }
     
       //Either provide ContactName or BusinessName (with optional bizContactName)
           if (validator.exists(transaction.getCoOrdinatingContactName()) && (validator.exists(transaction.getCoordinatingBusinessContactName()) || validator.exists(transaction.getCoordinatingBusinessName()))){
             // show an error since you can only have 1 or the other
             validator.addConcurrencyError(events,category,ServiceOrderRequest.COORDINATION_CONTACT_NAME,ServiceOrderRequest.COORDINATION_BUSINESS_NAME);
             return false;
           }
               
               if (setting == ValidationSetting.MANDATORY) {
                       //Must have at least one
                       if (!validator.exists(transaction.getCoOrdinatingContactName()) && !validator.exists(transaction.getCoordinatingBusinessName())){
                               String[] fieldnames = new String[] {ServiceOrderRequest.COORDINATION_CONTACT_NAME, ServiceOrderRequest.COORDINATION_BUSINESS_NAME};
                         try {
                               String choices = "[";
                               for (int i = 0; i < fieldnames.length; i++) {
                                 choices += ScreenName.getScreenName(category, fieldnames[i]);
                                 if (i < fieldnames.length - 1)
                                       choices += ", ";
                               }
                               choices += "]";
                               validator.addEvent(events, category, fieldnames[0], new String[]{choices}, EventCode.CODE_0201, "invalidChoiceMandatory");
                         } catch (UnableToLookupResourceException e)
                         {
                               validator.unexpectedException(events, fieldnames[0], e.getMessage());
                         }
                         return false;
                       }
               }
     
     
     
             
    //If no contactName
     if (!validator.exists(contactName)) {
          return validateCoordinatingBusinessName(ValidationSetting.OPTIONAL)
                 & validateCoordinatingBusinessContactName(ValidationSetting.OPTIONAL);
    }
    
     contactName.setEvents(events);
     contactName.setParentCategory(category);
     contactName.setParentField(fieldname);
     return contactName.validate();
   }
   
   private boolean validatePhoneNumber(ValidationSetting setting, String fieldName,Telephone contactPhone) {
     if (!checkMandatory(setting, fieldName, contactPhone)) return false;
     if (!validator.exists(contactPhone)) return true;
     
 	   // if phone exists then the contact name must exist
     
     
 	   boolean contactNameExists=validator.exists(transaction.getCoOrdinatingContactName());
           boolean businesNameExists=validator.exists(transaction.getCoordinatingBusinessName());
 	       if ( !(contactNameExists || businesNameExists) ){
 	         //generate error
 	         return validator.validateOneWayDependancy(events
 	                                                  ,category
 	                                                  ,fieldName
 	                                                  ,ServiceOrderRequest.COORDINATION_CONTACT_NAME
 	                                                  ,"1"  // fake values to cause an error to be generated
 	                                                  ,null);
 	        }
// 	   if (!validator.exists(contactName)){
//  	     //generate error
//  	     return validator.validateOneWayDependancy(events
//  	                                                 ,category
//  	                                                 ,phoneFieldname
//  	                                                 ,contactFieldName
//  	                                                 ,"1"  // fake values to cause an error to be generated
//  	                                                 ,null);
//	   }
	   contactPhone.setEvents(events);
	   contactPhone.setParentCategory(category);
	   contactPhone.setParentField(fieldName);
	   return contactPhone.validate();
   }
   
   
   
   protected boolean validateCoOrdinatingContactPhone1() {
     return validateCoOrdinatingContactPhone1(ValidationSetting.SKIPPED);
   }
   protected boolean validateCoOrdinatingContactPhone1(ValidationSetting setting) {
  	 if (setting == ValidationSetting.SKIPPED) return true;
     if (transaction.getCoOrdinationRequired().equals(YesNo.YES.getCode())){
       setting = ValidationSetting.MANDATORY;
     }
  	 Telephone contactPhone = transaction.getCoOrdinatingContactPhone1();
  	 PersonName contactName = transaction.getCoOrdinatingContactName();
         PersonName businessContactName = transaction.getCoordinatingBusinessContactName();
         String businessName = transaction.getCoordinatingBusinessName();
  	 String phoneFieldName = ServiceOrderRequest.COORDINATION_CONTACT_PHONENUMBER1;
  	 String contactFieldName = ServiceOrderRequest.COORDINATION_CONTACT_NAME;
       
       String contactFieldBusiness = ServiceOrderRequest.COORDINATION_BUSINESS_CONTACT_NAME;
       String contactFieldBusinessName = ServiceOrderRequest.COORDINATION_BUSINESS_NAME;
//       if(!(validatePhoneNumber(setting, contactPhone, contactName, phoneFieldName, contactFieldName,null)||validatePhoneNumber(setting, contactPhone, phoneFieldName, contactFieldBusinessName,businessName))){
//           return false;
//       }
      
       return validatePhoneNumber(setting, phoneFieldName, contactPhone);      

   }
   protected boolean validateCoOrdinatingContactPhone2() {
     return validateCoOrdinatingContactPhone2(ValidationSetting.SKIPPED);
   }
   protected boolean validateCoOrdinatingContactPhone2(ValidationSetting setting) {
  	 if (setting == ValidationSetting.SKIPPED) return true;
  	 Telephone contactPhone = transaction.getCoOrdinatingContactPhone2();
  	 PersonName contactName = transaction.getCoOrdinatingContactName();
  	 String phoneFieldName = ServiceOrderRequest.COORDINATION_CONTACT_PHONENUMBER2;
  	 String contactFieldName = ServiceOrderRequest.COORDINATION_CONTACT_NAME;
       return validatePhoneNumber(setting, phoneFieldName, contactPhone);   
   }
   protected boolean validateCoOrdinatingContactPhone3() {
     return validateCoOrdinatingContactPhone3(ValidationSetting.SKIPPED);
   }
   protected boolean validateCoOrdinatingContactPhone3(ValidationSetting setting) {
  	 if (setting == ValidationSetting.SKIPPED) return true;
  	 Telephone contactPhone = transaction.getCoOrdinatingContactPhone3();
  	 PersonName contactName = transaction.getCoOrdinatingContactName();
  	 String phoneFieldName = ServiceOrderRequest.COORDINATION_CONTACT_PHONENUMBER3;
  	 String contactFieldName = ServiceOrderRequest.COORDINATION_CONTACT_NAME;
       return validatePhoneNumber(setting, phoneFieldName, contactPhone);   
   }
   
   protected boolean validateDeenergisationReason() {
     return validateDeenergisationReason(ValidationSetting.SKIPPED);
   }
   protected boolean validateDeenergisationReason(ValidationSetting setting) {
     if (setting == ValidationSetting.SKIPPED) return true;
     String fieldname = ServiceOrderRequest.DEENERGISATION_REASON;
     String value = transaction.getDeEnergisationReason();
     if (!checkMandatory(setting, fieldname, value)) return false;
     if (!validator.exists(value)) return true;
     return validator.validateDeEnergisationReason(events,category,fieldname,value);
   }
   
   
    
}

