/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/SiteAccessNotificationBasicValidator.java-arc   1.10   11 Apr 2005 11:11:10   RayanS  $
 * $Revision:   1.10  $
 * $Date:   11 Apr 2005 11:11:10  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.SiteAccessNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;

/**
 * The class <code>SiteAccessNotificationBasicValidator</code> is the Basic Validator 
 * object of the Transaction.
 * 
 * @author $Author:   RayanS  $
 * @version $Revision:   1.10  $ $Date:   11 Apr 2005 11:11:10  $
 */
public class SiteAccessNotificationBasicValidator extends Validator {
  protected SiteAccessNotification transaction;
  
  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * @param transaction Transaction to use in the validator.
   */
  public SiteAccessNotificationBasicValidator(Transaction transaction) {
    super(transaction);
    this.transaction = (SiteAccessNotification) transaction;
  }

  /**
   * Validate the Transaction
   * 
   * @return boolean result of validation
   * @throws TransactionNotSetException is a transaction was not set before 
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID() 
        & validateTransactionDate()
        & validateNMI()
        & validateNMIChecksum()
        & validateAccessDetails()
        & validateHazardDescription1()
        & validateHazardDescription2()
        & validateHazardDescription3()
        & validateLastModifiedDateTime();

    return result;
  }

  protected boolean validateAccessDetails() {
    String field = SiteAccessNotification.ACCESS_DETAILS;
    return validator.validateMandatory(events, category, field, transaction.getAccessDetails())
        && validator.validateLengthRange(events, category, field, transaction.getAccessDetails(), 1, 160);
  }

  protected boolean validateHazardDescription1() {
    String field = SiteAccessNotification.HAZARD_DESCRIPTION1;
    return validator.validateMandatory(events, category, field, transaction.getHazardDescription1()) 
        && validator.validateLengthRange(events, category, field, transaction.getHazardDescription1(), 0, 80);
  }

  protected boolean validateHazardDescription2() {
    String field = SiteAccessNotification.HAZARD_DESCRIPTION2;
    return validator.validateLengthRange(events, category, field, transaction.getHazardDescription2(), 0, 80);
  }

  protected boolean validateHazardDescription3() {
    String field = SiteAccessNotification.HAZARD_DESCRIPTION3;
    return validator.validateLengthRange(events, category, field, transaction.getHazardDescription3(), 0, 80);
  }

  protected boolean validateLastModifiedDateTime() {
    String field = SiteAccessNotification.LAST_MODIFIED_DATE_TIME;
    
    if (transaction.isLastModifiedDateTimeSet()) {
      return validator.validateMandatory(events, category, field, transaction.getLastModifiedDateTime())
          && validator.validateXMLDateTime(events, category, field, transaction.getLastModifiedDateTime());
    } else {
      return validator.validateMandatory(events, category, field, transaction.getLastModifiedDisplayDateTime())
          && validator.validateB2BDateTime(events, category, field, transaction.getLastModifiedDisplayDateTime());
    }
  }
}

