/*
* SiteAccessRequestBasicValidator.java
*
* The Copyright of this work is vested in the Australian Energy Market
* Operator (AEMO) and the software is issued in confidence for the purpose
* only for which it is supplied. It must not be reproduced in whole or in
* part or used for tendering or manufacturing purposes except under an
* agreement or with the consent in writing of AEMO and then only on the
* condition that this notice is included in any such reproduction.
* No information as to the contents or subject matter of this software or
* any part thereof arising directly or indirectly there from shall be
* given orally or in writing or communicated in any manner whatsoever to
* any third party being an individual firm or company or any employee
* thereof without the prior consent in writing of AEMO.
*
* Copyright Australian Energy Market Operator Limited. 2017
*
* Modification History
*
* Date Changed   Changed By        Description
* ------------   ----------        -----------
* 25-Jun-2017    Harpreet Kaur     Created
* 24-Jul-2017    Harpreet Kaur     d7135 pkg7b : Fix for Defects : 7473, 7478, 7479
*/


package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.SiteAccessRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.Reason;
import com.capgemini.nemmco.b2bb.enums.SiteAccessRequestReasons;

/**
 * The class <code>SiteAccessRequestBasicValidator</code> is the Basic Validator
 * object of the Transaction.
 *
 * @author $Author:   HarpreetK  $
 * @version $Revision:   1.0  $ $Date:    20 Jun 2017 11:11:10  $
 */
public class SiteAccessRequestBasicValidator extends Validator {
   protected SiteAccessRequest transaction;
   /**
    * Constructor. Sets a transaction with the objects initialisation.
    * @param transaction Transaction to use in the validator.
    */
   public SiteAccessRequestBasicValidator(Transaction transaction) {
     super(transaction);
     this.transaction = (SiteAccessRequest) transaction;
   }

   /**
    * Validate the Transaction
    *
    * @return boolean result of validation
    */
   public boolean validate() {
     boolean result = true;

     result &= validateTransactionID()
         & validateTransactionDate()
         & validateNMI()
         & validateNMIChecksum()
         & validateRequestReason()
         & validateSpecialNotes1()
         & validateSpecialNotes2()
         & validateSpecialNotes3();

     return result;
   }
   protected boolean validateRequestReason() {
       String field = SiteAccessRequest.REASON;
       return validator.validateMandatory(events, category, field, transaction.getReason())
          && validator.validateEnumeratedValue(events, category, field, "Reason", SiteAccessRequestReasons.SITE_ACCESS_REQUEST_REASONS, transaction.getReason());
     }

     protected boolean validateSpecialNotes1() {
       boolean tmpResult = true;
       String fieldname = SiteAccessRequest.SPECIAL_NOTES1;
       String value = transaction.getSpecialNotes1();
       String conditionalFieldname = SiteAccessRequest.REASON;
       String conditionalValue = transaction.getReason();

       //Mandatory Special Note if reason is Other
       tmpResult =  validator.validateMandatoryConditionally1(events, category, fieldname, value, conditionalFieldname, conditionalValue, SiteAccessRequestReasons.OTHER.getCode());

       if (validator.exists(value))
           tmpResult = tmpResult && validator.validateSpecialNotes(events,category, fieldname, value)
                       & validator.validateLengthRange(events, category, fieldname, value, 0, 80);
      return tmpResult;
     }

   protected boolean validateSpecialNotes2() {
     boolean tmpResult = true;
     String fieldName = SiteAccessRequest.SPECIAL_NOTES2;
     String specialNotes2 = transaction.getSpecialNotes2();

    if (validator.exists(specialNotes2))
    {
      tmpResult = validator.validateSpecialNotes(events,category, fieldName, specialNotes2)
                      & validator.validateLengthRange(events, category, fieldName, specialNotes2, 0, 80);
    }
      return tmpResult;
   }

   protected boolean validateSpecialNotes3() {
     boolean tmpResult = true;
     String fieldName = SiteAccessRequest.SPECIAL_NOTES3;
     String specialNotes3 = transaction.getSpecialNotes3();

     if (validator.exists(specialNotes3))
      {
       tmpResult = validator.validateSpecialNotes(events,category, fieldName, specialNotes3)
                       & validator.validateLengthRange(events, category, fieldName, specialNotes3, 0, 80);
      }
       return tmpResult;
   }
}
